@@ -2,12 +2,9 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-
-// @dart = 2.12
 part of dart.ui;
 
 @pragma('vm:entry-point')
-// ignore: unused_element
 void _updateWindowMetrics(
   Object id,
   double devicePixelRatio,
@@ -70,8 +67,8 @@ void _updateUserSettingsData(String jsonData) {
 }
 
 @pragma('vm:entry-point')
-void _updateLifecycleState(String state) {
-  PlatformDispatcher.instance._updateLifecycleState(state);
+void _updateInitialLifecycleState(String state) {
+  PlatformDispatcher.instance._updateInitialLifecycleState(state);
 }
 
 @pragma('vm:entry-point')
@@ -95,8 +92,8 @@ void _dispatchPointerDataPacket(ByteData packet) {
 }
 
 @pragma('vm:entry-point')
-void _dispatchSemanticsAction(int id, int action, ByteData? args) {
-  PlatformDispatcher.instance._dispatchSemanticsAction(id, action, args);
+void _dispatchSemanticsAction(int nodeId, int action, ByteData? args) {
+  PlatformDispatcher.instance._dispatchSemanticsAction(nodeId, action, args);
 }
 
 @pragma('vm:entry-point')
@@ -127,11 +124,11 @@ typedef _ListStringArgFunction(List<String> args);
 void _runMain(Function startMainIsolateFunction,
               Function userMainFunction,
               List<String> args) {
-  startMainIsolateFunction(() {
+  startMainIsolateFunction(() { // ignore: avoid_dynamic_calls
     if (userMainFunction is _ListStringArgFunction) {
       userMainFunction(args);
     } else {
-      userMainFunction();
+      userMainFunction(); // ignore: avoid_dynamic_calls
     }
   }, null);
 }
@@ -141,9 +138,6 @@ void _invoke(void Function()? callback, Zone zone) {
   if (callback == null) {
     return;
   }
-
-  assert(zone != null);
-
   if (identical(zone, Zone.current)) {
     callback();
   } else {
@@ -160,9 +154,6 @@ void _invoke1<A>(void Function(A a)? callback, Zone zone, A arg) {
   if (callback == null) {
     return;
   }
-
-  assert(zone != null);
-
   if (identical(zone, Zone.current)) {
     callback(arg);
   } else {
@@ -179,9 +170,6 @@ void _invoke2<A1, A2>(void Function(A1 a1, A2 a2)? callback, Zone zone, A1 arg1,
   if (callback == null) {
     return;
   }
-
-  assert(zone != null);
-
   if (identical(zone, Zone.current)) {
     callback(arg1, arg2);
   } else {
@@ -200,9 +188,6 @@ void _invoke3<A1, A2, A3>(void Function(A1 a1, A2 a2, A3 a3)? callback, Zone zon
   if (callback == null) {
     return;
   }
-
-  assert(zone != null);
-
   if (identical(zone, Zone.current)) {
     callback(arg1, arg2, arg3);
   } else {
