@@ -2,8 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-
-// @dart = 2.12
 part of dart.ui;
 
 /// The possible actions that can be conveyed from the operating system
@@ -15,7 +13,7 @@ part of dart.ui;
 /// See also:
 ///   - file://./../../lib/ui/semantics/semantics_node.h
 class SemanticsAction {
-  const SemanticsAction._(this.index) : assert(index != null);
+  const SemanticsAction._(this.index);
 
   static const int _kTapIndex = 1 << 0;
   static const int _kLongPressIndex = 1 << 1;
@@ -230,6 +228,16 @@ class SemanticsAction {
     _kSetTextIndex: setText,
   };
 
+  /// Temporary API until [values] return a list.
+  /// https://github.com/flutter/flutter/issues/123346
+  @Deprecated('This getter is temporary and will be removed shortly.')
+  static List<SemanticsAction> get doNotUseWillBeDeletedWithoutWarningValuesAsList => values.values.toList(growable: false);
+
+  /// Temporary API until [values] return a list.
+  /// https://github.com/flutter/flutter/issues/123346
+  @Deprecated('This getter is temporary and will be removed shortly.')
+  static Iterable<int> get doNotUseWillBeDeletedWithoutWarningKeys => values.keys;
+
   @override
   String toString() {
     switch (index) {
@@ -287,9 +295,11 @@ class SemanticsAction {
 //
 // When changes are made to this class, the equivalent APIs in
 // `lib/ui/semantics/semantics_node.h` and in each of the embedders *must* be
-// updated.
+// updated. If the change affects the visibility of a [SemanticsNode] to
+// accessibility services, `flutter_test/controller.dart#SemanticsController._importantFlags`
+// must be updated as well.
 class SemanticsFlag {
-  const SemanticsFlag._(this.index) : assert(index != null);
+  const SemanticsFlag._(this.index);
 
   /// The numerical value for this flag.
   ///
@@ -321,11 +331,15 @@ class SemanticsFlag {
   static const int _kIsLinkIndex = 1 << 22;
   static const int _kIsSliderIndex = 1 << 23;
   static const int _kIsKeyboardKeyIndex = 1 << 24;
+  static const int _kIsCheckStateMixedIndex = 1 << 25;
   // READ THIS: if you add a flag here, you MUST update the numSemanticsFlags
   // value in testing/dart/semantics_test.dart, or tests will fail. Also,
   // please update the Flag enum in
   // flutter/shell/platform/android/io/flutter/view/AccessibilityBridge.java,
-  // and the SemanticsFlag class in lib/web_ui/lib/semantics.dart.
+  // and the SemanticsFlag class in lib/web_ui/lib/semantics.dart. If the new flag
+  // affects the visibility of a [SemanticsNode] to accessibility services,
+  // `flutter_test/controller.dart#SemanticsController._importantFlags`
+  // must be updated as well.
 
   /// The semantics node has the quality of either being "checked" or "unchecked".
   ///
@@ -350,6 +364,17 @@ class SemanticsFlag {
   ///   * [SemanticsFlag.hasCheckedState], which enables a checked state.
   static const SemanticsFlag isChecked = SemanticsFlag._(_kIsCheckedIndex);
 
+  /// Whether a tristate checkbox is in its mixed state.
+  ///
+  /// If this is true, the check box this semantics node represents
+  /// is in a mixed state.
+  ///
+  /// For example, a [Checkbox] with [Checkbox.tristate] set to true
+  /// can have checked,  unchecked, or mixed state.
+  ///
+  /// Must be false when the checkbox is either checked or unchecked.
+  static const SemanticsFlag isCheckStateMixed = SemanticsFlag._(_kIsCheckStateMixedIndex);
+
 
   /// Whether a semantics node is selected.
   ///
@@ -580,8 +605,19 @@ class SemanticsFlag {
     _kIsLinkIndex: isLink,
     _kIsSliderIndex: isSlider,
     _kIsKeyboardKeyIndex: isKeyboardKey,
+    _kIsCheckStateMixedIndex: isCheckStateMixed,
   };
 
+  /// Temporary API until [values] return a list.
+  /// https://github.com/flutter/flutter/issues/123346
+  @Deprecated('This getter is temporary and will be removed shortly.')
+  static List<SemanticsFlag> get doNotUseWillBeDeletedWithoutWarningValuesAsList => values.values.toList(growable: false);
+
+  /// Temporary API until [values] return a list.
+  /// https://github.com/flutter/flutter/issues/123346
+  @Deprecated('This getter is temporary and will be removed shortly.')
+  static Iterable<int> get doNotUseWillBeDeletedWithoutWarningKeys => values.keys;
+
   @override
   String toString() {
     switch (index) {
@@ -635,6 +671,8 @@ class SemanticsFlag {
         return 'SemanticsFlag.isSlider';
       case _kIsKeyboardKeyIndex:
         return 'SemanticsFlag.isKeyboardKey';
+      case _kIsCheckStateMixedIndex:
+        return 'SemanticsFlag.isCheckStateMixed';
     }
     assert(false, 'Unhandled index: $index (0x${index.toRadixString(8).padLeft(4, "0")})');
     return '';
@@ -694,11 +732,8 @@ class SpellOutStringAttribute extends StringAttribute {
     _initSpellOutStringAttribute(this, range.start, range.end);
   }
 
-  void _initSpellOutStringAttribute(
-    SpellOutStringAttribute instance,
-    int start,
-    int end,
-  ) native 'NativeStringAttribute_initSpellOutStringAttribute';
+  @Native<Void Function(Handle, Int32, Int32)>(symbol: 'NativeStringAttribute::initSpellOutStringAttribute')
+  external static void _initSpellOutStringAttribute(SpellOutStringAttribute instance, int start, int end);
 
   @override
   StringAttribute copy({required TextRange range}) {
@@ -733,12 +768,8 @@ class LocaleStringAttribute extends StringAttribute {
   /// The lanuage of this attribute.
   final Locale locale;
 
-  void _initLocaleStringAttribute(
-    LocaleStringAttribute instance,
-    int start,
-    int end,
-    String locale,
-  ) native 'NativeStringAttribute_initLocaleStringAttribute';
+  @Native<Void Function(Handle, Int32, Int32, Handle)>(symbol: 'NativeStringAttribute::initLocaleStringAttribute')
+  external static void _initLocaleStringAttribute(LocaleStringAttribute instance, int start, int end, String locale);
 
   @override
   StringAttribute copy({required TextRange range}) {
@@ -761,7 +792,9 @@ class SemanticsUpdateBuilder extends NativeFieldWrapperClass1 {
   /// Creates an empty [SemanticsUpdateBuilder] object.
   @pragma('vm:entry-point')
   SemanticsUpdateBuilder() { _constructor(); }
-  void _constructor() native 'SemanticsUpdateBuilder_constructor';
+
+  @Native<Void Function(Handle)>(symbol: 'SemanticsUpdateBuilder::Create')
+  external void _constructor();
 
   /// Update the information associated with the node with the given `id`.
   ///
@@ -879,10 +912,6 @@ class SemanticsUpdateBuilder extends NativeFieldWrapperClass1 {
     required Int32List additionalActions,
   }) {
     assert(_matrix4IsValid(transform));
-    assert(
-      scrollChildren == 0 || scrollChildren == null || (scrollChildren > 0 && childrenInHitTestOrder != null),
-      'If a node has scrollChildren, it must have childrenInHitTestOrder',
-    );
     _updateNode(
       id,
       flags,
@@ -921,43 +950,80 @@ class SemanticsUpdateBuilder extends NativeFieldWrapperClass1 {
       additionalActions,
     );
   }
-  void _updateNode(
-    int id,
-    int flags,
-    int actions,
-    int maxValueLength,
-    int currentValueLength,
-    int textSelectionBase,
-    int textSelectionExtent,
-    int platformViewId,
-    int scrollChildren,
-    int scrollIndex,
-    double scrollPosition,
-    double scrollExtentMax,
-    double scrollExtentMin,
-    double left,
-    double top,
-    double right,
-    double bottom,
-    double elevation,
-    double thickness,
-    String label,
-    List<StringAttribute> labelAttributes,
-    String value,
-    List<StringAttribute> valueAttributes,
-    String increasedValue,
-    List<StringAttribute> increasedValueAttributes,
-    String decreasedValue,
-    List<StringAttribute> decreasedValueAttributes,
-    String hint,
-    List<StringAttribute> hintAttributes,
-    String tooltip,
-    int textDirection,
-    Float64List transform,
-    Int32List childrenInTraversalOrder,
-    Int32List childrenInHitTestOrder,
-    Int32List additionalActions,
-  ) native 'SemanticsUpdateBuilder_updateNode';
+  @Native<
+      Void Function(
+          Pointer<Void>,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Int32,
+          Double,
+          Double,
+          Double,
+          Double,
+          Double,
+          Double,
+          Double,
+          Double,
+          Double,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Handle,
+          Int32,
+          Handle,
+          Handle,
+          Handle,
+          Handle)>(symbol: 'SemanticsUpdateBuilder::updateNode')
+  external void _updateNode(
+      int id,
+      int flags,
+      int actions,
+      int maxValueLength,
+      int currentValueLength,
+      int textSelectionBase,
+      int textSelectionExtent,
+      int platformViewId,
+      int scrollChildren,
+      int scrollIndex,
+      double scrollPosition,
+      double scrollExtentMax,
+      double scrollExtentMin,
+      double left,
+      double top,
+      double right,
+      double bottom,
+      double elevation,
+      double thickness,
+      String label,
+      List<StringAttribute> labelAttributes,
+      String value,
+      List<StringAttribute> valueAttributes,
+      String increasedValue,
+      List<StringAttribute> increasedValueAttributes,
+      String decreasedValue,
+      List<StringAttribute> decreasedValueAttributes,
+      String hint,
+      List<StringAttribute> hintAttributes,
+      String tooltip,
+      int textDirection,
+      Float64List transform,
+      Int32List childrenInTraversalOrder,
+      Int32List childrenInHitTestOrder,
+      Int32List additionalActions);
 
   /// Update the custom semantics action associated with the given `id`.
   ///
@@ -975,27 +1041,25 @@ class SemanticsUpdateBuilder extends NativeFieldWrapperClass1 {
   /// [SemanticsAction.index] value. For custom actions this argument should not be
   /// provided.
   void updateCustomAction({required int id, String? label, String? hint, int overrideId = -1}) {
-    assert(id != null);
-    assert(overrideId != null);
     _updateCustomAction(id, label ?? '', hint ?? '', overrideId);
   }
-  void _updateCustomAction(
-      int id,
-      String label,
-      String hint,
-      int overrideId) native 'SemanticsUpdateBuilder_updateCustomAction';
+  @Native<Void Function(Pointer<Void>, Int32, Handle, Handle, Int32)>(symbol: 'SemanticsUpdateBuilder::updateCustomAction')
+  external void _updateCustomAction(int id, String label, String hint, int overrideId);
 
   /// Creates a [SemanticsUpdate] object that encapsulates the updates recorded
   /// by this object.
   ///
   /// The returned object can be passed to [PlatformDispatcher.updateSemantics]
   /// to actually update the semantics retained by the system.
+  ///
+  /// This object is unusable after calling build.
   SemanticsUpdate build() {
     final SemanticsUpdate semanticsUpdate = SemanticsUpdate._();
     _build(semanticsUpdate);
     return semanticsUpdate;
   }
-  void _build(SemanticsUpdate outSemanticsUpdate) native 'SemanticsUpdateBuilder_build';
+  @Native<Void Function(Pointer<Void>, Handle)>(symbol: 'SemanticsUpdateBuilder::build')
+  external void _build(SemanticsUpdate outSemanticsUpdate);
 }
 
 /// An opaque object representing a batch of semantics updates.
@@ -1017,5 +1081,9 @@ class SemanticsUpdate extends NativeFieldWrapperClass1 {
   ///
   /// After calling this function, the semantics update is cannot be used
   /// further.
-  void dispose() native 'SemanticsUpdate_dispose';
+  ///
+  /// This can't be a leaf call because the native function calls Dart API
+  /// (Dart_SetNativeInstanceField).
+  @Native<Void Function(Pointer<Void>)>(symbol: 'SemanticsUpdate::dispose')
+  external void dispose();
 }
