@@ -8,11 +8,13 @@ import 'dart:io' hide Platform;
 import 'dart:typed_data';
 
 import 'package:args/args.dart';
+import 'package:convert/convert.dart';
 import 'package:crypto/crypto.dart';
 import 'package:crypto/src/digest_sink.dart';
 import 'package:http/http.dart' as http;
 import 'package:path/path.dart' as path;
-import 'package:platform/platform.dart' show Platform, LocalPlatform;
+import 'package:platform/platform.dart' show LocalPlatform, Platform;
+import 'package:pool/pool.dart';
 import 'package:process/process.dart';
 
 const String gobMirror =
@@ -41,9 +43,7 @@ class PreparePackageException implements Exception {
   @override
   String toString() {
     String output = runtimeType.toString();
-    if (message != null) {
-      output += ': $message';
-    }
+    output += ': $message';
     final String stderr = result?.stderr as String? ?? '';
     if (stderr.isNotEmpty) {
       output += ':\n$stderr';
@@ -52,30 +52,11 @@ class PreparePackageException implements Exception {
   }
 }
 
-enum Branch { dev, beta, stable }
-
-String getBranchName(Branch branch) {
-  switch (branch) {
-    case Branch.beta:
-      return 'beta';
-    case Branch.dev:
-      return 'dev';
-    case Branch.stable:
-      return 'stable';
-  }
-}
-
-Branch fromBranchName(String name) {
-  switch (name) {
-    case 'beta':
-      return Branch.beta;
-    case 'dev':
-      return Branch.dev;
-    case 'stable':
-      return Branch.stable;
-    default:
-      throw ArgumentError('Invalid branch name.');
-  }
+enum Branch {
+  beta,
+  stable,
+  master,
+  main;
 }
 
 /// A helper class for classes that want to run a process, optionally have the
@@ -185,7 +166,7 @@ typedef HttpReader = Future<Uint8List> Function(Uri url, {Map<String, String> he
 
 /// Creates a pre-populated Flutter archive from a git repo.
 class ArchiveCreator {
-  /// [tempDir] is the directory to use for creating the archive.  The script
+  /// [tempDir] is the directory to use for creating the archive. The script
   /// will place several GiB of data there, so it should have available space.
   ///
   /// The processManager argument is used to inject a mock of [ProcessManager] for
@@ -210,7 +191,7 @@ class ArchiveCreator {
       subprocessOutput: subprocessOutput,
       platform: platform,
     )..environment['PUB_CACHE'] = path.join(
-      flutterRoot.absolute.path, '.pub-cache',
+      tempDir.path, '.pub-cache',
     );
     final String flutterExecutable = path.join(
       flutterRoot.absolute.path,
@@ -263,7 +244,7 @@ class ArchiveCreator {
   /// platform we're running on.
   final Platform platform;
 
-  /// The branch to build the archive for.  The branch must contain [revision].
+  /// The branch to build the archive for. The branch must contain [revision].
   final Branch branch;
 
   /// The git revision hash to build the archive for. This revision has
@@ -304,14 +285,11 @@ class ArchiveCreator {
         .trim().split(' ').last.replaceAll('"', '').split('_')[1];
   })();
 
-  /// Get the name of the channel as a string.
-  String get branchName => getBranchName(branch);
-
   /// Returns a default archive name when given a Git revision.
   /// Used when an output filename is not given.
   Future<String> get _archiveName async {
     final String os = platform.operatingSystem.toLowerCase();
-    // Include the intended host archetecture in the file name for non-x64.
+    // Include the intended host architecture in the file name for non-x64.
     final String arch = await _dartArch == 'x64' ? '' : '${await _dartArch}_';
     // We don't use .tar.xz on Mac because although it can unpack them
     // on the command line (with tar), the "Archive Utility" that runs
@@ -321,7 +299,8 @@ class ArchiveCreator {
     // unpacking it!) So, we use .zip for Mac, and the files are about
     // 220MB larger than they need to be. :-(
     final String suffix = platform.isLinux ? 'tar.xz' : 'zip';
-    return 'flutter_${os}_$arch${_version[frameworkVersionTag]}-$branchName.$suffix';
+    final String package = '${os}_$arch${_version[frameworkVersionTag]}';
+    return 'flutter_$package-${branch.name}.$suffix';
   }
 
   /// Checks out the flutter repo and prepares it for other operations.
@@ -353,7 +332,7 @@ class ArchiveCreator {
   /// Validates the integrity of the release package.
   ///
   /// Currently only checks that macOS binaries are codesigned. Will throw a
-  /// [PreparePackageException] if the test failes.
+  /// [PreparePackageException] if the test fails.
   Future<void> _validate() async {
     // Only validate in strict mode, which means `--publish`
     if (!strict || !platform.isMacOS) {
@@ -385,7 +364,7 @@ class ArchiveCreator {
   /// git will give an error.
   ///
   /// If [strict] is true, the exact [revision] must be tagged to return the
-  /// version.  If [strict] is not true, will look backwards in time starting at
+  /// version. If [strict] is not true, will look backwards in time starting at
   /// [revision] to find the most recent version tag.
   ///
   /// The version found as a git tag is added to the information given by
@@ -425,11 +404,14 @@ class ArchiveCreator {
     // We want the user to start out the in the specified branch instead of a
     // detached head. To do that, we need to make sure the branch points at the
     // desired revision.
-    await _runGit(<String>['clone', '-b', branchName, gobMirror], workingDirectory: tempDir);
+    await _runGit(<String>['clone', '-b', branch.name, gobMirror], workingDirectory: tempDir);
     await _runGit(<String>['reset', '--hard', revision]);
 
     // Make the origin point to github instead of the chromium mirror.
     await _runGit(<String>['remote', 'set-url', 'origin', githubRepo]);
+
+    // Minify `.git` footprint (saving about ~100 MB as of Oct 2022)
+    await _runGit(<String>['gc', '--prune=now', '--aggressive']);
   }
 
   /// Retrieve the MinGit executable from storage and unpack it.
@@ -446,6 +428,104 @@ class ArchiveCreator {
     await _unzipArchive(gitFile, workingDirectory: minGitPath);
   }
 
+  /// Downloads an archive of every package that is present in the temporary
+  /// pub-cache from pub.dev. Stores the archives in
+  /// $flutterRoot/.pub-preload-cache.
+  ///
+  /// These archives will be installed in the user-level cache on first
+  /// following flutter command that accesses the cache.
+  ///
+  /// Precondition: all packages currently in the PUB_CACHE of [_processRunner]
+  /// are installed from pub.dev.
+  Future<void> _downloadPubPackageArchives() async {
+    final Pool pool = Pool(10); // Number of simultaneous downloads.
+    final http.Client client = http.Client();
+    final Directory preloadCache = Directory(path.join(flutterRoot.path, '.pub-preload-cache'));
+    preloadCache.createSync(recursive: true);
+    /// Fetch a single package.
+    Future<void> fetchPackageArchive(String name, String version) async {
+      await pool.withResource(() async {
+        stderr.write('Fetching package archive for $name-$version.\n');
+        int retries = 7;
+        while (true) {
+          retries-=1;
+          try {
+            final Uri packageListingUrl =
+              Uri.parse('https://pub.dev/api/packages/$name');
+            // Fetch the package listing to obtain the package download url.
+            final http.Response packageListingResponse =
+                await client.get(packageListingUrl);
+            if (packageListingResponse.statusCode != 200) {
+              throw Exception('Downloading $packageListingUrl failed. Status code ${packageListingResponse.statusCode}.');
+            }
+            final dynamic decodedPackageListing = json.decode(packageListingResponse.body);
+            if (decodedPackageListing is! Map) {
+              throw const FormatException('Package listing should be a map');
+            }
+            final dynamic versions =  decodedPackageListing['versions'];
+            if (versions is! List) {
+              throw const FormatException('.versions should be a list');
+            }
+            final Map<String, dynamic> versionDescription = versions.firstWhere(
+              (dynamic description) {
+                if (description is! Map) {
+                  throw const FormatException('.versions elements should be maps');
+                }
+                return description['version'] == version;
+              },
+              orElse: () => throw FormatException('Could not find $name-$version in package listing')
+            ) as Map<String, dynamic>;
+            final dynamic downloadUrl = versionDescription['archive_url'];
+            if (downloadUrl is! String) {
+              throw const FormatException('archive_url should be a string');
+            }
+            final dynamic archiveSha256 = versionDescription['archive_sha256'];
+            if (archiveSha256 is! String) {
+              throw const FormatException('archive_sha256 should be a string');
+            }
+            final http.Request request = http.Request('get', Uri.parse(downloadUrl));
+            final http.StreamedResponse response = await client.send(request);
+            if (response.statusCode != 200) {
+              throw Exception('Downloading ${request.url} failed. Status code ${response.statusCode}.');
+            }
+            final File archiveFile = File(
+              path.join(preloadCache.path, '$name-$version.tar.gz'),
+            );
+            await response.stream.pipe(archiveFile.openWrite());
+            final Stream<List<int>> archiveStream = archiveFile.openRead();
+            final Digest r = await sha256.bind(archiveStream).first;
+            if (hex.encode(r.bytes) != archiveSha256) {
+              throw Exception('Hash mismatch of downloaded archive');
+            }
+          } on Exception catch (e) {
+            stderr.write('Failed downloading $name-$version. $e\n');
+            if (retries > 0) {
+              stderr.write('Retrying download of $name-$version...');
+              // Retry.
+              continue;
+            } else {
+              rethrow;
+            }
+          }
+          break;
+        }
+      });
+    }
+    final Map<String, dynamic> cacheDescription =
+        json.decode(await _runFlutter(<String>['pub', 'cache', 'list'])) as Map<String, dynamic>;
+    final Map<String, dynamic> packages = cacheDescription['packages'] as Map<String, dynamic>;
+    final List<Future<void>> downloads = <Future<void>>[];
+    for (final MapEntry<String, dynamic> package in packages.entries) {
+      final String name = package.key;
+      final Map<String, dynamic> versions = package.value as Map<String, dynamic>;
+      for (final String version in versions.keys) {
+        downloads.add(fetchPackageArchive(name, version));
+      }
+    }
+    await Future.wait(downloads);
+    client.close();
+  }
+
   /// Prepare the archive repo so that it has all of the caches warmed up and
   /// is configured for the user to begin working.
   Future<void> _populateCaches() async {
@@ -466,7 +546,7 @@ class ArchiveCreator {
         workingDirectory: tempDir,
       );
     }
-
+    await _downloadPubPackageArchives();
     // Yes, we could just skip all .packages files when constructing
     // the archive, but some are checked in, and we don't want to skip
     // those.
@@ -624,8 +704,7 @@ class ArchivePublisher {
   final File outputFile;
   final ProcessRunner _processRunner;
   final bool dryRun;
-  String get branchName => getBranchName(branch);
-  String get destinationArchivePath => '$branchName/$platformName/${path.basename(outputFile.path)}';
+  String get destinationArchivePath => '${branch.name}/$platformName/${path.basename(outputFile.path)}';
   static String getMetadataFilename(Platform platform) => 'releases_${platform.operatingSystem.toLowerCase()}.json';
 
   Future<String> _getChecksum(File archiveFile) async {
@@ -658,6 +737,12 @@ class ArchivePublisher {
       dest: destGsPath,
     );
     assert(tempDir.existsSync());
+    final String gcsPath = '$gsReleaseFolder/${getMetadataFilename(platform)}';
+    await _publishMetadata(gcsPath);
+  }
+
+  /// Downloads and updates the metadata file without publishing it.
+  Future<void> generateLocalMetadata() async {
     await _updateMetadata('$gsReleaseFolder/${getMetadataFilename(platform)}');
   }
 
@@ -666,14 +751,14 @@ class ArchivePublisher {
     if (!jsonData.containsKey('current_release')) {
       jsonData['current_release'] = <String, String>{};
     }
-    (jsonData['current_release'] as Map<String, dynamic>)[branchName] = revision;
+    (jsonData['current_release'] as Map<String, dynamic>)[branch.name] = revision;
     if (!jsonData.containsKey('releases')) {
       jsonData['releases'] = <Map<String, dynamic>>[];
     }
 
     final Map<String, dynamic> newEntry = <String, dynamic>{};
     newEntry['hash'] = revision;
-    newEntry['channel'] = branchName;
+    newEntry['channel'] = branch.name;
     newEntry['version'] = version[frameworkVersionTag];
     newEntry['dart_sdk_version'] = version[dartVersionTag];
     newEntry['dart_sdk_arch'] = version[dartTargetArchTag];
@@ -726,6 +811,13 @@ class ArchivePublisher {
 
     const JsonEncoder encoder = JsonEncoder.withIndent('  ');
     metadataFile.writeAsStringSync(encoder.convert(jsonData));
+  }
+
+  /// Publishes the metadata file to GCS.
+  Future<void> _publishMetadata(String gsPath) async {
+    final File metadataFile = File(
+      path.join(tempDir.absolute.path, getMetadataFilename(platform)),
+    );
     await _cloudCopy(
       src: metadataFile.absolute.path,
       dest: gsPath,
@@ -803,8 +895,8 @@ class ArchivePublisher {
   }
 }
 
-/// Prepares a flutter git repo to be packaged up for distribution.
-/// It mainly serves to populate the .pub-cache with any appropriate Dart
+/// Prepares a flutter git repo to be packaged up for distribution. It mainly
+/// serves to populate the .pub-preload-cache with any appropriate Dart
 /// packages, and the flutter cache in bin/cache with the appropriate
 /// dependencies and snapshots.
 ///
@@ -825,7 +917,7 @@ Future<void> main(List<String> rawArguments) async {
           'archive with. Must be the full 40-character hash. Required.');
   argParser.addOption(
     'branch',
-    allowed: Branch.values.map<String>((Branch branch) => getBranchName(branch)),
+    allowed: Branch.values.map<String>((Branch branch) => branch.name),
     help: 'The Flutter branch to build the archive with. Required.',
   );
   argParser.addOption(
@@ -870,10 +962,10 @@ Future<void> main(List<String> rawArguments) async {
     exit(exitCode);
   }
 
-  final String revision = parsedArguments['revision'] as String;
   if (!parsedArguments.wasParsed('revision')) {
     errorExit('Invalid argument: --revision must be specified.');
   }
+  final String revision = parsedArguments['revision'] as String;
   if (revision.length != 40) {
     errorExit('Invalid argument: --revision must be the entire hash, not just a prefix.');
   }
@@ -882,7 +974,7 @@ Future<void> main(List<String> rawArguments) async {
     errorExit('Invalid argument: --branch must be specified.');
   }
 
-  final String tempDirArg = parsedArguments['temp_dir'] as String;
+  final String? tempDirArg = parsedArguments['temp_dir'] as String?;
   Directory tempDir;
   bool removeTempDir = false;
   if (tempDirArg == null || tempDirArg.isEmpty) {
@@ -907,7 +999,7 @@ Future<void> main(List<String> rawArguments) async {
 
   final bool publish = parsedArguments['publish'] as bool;
   final bool dryRun = parsedArguments['dry_run'] as bool;
-  final Branch branch = fromBranchName(parsedArguments['branch'] as String);
+  final Branch branch = Branch.values.byName(parsedArguments['branch'] as String);
   final ArchiveCreator creator = ArchiveCreator(
     tempDir,
     outputDir,
@@ -920,15 +1012,16 @@ Future<void> main(List<String> rawArguments) async {
   try {
     final Map<String, String> version = await creator.initializeRepo();
     final File outputFile = await creator.createArchive();
+    final ArchivePublisher publisher = ArchivePublisher(
+      tempDir,
+      revision,
+      branch,
+      version,
+      outputFile,
+      dryRun,
+    );
+    await publisher.generateLocalMetadata();
     if (parsedArguments['publish'] as bool) {
-      final ArchivePublisher publisher = ArchivePublisher(
-        tempDir,
-        revision,
-        branch,
-        version,
-        outputFile,
-        dryRun,
-      );
       await publisher.publishArchive(parsedArguments['force'] as bool);
     }
   } on PreparePackageException catch (e) {
