@@ -2,31 +2,28 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for InputDecoration
-
 import 'package:flutter/material.dart';
 
-void main() => runApp(const MyApp());
+/// Flutter code sample for [InputDecoration].
 
-class MyApp extends StatelessWidget {
-  const MyApp({Key? key}) : super(key: key);
+void main() => runApp(const MaterialStateExampleApp());
 
-  static const String _title = 'Flutter Code Sample';
+class MaterialStateExampleApp extends StatelessWidget {
+  const MaterialStateExampleApp({super.key});
 
   @override
   Widget build(BuildContext context) {
     return MaterialApp(
-      title: _title,
       home: Scaffold(
-        appBar: AppBar(title: const Text(_title)),
-        body: const MyStatelessWidget(),
+        appBar: AppBar(title: const Text('InputDecoration Sample')),
+        body: const MaterialStateExample(),
       ),
     );
   }
 }
 
-class MyStatelessWidget extends StatelessWidget {
-  const MyStatelessWidget({Key? key}) : super(key: key);
+class MaterialStateExample extends StatelessWidget {
+  const MaterialStateExample({super.key});
 
   @override
   Widget build(BuildContext context) {
@@ -37,7 +34,8 @@ class MyStatelessWidget extends StatelessWidget {
         prefixIconColor: MaterialStateColor.resolveWith((Set<MaterialState> states) {
           if (states.contains(MaterialState.focused)) {
             return Colors.green;
-          } if (states.contains(MaterialState.error)) {
+          }
+          if (states.contains(MaterialState.error)) {
             return Colors.red;
           }
           return Colors.grey;
