@@ -14,7 +14,7 @@ export 'dart:ui' show KeyData;
 export 'package:flutter/foundation.dart' show DiagnosticPropertiesBuilder;
 
 export 'keyboard_key.g.dart' show LogicalKeyboardKey, PhysicalKeyboardKey;
-export 'raw_keyboard.dart' show RawKeyboard, RawKeyEvent;
+export 'raw_keyboard.dart' show RawKeyEvent, RawKeyboard;
 
 /// Represents a lock mode of a keyboard, such as [KeyboardLockMode.capsLock].
 ///
@@ -26,34 +26,33 @@ export 'raw_keyboard.dart' show RawKeyboard, RawKeyEvent;
 /// Only a limited number of modes are supported, which are enumerated as
 /// static members of this class. Manual constructing of this class is
 /// prohibited.
-@immutable
-class KeyboardLockMode {
-  // KeyboardLockMode has a fixed pool of supported keys, enumerated as static
-  // members of this class, therefore constructing is prohibited.
-  const KeyboardLockMode._(this.logicalKey);
-
-  /// The logical key that triggers this lock mode.
-  final LogicalKeyboardKey logicalKey;
-
+enum KeyboardLockMode {
   /// Represents the number lock mode on the keyboard.
   ///
   /// On supporting systems, enabling number lock mode usually allows key
   /// presses of the number pad to input numbers, instead of acting as up, down,
   /// left, right, page up, end, etc.
-  static const KeyboardLockMode numLock = KeyboardLockMode._(LogicalKeyboardKey.numLock);
+  numLock._(LogicalKeyboardKey.numLock),
 
   /// Represents the scrolling lock mode on the keyboard.
   ///
   /// On supporting systems and applications (such as a spreadsheet), enabling
   /// scrolling lock mode usually allows key presses of the cursor keys to
   /// scroll the document instead of the cursor.
-  static const KeyboardLockMode scrollLock = KeyboardLockMode._(LogicalKeyboardKey.scrollLock);
+  scrollLock._(LogicalKeyboardKey.scrollLock),
 
   /// Represents the capital letters lock mode on the keyboard.
   ///
   /// On supporting systems, enabling capital lock mode allows key presses of
   /// the letter keys to input uppercase letters instead of lowercase.
-  static const KeyboardLockMode capsLock = KeyboardLockMode._(LogicalKeyboardKey.capsLock);
+  capsLock._(LogicalKeyboardKey.capsLock);
+
+  // KeyboardLockMode has a fixed pool of supported keys, enumerated as static
+  // members of this class, therefore constructing is prohibited.
+  const KeyboardLockMode._(this.logicalKey);
+
+  /// The logical key that triggers this lock mode.
+  final LogicalKeyboardKey logicalKey;
 
   static final Map<int, KeyboardLockMode> _knownLockModes = <int, KeyboardLockMode>{
     numLock.logicalKey.keyId: numLock,
@@ -101,7 +100,7 @@ abstract class KeyEvent with Diagnosticable {
   /// is a mnemonic ("keyA" is easier to remember than 0x70004), derived from the
   /// key's effect on a QWERTY keyboard. The name does not represent the key's
   /// effect whatsoever (a physical "keyA" can be the Q key on an AZERTY
-  /// keyboard.)
+  /// keyboard).
   ///
   /// For instance, if you wanted to make a game where the key to the right of
   /// the CAPS LOCK key made the player move left, you would be comparing a
@@ -115,10 +114,10 @@ abstract class KeyEvent with Diagnosticable {
   ///
   /// Also, even though physical keys are defined with USB HID codes, their
   /// values are not necessarily the same HID codes produced by the hardware and
-  /// presented to the driver, because on most platforms Flutter has to map the
-  /// platform representation back to an HID code since the original HID
-  /// code is not provided. USB HID is simply a conveniently well-defined
-  /// standard to list possible keys that a Flutter app can encounter.
+  /// presented to the driver. On most platforms, Flutter has to map the
+  /// platform representation back to a HID code because the original HID
+  /// code is not provided. USB HID was chosen because it is a well-defined
+  /// standard for referring to keys such as those a Flutter app may encounter.
   ///
   /// See also:
   ///
@@ -146,7 +145,7 @@ abstract class KeyEvent with Diagnosticable {
   /// keystroke, if any.
   ///
   /// This will only return a character if this keystroke, combined with any
-  /// preceding keystroke(s), generats a character, and only on a "key down"
+  /// preceding keystroke(s), generates a character, and only on a "key down"
   /// event. It will return null if no character has been generated by the
   /// keystroke (e.g. a "dead" or "combining" key), or if the corresponding key
   /// is a key without a visual representation, such as a modifier key or a
@@ -178,7 +177,7 @@ abstract class KeyEvent with Diagnosticable {
   /// might be converted to a Flutter down event when necessary.)
   ///
   /// A [synthesized] event is created without a source native event in order to
-  /// synthronize key states. For example, if the native platform shows that a
+  /// synchronize key states. For example, if the native platform shows that a
   /// shift key that was previously held has been released somehow without the
   /// key up event dispatched (probably due to loss of focus), a synthesized key
   /// up event will be added to regularized the event stream.
@@ -242,6 +241,9 @@ class KeyUpEvent extends KeyEvent {
 /// An event indicating that the user has been holding a key on the keyboard
 /// and causing repeated events.
 ///
+/// Repeat events are not guaranteed and are provided only if supported by the
+/// underlying platform.
+///
 /// See also:
 ///
 ///  * [KeyDownEvent], a key event representing the user
@@ -259,7 +261,7 @@ class KeyRepeatEvent extends KeyEvent {
   });
 }
 
-/// The signature for [HardwareKeyboard.addHandler], a callback to to decide whether
+/// The signature for [HardwareKeyboard.addHandler], a callback to decide whether
 /// the entire framework handles a key event.
 typedef KeyEventCallback = bool Function(KeyEvent event);
 
@@ -353,7 +355,7 @@ typedef KeyEventCallback = bool Function(KeyEvent event);
 /// ## Compared to [RawKeyboard]
 ///
 /// [RawKeyboard] is the legacy API, and will be deprecated and removed in the
-/// future.  It is recommended to always use [HardwareKeyboard] and [KeyEvent]
+/// future. It is recommended to always use [HardwareKeyboard] and [KeyEvent]
 /// APIs (such as [FocusNode.onKeyEvent]) to handle key events.
 ///
 /// Behavior-wise, [RawKeyboard] provides a less unified, less regular
@@ -618,25 +620,50 @@ enum KeyDataTransitMode {
   keyDataThenRawKeyData,
 }
 
-/// The assumbled information corresponding to a native key message.
+/// The assembled information converted from a native key message.
 ///
-/// While Flutter's [KeyEvent]s are created from key messages from the native
-/// platform, every native message might result in multiple [KeyEvent]s. For
-/// example, this might happen in order to synthesize missed modifier key
-/// presses or releases.
+/// Native key messages, produced by physically pressing or releasing
+/// keyboard keys, are translated into two different event streams in Flutter:
 ///
-/// A [KeyMessage] bundles all information related to a native key message
-/// together for the convenience of propagation on the [FocusNode] tree.
+///  * The [KeyEvent] stream, represented by [KeyMessage.events] (recommended).
+///  * The [RawKeyEvent] stream, represented by [KeyMessage.rawEvent] (legacy,
+///    to be deprecated).
 ///
-/// When dispatched to handlers or listeners, or propagated through the
-/// [FocusNode] tree, all handlers or listeners belonging to a node are
-/// executed regardless of their [KeyEventResult], and all results are combined
-/// into the result of the node using [combineKeyEventResults]. Empty [events]
-/// or [rawEvent] should be considered as a result of [KeyEventResult.ignored].
+/// Either the [KeyEvent] stream or the [RawKeyEvent] stream alone provides a
+/// complete description of the keyboard messages, but in different event
+/// models. Flutter is still transitioning from the legacy model to the new
+/// model, therefore it dispatches both streams simultaneously until the
+/// transition is completed. [KeyMessage] is used to bundle the
+/// stream segments of both models from a native key message together for the
+/// convenience of propagation.
 ///
-/// In very rare cases, a native key message might not result in a [KeyMessage].
-/// For example, key messages for Fn key are ignored on macOS for the
-/// convenience of cross-platform code.
+/// Typically, an application either processes [KeyMessage.events]
+/// or [KeyMessage.rawEvent], not both. For example, handling a
+/// [KeyMessage], means handling each event in [KeyMessage.events].
+///
+/// In advanced cases, a widget needs to process both streams at the same time.
+/// For example, [FocusNode] has an `onKey` that dispatches [RawKeyEvent]s and
+/// an `onKeyEvent` that dispatches [KeyEvent]s. To processes a [KeyMessage],
+/// it first calls `onKeyEvent` with each [KeyEvent] of [events], and then
+/// `onKey` with [rawEvent]. All callbacks are invoked regardless of their
+/// [KeyEventResult]. Their results are combined into the result of the node
+/// using [combineKeyEventResults].
+///
+/// ```dart
+/// void handleMessage(FocusNode node, KeyMessage message) {
+///   final List<KeyEventResult> results = <KeyEventResult>[];
+///   if (node.onKeyEvent != null) {
+///     for (final KeyEvent event in message.events) {
+///       results.add(node.onKeyEvent!(node, event));
+///     }
+///   }
+///   if (node.onKey != null && message.rawEvent != null) {
+///     results.add(node.onKey!(node, message.rawEvent!));
+///   }
+///   final KeyEventResult result = combineKeyEventResults(results);
+///   // Progress based on `result`...
+/// }
+/// ```
 @immutable
 class KeyMessage {
   /// Create a [KeyMessage] by providing all information.
@@ -647,7 +674,7 @@ class KeyMessage {
   /// The list of [KeyEvent]s converted from the native key message.
   ///
   /// A native key message is converted into multiple [KeyEvent]s in a regular
-  /// event model.  The [events] might contain zero or any number of
+  /// event model. The [events] might contain zero or any number of
   /// [KeyEvent]s.
   ///
   /// See also:
@@ -710,39 +737,87 @@ class KeyEventManager {
   /// This is typically only called by [ServicesBinding].
   KeyEventManager(this._hardwareKeyboard, this._rawKeyboard);
 
-  /// The global handler for all hardware key messages of Flutter.
+  /// The global entrance which handles all key events sent to Flutter.
   ///
-  /// Key messages received from the platform are first sent to [RawKeyboard]'s
-  /// listeners and [HardwareKeyboard]'s handlers, then sent to
-  /// [keyMessageHandler], regardless of the results of [HardwareKeyboard]'s
-  /// handlers. The event results from the handlers and [keyMessageHandler] are
-  /// combined and returned to the platform. The event result is explained
-  /// below.
+  /// Typical applications use [WidgetsBinding], where this field is
+  /// set by the focus system (see `FocusManager`) on startup to a function that
+  /// dispatches incoming events to the focus system, including
+  /// `FocusNode.onKey`, `FocusNode.onKeyEvent`, and `Shortcuts`. In this case,
+  /// the application does not need to read, assign, or invoke this value.
   ///
-  /// For most common applications, which use [WidgetsBinding], this field
-  /// is set by the focus system (see `FocusManger`) on startup and should not
-  /// be change explicitly.
+  /// For advanced uses, the application can "patch" this callback. See below
+  /// for details.
+  ///
+  /// ## Handlers and event results
+  ///
+  /// Roughly speaking, Flutter processes each native key event with the
+  /// following phases:
+  ///
+  /// 1. Platform-side pre-filtering, sometimes used for IME.
+  /// 2. The key event system.
+  /// 3. The text input system.
+  /// 4. Other native components (possibly non-Flutter).
+  ///
+  /// Each phase will conclude with a boolean called an "event result". If the
+  /// result is true, this phase _handles_ the event and prevents the event
+  /// from being propagated to the next phase. This mechanism allows shortcuts
+  /// such as "Ctrl-C" to not generate a text "C" in the text field, or
+  /// shortcuts that are not handled by any components to trigger special alerts
+  /// (such as the "bonk" noise on macOS).
+  ///
+  /// In the second phase, known as "the key event system", the event is dispatched
+  /// to several destinations: [RawKeyboard]'s listeners,
+  /// [HardwareKeyboard]'s handlers, and [keyMessageHandler].
+  /// All destinations will always receive the event regardless of the handlers'
+  /// results. If any handler's result is true, then the overall result of the
+  /// second phase is true, and event propagation is stopped.
+  ///
+  /// See also:
+  ///
+  ///  * [RawKeyboard.addListener], which adds a raw keyboard listener.
+  ///  * [RawKeyboardListener], which is also implemented by adding a raw
+  ///    keyboard listener.
+  ///  * [HardwareKeyboard.addHandler], which adds a hardware keyboard handler.
+  ///
+  /// ## Advanced usages: Manual assignment or patching
   ///
   /// If you are not using the focus system to manage focus, set this
   /// attribute to a [KeyMessageHandler] that returns true if the propagation
   /// on the platform should not be continued. If this field is null, key events
-  /// will be assumed to not have been handled by Flutter.
+  /// will be assumed to not have been handled by Flutter, a result of "false".
   ///
-  /// ## Event result
+  /// Even if you are using the focus system, you might also want to do more
+  /// than the focus system allows. In these cases, you can _patch_
+  /// [keyMessageHandler] by setting it to a callback that performs your tasks
+  /// and calls the original callback in between (or not at all.)
   ///
-  /// Key messages on the platform are given to Flutter to be handled by the
-  /// engine. If they are not handled, then the platform will continue to
-  /// distribute the keys (i.e. propagate them) to other (possibly non-Flutter)
-  /// components in the application. The return value from this handler tells
-  /// the platform to either stop propagation (by returning true: "event
-  /// handled"), or pass the event on to other controls (false: "event not
-  /// handled"). Some platforms might trigger special alerts if the event
-  /// is not handled by other controls either (such as the "bonk" noise on
-  /// macOS).
+  /// Patching [keyMessageHandler] can not be reverted. You should always assume
+  /// that another component might have patched it before you and after you.
+  /// This means that you might want to write your own global notification
+  /// manager, to which callbacks can be added and removed.
   ///
-  /// The result from [keyMessageHandler] and [HardwareKeyboard]'s handlers
-  /// are combined. If any of the handlers claim to handle the event,
-  /// the overall result will be "event handled".
+  /// You should not patch [keyMessageHandler] until the `FocusManager` has assigned
+  /// its callback. This is assured during any time within the widget lifecycle
+  /// (such as `initState`), or after calling `WidgetManager.instance`.
+  ///
+  /// {@tool dartpad}
+  /// This example shows how to process key events that are not handled by any
+  /// focus handler (such as `Shortcuts`) by patching [keyMessageHandler].
+  ///
+  /// The app prints out any key events that are not handled by the app body.
+  /// Try typing something in the first text field. These key presses are not
+  /// handled by `Shortcuts` and will be sent to the fallback handler and printed
+  /// out. Now try some text shortcuts, such as Ctrl+A. The KeyA press is
+  /// handled as a shortcut, and is not sent to the fallback handler and so is
+  /// not printed out.
+  ///
+  /// The key widget is `FallbackKeyEventRegistrar`, a necessity class to allow
+  /// reversible patching. `FallbackFocus` and `FallbackFocusNode` are also
+  /// useful to recognize the widget tree's structure. `FallbackDemo` is an
+  /// example of using them in an app.
+  ///
+  /// ** See code in examples/api/lib/widgets/hardware_keyboard/key_event_manager.0.dart **
+  /// {@end-tool}
   ///
   /// See also:
   ///
@@ -789,7 +864,7 @@ class KeyEventManager {
         return false;
       case KeyDataTransitMode.keyDataThenRawKeyData:
         // Having 0 as the physical and logical ID indicates an empty key data
-        // (the only occassion either field can be 0,) transmitted to ensure
+        // (the only occasion either field can be 0,) transmitted to ensure
         // that the transit mode is correctly inferred. These events should be
         // ignored.
         if (data.physical == 0 && data.logical == 0) {
