@@ -3,9 +3,11 @@
 // found in the LICENSE file.
 
 @TestOn('!chrome')
+library;
 import 'dart:async';
 
 import 'package:flutter/cupertino.dart';
+import 'package:flutter/foundation.dart';
 import 'package:flutter/services.dart';
 import 'package:flutter_test/flutter_test.dart';
 
@@ -150,13 +152,25 @@ void main() {
           refreshTriggerPullDistance: 100,  // default value.
           refreshIndicatorExtent: 60,  // default value.
         ),
-        matchesBuilder(
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) matchesBuilder(
+          refreshState: RefreshIndicatorMode.drag,
+          pulledExtent: moreOrLessEquals(48.07979523362715),
+          refreshTriggerPullDistance: 100,  // default value.
+          refreshIndicatorExtent: 60,  // default value.
+        )
+        else matchesBuilder(
           refreshState: RefreshIndicatorMode.drag,
           pulledExtent: moreOrLessEquals(48.36801747187993),
           refreshTriggerPullDistance: 100,  // default value.
           refreshIndicatorExtent: 60,  // default value.
         ),
-        matchesBuilder(
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) matchesBuilder(
+          refreshState: RefreshIndicatorMode.drag,
+          pulledExtent: moreOrLessEquals(43.98499220391114),
+          refreshTriggerPullDistance: 100,  // default value.
+          refreshIndicatorExtent: 60,  // default value.
+        )
+        else matchesBuilder(
           refreshState: RefreshIndicatorMode.drag,
           pulledExtent: moreOrLessEquals(44.63031931875867),
           refreshTriggerPullDistance: 100,  // default value.
@@ -197,9 +211,19 @@ void main() {
       final TestGesture gesture = await tester.startGesture(Offset.zero);
       await gesture.moveBy(const Offset(0.0, 99.0));
       await tester.pump();
-      await gesture.moveBy(const Offset(0.0, -30.0));
+      if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+        await gesture.moveBy(const Offset(0.0, -3.0));
+      }
+      else {
+        await gesture.moveBy(const Offset(0.0, -30.0));
+      }
       await tester.pump();
-      await gesture.moveBy(const Offset(0.0, 50.0));
+      if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+        await gesture.moveBy(const Offset(0.0, 90.0));
+      }
+      else {
+        await gesture.moveBy(const Offset(0.0, 50.0));
+      }
       await tester.pump();
 
       expect(mockHelper.invocations, containsAllInOrder(<void>[
@@ -209,13 +233,25 @@ void main() {
           refreshTriggerPullDistance: 100,  // default value.
           refreshIndicatorExtent: 60,  // default value.
         ),
-        matchesBuilder(
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) matchesBuilder(
+          refreshState: RefreshIndicatorMode.drag,
+          pulledExtent: moreOrLessEquals(96),
+          refreshTriggerPullDistance: 100,  // default value.
+          refreshIndicatorExtent: 60,  // default value.
+        )
+        else matchesBuilder(
           refreshState: RefreshIndicatorMode.drag,
           pulledExtent: moreOrLessEquals(86.78169),
           refreshTriggerPullDistance: 100,  // default value.
           refreshIndicatorExtent: 60,  // default value.
         ),
-        matchesBuilder(
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) matchesBuilder(
+          refreshState: RefreshIndicatorMode.armed,
+          pulledExtent: moreOrLessEquals(100.44528),
+          refreshTriggerPullDistance: 100,  // default value.
+          refreshIndicatorExtent: 60,  // default value.
+        )
+        else matchesBuilder(
           refreshState: RefreshIndicatorMode.armed,
           pulledExtent: moreOrLessEquals(105.80452021305739),
           refreshTriggerPullDistance: 100,  // default value.
@@ -262,7 +298,13 @@ void main() {
             refreshIndicatorExtent: 60, // Default value.
           ),
           equals(const RefreshTaskInvocation()),
-          matchesBuilder(
+          if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) matchesBuilder(
+            refreshState: RefreshIndicatorMode.armed,
+            pulledExtent: moreOrLessEquals(124.87933920045268),
+            refreshTriggerPullDistance: 100, // Default value.
+            refreshIndicatorExtent: 60, // Default value.
+          )
+          else matchesBuilder(
             refreshState: RefreshIndicatorMode.armed,
             pulledExtent: moreOrLessEquals(127.10396988577114),
             refreshTriggerPullDistance: 100, // Default value.
@@ -306,6 +348,7 @@ void main() {
       (WidgetTester tester) async {
         final FlutterError error = FlutterError('Oops');
         double errorCount = 0;
+        final TargetPlatform? platform = debugDefaultTargetPlatformOverride; // Will not be correct within the zone.
 
         runZonedGuarded(
           () async {
@@ -337,7 +380,13 @@ void main() {
                 refreshTriggerPullDistance: 100, // Default value.
               ),
               equals(const RefreshTaskInvocation()),
-              matchesBuilder(
+              if (platform == TargetPlatform.macOS) matchesBuilder(
+                refreshState: RefreshIndicatorMode.armed,
+                pulledExtent: moreOrLessEquals(124.87933920045268),
+                refreshTriggerPullDistance: 100, // Default value.
+                refreshIndicatorExtent: 60, // Default value.
+              )
+              else matchesBuilder(
                 refreshState: RefreshIndicatorMode.armed,
                 pulledExtent: moreOrLessEquals(127.10396988577114),
                 refreshIndicatorExtent: 60, // Default value.
@@ -415,7 +464,12 @@ void main() {
         const Rect.fromLTRB(0.0, 0.0, 800.0, 150.0),
       );
 
-      await tester.drag(find.text('0'), const Offset(0.0, -300.0), touchSlopY: 0, warnIfMissed: false); // hits the list
+      if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+        await tester.drag(find.text('0'), const Offset(0.0, -130.0), touchSlopY: 0, warnIfMissed: false); // hits the list
+      }
+      else {
+        await tester.drag(find.text('0'), const Offset(0.0, -300.0), touchSlopY: 0, warnIfMissed: false); // hits the list
+      }
       await tester.pump();
 
       // Refresh indicator still being told to layout the same way.
@@ -427,18 +481,34 @@ void main() {
       )));
 
       // Now the sliver is scrolled off screen.
-      expect(
-        tester.getTopLeft(find.widgetWithText(Center, '-1', skipOffstage: false)).dy,
-        moreOrLessEquals(-175.38461538461536),
-      );
-      expect(
-        tester.getBottomLeft(find.widgetWithText(Center, '-1', skipOffstage: false)).dy,
-        moreOrLessEquals(-115.38461538461536),
-      );
-      expect(
-        tester.getTopLeft(find.widgetWithText(Center, '0')).dy,
-        moreOrLessEquals(-115.38461538461536),
-      );
+      if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+        expect(
+          tester.getTopLeft(find.widgetWithText(Center, '-1', skipOffstage: false)).dy,
+          moreOrLessEquals(-40),
+        );
+        expect(
+          tester.getBottomLeft(find.widgetWithText(Center, '-1', skipOffstage: false)).dy,
+          moreOrLessEquals(20),
+        );
+        expect(
+          tester.getTopLeft(find.widgetWithText(Center, '0')).dy,
+          moreOrLessEquals(20),
+        );
+      }
+      else {
+        expect(
+          tester.getTopLeft(find.widgetWithText(Center, '-1', skipOffstage: false)).dy,
+          moreOrLessEquals(-175.38461538461536),
+        );
+        expect(
+          tester.getBottomLeft(find.widgetWithText(Center, '-1', skipOffstage: false)).dy,
+          moreOrLessEquals(-115.38461538461536),
+        );
+        expect(
+          tester.getTopLeft(find.widgetWithText(Center, '0')).dy,
+          moreOrLessEquals(-115.38461538461536),
+        );
+      }
 
       // Scroll the top of the refresh indicator back to overscroll, it will
       // snap to the size of the refresh indicator and stay there.
@@ -586,20 +656,38 @@ void main() {
       // Waiting for refresh control to reach approximately 5% of height
       await tester.pump(const Duration(milliseconds: 400));
 
-      expect(
-        tester.getRect(find.widgetWithText(Center, '0')).top,
-        moreOrLessEquals(3.0, epsilon: 4e-1),
-      );
-      expect(
-        tester.getRect(find.widgetWithText(Center, '-1')).height,
-        moreOrLessEquals(3.0, epsilon: 4e-1),
-      );
-      expect(mockHelper.invocations, contains(matchesBuilder(
-        refreshState: RefreshIndicatorMode.inactive,
-        pulledExtent: 2.6980688300546443, // ~5% of 60.0
-        refreshTriggerPullDistance: 100,  // default value.
-        refreshIndicatorExtent: 60,  // default value.
-      )));
+      if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+        expect(
+          tester.getRect(find.widgetWithText(Center, '0')).top,
+          moreOrLessEquals(3.9543032206542765, epsilon: 4e-1),
+        );
+        expect(
+          tester.getRect(find.widgetWithText(Center, '-1')).height,
+          moreOrLessEquals(3.9543032206542765, epsilon: 4e-1),
+        );
+        expect(mockHelper.invocations, contains(matchesBuilder(
+          refreshState: RefreshIndicatorMode.inactive,
+          pulledExtent: 3.9543032206542765, // ~5% of 60.0
+          refreshTriggerPullDistance: 100,  // default value.
+          refreshIndicatorExtent: 60,  // default value.
+        )));
+      }
+      else {
+        expect(
+          tester.getRect(find.widgetWithText(Center, '0')).top,
+          moreOrLessEquals(3.0, epsilon: 4e-1),
+        );
+        expect(
+          tester.getRect(find.widgetWithText(Center, '-1')).height,
+          moreOrLessEquals(3.0, epsilon: 4e-1),
+        );
+        expect(mockHelper.invocations, contains(matchesBuilder(
+          refreshState: RefreshIndicatorMode.inactive,
+          pulledExtent: 2.6980688300546443, // ~5% of 60.0
+          refreshTriggerPullDistance: 100,  // default value.
+          refreshIndicatorExtent: 60,  // default value.
+        )));
+      }
       expect(find.text('-1'), findsOneWidget);
     }, variant: const TargetPlatformVariant(<TargetPlatform>{ TargetPlatform.iOS,  TargetPlatform.macOS }));
 
@@ -638,17 +726,30 @@ void main() {
         // Let it start going away but not fully.
         await tester.pump(const Duration(milliseconds: 100));
         // The refresh indicator is still building.
-        expect(mockHelper.invocations, contains(matchesBuilder(
-          refreshState: RefreshIndicatorMode.done,
-          pulledExtent: 91.31180913199277,
-          refreshTriggerPullDistance: 100,  // default value.
-          refreshIndicatorExtent: 60,  // default value.
-        )));
-
-        expect(
-          tester.getBottomLeft(find.widgetWithText(Center, '-1')).dy,
-          moreOrLessEquals(91.311809131992776),
-        );
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          expect(mockHelper.invocations, contains(matchesBuilder(
+            refreshState: RefreshIndicatorMode.done,
+            pulledExtent: 90.13497854600749,
+            refreshTriggerPullDistance: 100,  // default value.
+            refreshIndicatorExtent: 60,  // default value.
+          )));
+          expect(
+            tester.getBottomLeft(find.widgetWithText(Center, '-1')).dy,
+            moreOrLessEquals(90.13497854600749),
+          );
+        }
+        else {
+          expect(mockHelper.invocations, contains(matchesBuilder(
+            refreshState: RefreshIndicatorMode.done,
+            pulledExtent: 91.31180913199277,
+            refreshTriggerPullDistance: 100,  // default value.
+            refreshIndicatorExtent: 60,  // default value.
+          )));
+          expect(
+            tester.getBottomLeft(find.widgetWithText(Center, '-1')).dy,
+            moreOrLessEquals(91.311809131992776),
+          );
+        }
 
         // Start another drag by an amount that would have been enough to
         // trigger another refresh if it were in the right state.
@@ -657,12 +758,22 @@ void main() {
 
         // Instead, it's still in the done state because the sliver never
         // fully retracted.
-        expect(mockHelper.invocations, contains(matchesBuilder(
-          refreshState: RefreshIndicatorMode.done,
-          pulledExtent: 147.3772721631821,
-          refreshTriggerPullDistance: 100,  // default value.
-          refreshIndicatorExtent: 60,  // default value.
-        )));
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          expect(mockHelper.invocations, contains(matchesBuilder(
+            refreshState: RefreshIndicatorMode.done,
+            pulledExtent: 97.71721346565732,
+            refreshTriggerPullDistance: 100,  // default value.
+            refreshIndicatorExtent: 60,  // default value.
+          )));
+        }
+        else {
+          expect(mockHelper.invocations, contains(matchesBuilder(
+            refreshState: RefreshIndicatorMode.done,
+            pulledExtent: 147.3772721631821,
+            refreshTriggerPullDistance: 100,  // default value.
+            refreshIndicatorExtent: 60,  // default value.
+          )));
+        }
 
         // Now let it fully go away.
         await tester.pump(const Duration(seconds: 5));
@@ -881,12 +992,22 @@ void main() {
 
         await tester.pump(const Duration(milliseconds: 10));
 
-        expect(mockHelper.invocations.last, matchesBuilder(
-          refreshState: RefreshIndicatorMode.done,
-          pulledExtent: moreOrLessEquals(148.6463892921364),
-          refreshTriggerPullDistance: 100.0, // Default value.
-          refreshIndicatorExtent: 60.0, // Default value.
-        ));
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          expect(mockHelper.invocations.last, matchesBuilder(
+            refreshState: RefreshIndicatorMode.done,
+            pulledExtent: moreOrLessEquals(148.36088180097366),
+            refreshTriggerPullDistance: 100.0, // Default value.
+            refreshIndicatorExtent: 60.0, // Default value.
+          ));
+        }
+        else {
+          expect(mockHelper.invocations.last, matchesBuilder(
+            refreshState: RefreshIndicatorMode.done,
+            pulledExtent: moreOrLessEquals(148.6463892921364),
+            refreshTriggerPullDistance: 100.0, // Default value.
+            refreshIndicatorExtent: 60.0, // Default value.
+          ));
+        }
 
         await tester.pump(const Duration(seconds: 5));
         expect(find.text('-1'), findsNothing);
@@ -1038,8 +1159,12 @@ void main() {
         CupertinoSliverRefreshControl.state(tester.element(find.byType(LayoutBuilder))),
         RefreshIndicatorMode.drag,
       );
-
-      await gesture.moveBy(const Offset(0.0, 3.0)); // Overscrolling, need to move more than 1px.
+      if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+        await gesture.moveBy(const Offset(0.0, 20.0)); // Overscrolling, need to move more than 1px.
+      }
+      else {
+        await gesture.moveBy(const Offset(0.0, 3.0)); // Overscrolling, need to move more than 1px.
+      }
       await tester.pump();
       expect(
         CupertinoSliverRefreshControl.state(tester.element(find.byType(LayoutBuilder))),
@@ -1074,12 +1199,25 @@ void main() {
           RefreshIndicatorMode.armed,
         );
 
-        await gesture.moveBy(const Offset(0.0, -80.0)); // Overscrolling, need to move more than -40.
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          await gesture.moveBy(const Offset(0.0, -41.0)); // Overscrolling, need to move more than -40.
+        }
+        else {
+          await gesture.moveBy(const Offset(0.0, -80.0)); // Overscrolling, need to move more than -40.
+        }
         await tester.pump();
-        expect(
-          tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
-          moreOrLessEquals(49.775111111111116), // Below 50 now.
-        );
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          expect(
+            tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
+            moreOrLessEquals(49), // Below 50 now.
+          );
+        }
+        else {
+          expect(
+            tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
+            moreOrLessEquals(49.775111111111116), // Below 50 now.
+          );
+        }
         expect(
           CupertinoSliverRefreshControl.state(tester.element(find.byType(LayoutBuilder))),
           RefreshIndicatorMode.refresh,
@@ -1169,24 +1307,50 @@ void main() {
         await tester.pump();
 
         // Now back in overscroll mode.
-        await gesture.moveBy(const Offset(0.0, -200.0));
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          await gesture.moveBy(const Offset(0.0, -125.0));
+        }
+        else {
+          await gesture.moveBy(const Offset(0.0, -200.0));
+        }
         await tester.pump();
-        expect(
-          tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
-          moreOrLessEquals(27.944444444444457),
-        );
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          expect(
+            tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
+            moreOrLessEquals(25),
+          );
+        }
+        else {
+          expect(
+            tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
+            moreOrLessEquals(27.944444444444457),
+          );
+        }
         // Need to bring it to 100 * 0.1 to reset to inactive.
         expect(
           CupertinoSliverRefreshControl.state(tester.element(find.byType(LayoutBuilder))),
           RefreshIndicatorMode.done,
         );
 
-        await gesture.moveBy(const Offset(0.0, -35.0));
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          await gesture.moveBy(const Offset(0.0, -15.0));
+        }
+        else {
+          await gesture.moveBy(const Offset(0.0, -35.0));
+        }
         await tester.pump();
-        expect(
-          tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
-          moreOrLessEquals(9.313890708161875),
-        );
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          expect(
+            tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
+            moreOrLessEquals(10),
+          );
+        }
+        else {
+          expect(
+            tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
+            moreOrLessEquals(9.313890708161875),
+          );
+        }
         expect(
           CupertinoSliverRefreshControl.state(tester.element(find.byType(LayoutBuilder))),
           RefreshIndicatorMode.inactive,
@@ -1222,11 +1386,15 @@ void main() {
         await tester.pump(); // Sliver scroll offset correction is applied one frame later.
 
         await gesture.moveBy(const Offset(0.0, -300.0));
+        double indicatorDestinationPosition = -145.0332383665717;
+        if (debugDefaultTargetPlatformOverride == TargetPlatform.macOS) {
+          indicatorDestinationPosition = -150.0;
+        }
         await tester.pump();
         // The refresh indicator is offscreen now.
         expect(
           tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
-          moreOrLessEquals(-145.0332383665717),
+          moreOrLessEquals(indicatorDestinationPosition),
         );
         expect(
           CupertinoSliverRefreshControl.state(tester.element(find.byType(LayoutBuilder, skipOffstage: false))),
@@ -1243,13 +1411,13 @@ void main() {
         // Nothing moved.
         expect(
           tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
-          moreOrLessEquals(-145.0332383665717),
+          moreOrLessEquals(indicatorDestinationPosition),
         );
         await tester.pump(const Duration(seconds: 2));
         // Everything stayed as is.
         expect(
           tester.getTopLeft(find.widgetWithText(SizedBox, '0')).dy,
-          moreOrLessEquals(-145.0332383665717),
+          moreOrLessEquals(indicatorDestinationPosition),
         );
       },
       variant: const TargetPlatformVariant(<TargetPlatform>{ TargetPlatform.iOS,  TargetPlatform.macOS }),
