@@ -125,15 +125,14 @@ void main() {
           // Measure the delta in rotation.
           // Check that it never grows by more than a safe amount.
           //
-          // Note that there may be multiple transitions all active at
+          // There may be multiple transitions all active at
           // the same time. We are concerned only with the closest one.
           final Iterable<RotationTransition> rotationTransitions = tester.widgetList(
             find.byType(RotationTransition),
           );
           final Iterable<double> currentRotations = rotationTransitions.map((RotationTransition t) => t.turns.value);
 
-          if (previousRotations != null && previousRotations!.isNotEmpty
-              && currentRotations != null && currentRotations.isNotEmpty
+          if (previousRotations != null && previousRotations!.isNotEmpty && currentRotations.isNotEmpty
               && previousRect != null && currentRect != null) {
             final List<double> deltas = <double>[];
             for (final double currentRotation in currentRotations) {
@@ -167,7 +166,7 @@ void main() {
         // We create the geometry listener here, but it can only be set up
         // after it is pumped into the widget tree and a tester is
         // available.
-        geometryListener = _GeometryListener();
+        geometryListener = const _GeometryListener();
         geometry = null;
         listenerState = null;
         previousRect = null;
@@ -296,6 +295,21 @@ void main() {
     expect(tester.getCenter(find.byType(FloatingActionButton)), const Offset(756.0, 572.0));
   });
 
+  testWidgets('Contained floating action button locations', (WidgetTester tester) async {
+    await tester.pumpWidget(
+      _buildFrame(
+        location: FloatingActionButtonLocation.endContained,
+        bab: const SizedBox(height: 100.0),
+        viewInsets: EdgeInsets.zero,
+      ),
+    );
+
+    // Scaffold 800x600, FAB is 56x56, BAB is 800x100, FAB's center is
+    // at the top of the BAB.
+    // Formula: scaffold height - BAB height + FAB height / 2 + BAB top & bottom margins.
+    expect(tester.getCenter(find.byType(FloatingActionButton)), const Offset(756.0, 550.0));
+ });
+
   testWidgets('Mini-start-top floating action button location', (WidgetTester tester) async {
     await tester.pumpWidget(
       MaterialApp(
@@ -303,8 +317,8 @@ void main() {
           appBar: AppBar(),
           floatingActionButton: FloatingActionButton(onPressed: () { }, mini: true),
           floatingActionButtonLocation: FloatingActionButtonLocation.miniStartTop,
-          body: Column(
-            children: const <Widget>[
+          body: const Column(
+            children: <Widget>[
               ListTile(
                 leading: CircleAvatar(),
               ),
@@ -430,6 +444,12 @@ void main() {
       expect(tester.getCenter(find.byType(FloatingActionButton)), const Offset(_rightOffsetX, _dockedOffsetY));
     });
 
+    testWidgets('endContained', (WidgetTester tester) async {
+      await tester.pumpWidget(_singleFabScaffold(FloatingActionButtonLocation.endContained));
+
+      expect(tester.getCenter(find.byType(FloatingActionButton)), const Offset(_rightOffsetX, _containedOffsetY));
+    });
+
     testWidgets('miniStartTop', (WidgetTester tester) async {
       await tester.pumpWidget(_singleFabScaffold(FloatingActionButtonLocation.miniStartTop));
 
@@ -607,10 +627,62 @@ void main() {
 
       expect(tester.binding.transientCallbackCount, 0);
     });
+
+    testWidgets('Animator can be updated', (WidgetTester tester) async {
+      FloatingActionButtonAnimator fabAnimator = FloatingActionButtonAnimator.scaling;
+      FloatingActionButtonLocation fabLocation = FloatingActionButtonLocation.startFloat;
+
+      final Duration animationDuration = kFloatingActionButtonSegue * 2;
+
+      await tester.pumpWidget(_singleFabScaffold(
+        fabLocation,
+        animator: fabAnimator,
+      ));
+
+      expect(find.byType(FloatingActionButton), findsOneWidget);
+      expect(tester.binding.transientCallbackCount, 0);
+      expect(tester.getCenter(find.byType(FloatingActionButton)).dx, 44.0);
+
+      fabLocation = FloatingActionButtonLocation.endFloat;
+      await tester.pumpWidget(_singleFabScaffold(
+        fabLocation,
+        animator: fabAnimator,
+      ));
+
+      expect(tester.getTopLeft(find.byType(FloatingActionButton)).dx, lessThan(16.0));
+
+      await tester.pump(animationDuration * 0.25);
+      expect(tester.getTopLeft(find.byType(FloatingActionButton)).dx, greaterThan(16));
+
+      await tester.pump(animationDuration * 0.25);
+      expect(tester.getCenter(find.byType(FloatingActionButton)).dx, 756.0);
+      expect(tester.getTopRight(find.byType(FloatingActionButton)).dx, lessThan(800 - 16));
+
+      await tester.pump(animationDuration * 0.25);
+      expect(tester.getTopRight(find.byType(FloatingActionButton)).dx, lessThan(800 - 16));
+
+      await tester.pump(animationDuration * 0.25);
+      expect(tester.getTopRight(find.byType(FloatingActionButton)).dx, equals(800 - 16));
+
+      fabLocation = FloatingActionButtonLocation.startFloat;
+      fabAnimator = _NoScalingFabAnimator();
+      await tester.pumpWidget(_singleFabScaffold(
+        fabLocation,
+        animator: fabAnimator,
+      ));
+
+      await tester.pump(animationDuration * 0.25);
+      expect(tester.getCenter(find.byType(FloatingActionButton)).dx, 756.0);
+      expect(tester.getTopRight(find.byType(FloatingActionButton)).dx, equals(800 - 16));
+
+      await tester.pump(animationDuration * 0.25);
+      expect(tester.getCenter(find.byType(FloatingActionButton)).dx, 44.0);
+      expect(tester.getTopLeft(find.byType(FloatingActionButton)).dx, lessThan(16.0));
+    });
   });
 
   group('Locations account for safe interactive areas', () {
-    Widget _buildTest(
+    Widget buildTest(
       FloatingActionButtonLocation location,
       MediaQueryData data,
       Key key, {
@@ -672,7 +744,7 @@ void main() {
     //  - with BottomSheet
     //  - with BottomSheet and resizeToAvoidBottomInset: false
     //  - with SnackBar
-    Future<void> _runFloatTests(
+    Future<void> runFloatTests(
       WidgetTester tester,
       FloatingActionButtonLocation location, {
       required Rect defaultRect,
@@ -686,7 +758,7 @@ void main() {
       final Key floatingActionButton = UniqueKey();
       const double bottomNavHeight = 106.0;
       // Default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(bottom: viewPadding)),
         floatingActionButton,
@@ -697,7 +769,7 @@ void main() {
         rectMoreOrLessEquals(defaultRect),
       );
       // Present keyboard and check position, should change
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -716,7 +788,7 @@ void main() {
 
       // With resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -732,7 +804,7 @@ void main() {
       );
 
       // BottomNavigationBar default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -747,7 +819,7 @@ void main() {
         rectMoreOrLessEquals(bottomNavigationBarRect),
       );
       // Present keyboard and check position, FAB position changes
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -768,7 +840,7 @@ void main() {
 
       // BottomNavigationBar with resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -786,7 +858,7 @@ void main() {
       );
 
       // BottomNavigationBar + BottomSheet default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -805,7 +877,7 @@ void main() {
         )),
       );
       // Present keyboard and check position, FAB position changes
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -827,7 +899,7 @@ void main() {
 
       // BottomNavigationBar + BottomSheet with resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -849,7 +921,7 @@ void main() {
       );
 
       // BottomSheet default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(bottom: viewPadding)),
         floatingActionButton,
@@ -861,7 +933,7 @@ void main() {
         rectMoreOrLessEquals(bottomSheetRect),
       );
       // Present keyboard and check position, bottomSheet and FAB both resize
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -878,7 +950,7 @@ void main() {
 
       // bottomSheet with resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default bottomSheet position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -895,7 +967,7 @@ void main() {
       );
 
       // SnackBar default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(bottom: viewPadding)),
         floatingActionButton,
@@ -909,7 +981,7 @@ void main() {
       );
 
       // SnackBar when resized for presented keyboard
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -934,7 +1006,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(16.0, 472.0, 72.0, 528.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(16.0, 478.0, 72.0, 534.0);
-      await _runFloatTests(
+      await runFloatTests(
         tester,
         FloatingActionButtonLocation.startFloat,
         defaultRect: defaultRect,
@@ -952,7 +1024,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(12.0, 480.0, 60.0, 528.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(12.0, 490.0, 60.0, 538.0);
-      await _runFloatTests(
+      await runFloatTests(
         tester,
         FloatingActionButtonLocation.miniStartFloat,
         defaultRect: defaultRect,
@@ -971,7 +1043,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(372.0, 472.0, 428.0, 528.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(372.0, 478.0, 428.0, 534.0);
-      await _runFloatTests(
+      await runFloatTests(
         tester,
         FloatingActionButtonLocation.centerFloat,
         defaultRect: defaultRect,
@@ -989,7 +1061,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(376.0, 480.0, 424.0, 528.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(376.0, 490.0, 424.0, 538.0);
-      await _runFloatTests(
+      await runFloatTests(
         tester,
         FloatingActionButtonLocation.miniCenterFloat,
         defaultRect: defaultRect,
@@ -1008,7 +1080,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(728.0, 472.0, 784.0, 528.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(728.0, 478.0, 784.0, 534.0);
-      await _runFloatTests(
+      await runFloatTests(
         tester,
         FloatingActionButtonLocation.endFloat,
         defaultRect: defaultRect,
@@ -1026,7 +1098,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(740.0, 480.0, 788.0, 528.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(740.0, 490.0, 788.0, 538.0);
-      await _runFloatTests(
+      await runFloatTests(
         tester,
         FloatingActionButtonLocation.miniEndFloat,
         defaultRect: defaultRect,
@@ -1047,7 +1119,7 @@ void main() {
     //  - docked with BottomNavigationBar & BottomSheet
     //  - docked with BottomNavigationBar & BottomSheet, resizeToAvoidBottomInset: false
     //  - with SnackBar
-    Future<void> _runDockedTests(
+    Future<void> runDockedTests(
       WidgetTester tester,
       FloatingActionButtonLocation location, {
       required Rect defaultRect,
@@ -1063,7 +1135,7 @@ void main() {
       final Key floatingActionButton = UniqueKey();
       final double fabHeight = mini ? 48.0 : 56.0;
       // Default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(bottom: viewPadding)),
         floatingActionButton,
@@ -1074,7 +1146,7 @@ void main() {
         rectMoreOrLessEquals(defaultRect),
       );
       // Present keyboard and check position, should change
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -1098,7 +1170,7 @@ void main() {
 
       // With resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -1114,7 +1186,7 @@ void main() {
       );
 
       // BottomNavigationBar default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1129,7 +1201,7 @@ void main() {
         rectMoreOrLessEquals(bottomNavigationBarRect),
       );
       // Present keyboard and check position, FAB position changes
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1155,7 +1227,7 @@ void main() {
 
       // BottomNavigationBar with resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1173,7 +1245,7 @@ void main() {
       );
 
       // BottomNavigationBar + BottomSheet default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1189,7 +1261,7 @@ void main() {
         rectMoreOrLessEquals(bottomSheetRect),
       );
       // Present keyboard and check position, FAB position changes
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1216,7 +1288,7 @@ void main() {
 
       // BottomNavigationBar + BottomSheet with resizeToAvoidBottomInset: false
       // With keyboard presented, should maintain default position
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1235,7 +1307,7 @@ void main() {
       );
 
       // SnackBar default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(bottom: viewPadding)),
         floatingActionButton,
@@ -1249,7 +1321,7 @@ void main() {
       );
 
       // SnackBar with BottomNavigationBar
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           padding: EdgeInsets.only(bottom: viewPadding),
@@ -1267,7 +1339,7 @@ void main() {
       );
 
       // SnackBar when resized for presented keyboard
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(
           viewPadding: EdgeInsets.only(bottom: viewPadding),
@@ -1297,7 +1369,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(16.0, 366.0, 72.0, 422.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(16.0, 486.0, 72.0, 542.0);
-      await _runDockedTests(
+      await runDockedTests(
         tester,
         FloatingActionButtonLocation.startDocked,
         defaultRect: defaultRect,
@@ -1315,7 +1387,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(12.0, 370.0, 60.0, 418.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(12.0, 494.0, 60.0, 542.0);
-      await _runDockedTests(
+      await runDockedTests(
         tester,
         FloatingActionButtonLocation.miniStartDocked,
         defaultRect: defaultRect,
@@ -1334,7 +1406,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(372.0, 366.0, 428.0, 422.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(372.0, 486.0, 428.0, 542.0);
-      await _runDockedTests(
+      await runDockedTests(
         tester,
         FloatingActionButtonLocation.centerDocked,
         defaultRect: defaultRect,
@@ -1352,7 +1424,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(376.0, 370.0, 424.0, 418.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(376.0, 494.0, 424.0, 542.0);
-      await _runDockedTests(
+      await runDockedTests(
         tester,
         FloatingActionButtonLocation.miniCenterDocked,
         defaultRect: defaultRect,
@@ -1371,7 +1443,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(728.0, 366.0, 784.0, 422.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(728.0, 486.0, 784.0, 542.0);
-      await _runDockedTests(
+      await runDockedTests(
         tester,
         FloatingActionButtonLocation.endDocked,
         defaultRect: defaultRect,
@@ -1389,7 +1461,7 @@ void main() {
       const Rect bottomSheetRect = Rect.fromLTRB(740.0, 370.0, 788.0, 418.0);
       // Positioned relative to SnackBar
       const Rect snackBarRect = Rect.fromLTRB(740.0, 494.0, 788.0, 542.0);
-      await _runDockedTests(
+      await runDockedTests(
         tester,
         FloatingActionButtonLocation.miniEndDocked,
         defaultRect: defaultRect,
@@ -1403,7 +1475,7 @@ void main() {
     // Test top locations, for each (6):
     //  - Default
     //  - with an AppBar
-    Future<void> _runTopTests(
+    Future<void> runTopTests(
       WidgetTester tester,
       FloatingActionButtonLocation location, {
       required Rect defaultRect,
@@ -1413,7 +1485,7 @@ void main() {
       const double viewPadding = 50.0;
       final Key floatingActionButton = UniqueKey();
       // Default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(top: viewPadding)),
         floatingActionButton,
@@ -1425,7 +1497,7 @@ void main() {
       );
 
       // AppBar default
-      await tester.pumpWidget(_buildTest(
+      await tester.pumpWidget(buildTest(
         location,
         const MediaQueryData(viewPadding: EdgeInsets.only(top: viewPadding)),
         floatingActionButton,
@@ -1442,7 +1514,7 @@ void main() {
       const Rect defaultRect = Rect.fromLTRB(16.0, 50.0, 72.0, 106.0);
       // Positioned relative to AppBar
       const Rect appBarRect = Rect.fromLTRB(16.0, 28.0, 72.0, 84.0);
-      await _runTopTests(
+      await runTopTests(
         tester,
         FloatingActionButtonLocation.startTop,
         defaultRect: defaultRect,
@@ -1454,7 +1526,7 @@ void main() {
       const Rect defaultRect = Rect.fromLTRB(12.0, 50.0, 60.0, 98.0);
       // Positioned relative to AppBar
       const Rect appBarRect = Rect.fromLTRB(12.0, 32.0, 60.0, 80.0);
-      await _runTopTests(
+      await runTopTests(
         tester,
         FloatingActionButtonLocation.miniStartTop,
         defaultRect: defaultRect,
@@ -1467,7 +1539,7 @@ void main() {
       const Rect defaultRect = Rect.fromLTRB(372.0, 50.0, 428.0, 106.0);
       // Positioned relative to AppBar
       const Rect appBarRect = Rect.fromLTRB(372.0, 28.0, 428.0, 84.0);
-      await _runTopTests(
+      await runTopTests(
         tester,
         FloatingActionButtonLocation.centerTop,
         defaultRect: defaultRect,
@@ -1479,7 +1551,7 @@ void main() {
       const Rect defaultRect = Rect.fromLTRB(376.0, 50.0, 424.0, 98.0);
       // Positioned relative to AppBar
       const Rect appBarRect = Rect.fromLTRB(376.0, 32.0, 424.0, 80.0);
-      await _runTopTests(
+      await runTopTests(
         tester,
         FloatingActionButtonLocation.miniCenterTop,
         defaultRect: defaultRect,
@@ -1492,7 +1564,7 @@ void main() {
       const Rect defaultRect = Rect.fromLTRB(728.0, 50.0, 784.0, 106.0);
       // Positioned relative to AppBar
       const Rect appBarRect = Rect.fromLTRB(728.0, 28.0, 784.0, 84.0);
-      await _runTopTests(
+      await runTopTests(
         tester,
         FloatingActionButtonLocation.endTop,
         defaultRect: defaultRect,
@@ -1504,7 +1576,7 @@ void main() {
       const Rect defaultRect = Rect.fromLTRB(740.0, 50.0, 788.0, 98.0);
       // Positioned relative to AppBar
       const Rect appBarRect = Rect.fromLTRB(740.0, 32.0, 788.0, 80.0);
-      await _runTopTests(
+      await runTopTests(
         tester,
         FloatingActionButtonLocation.miniEndTop,
         defaultRect: defaultRect,
@@ -1516,6 +1588,8 @@ void main() {
 }
 
 class _GeometryListener extends StatefulWidget {
+  const _GeometryListener();
+
   @override
   State createState() => _GeometryListenerState();
 }
@@ -1536,11 +1610,13 @@ class _GeometryListenerState extends State<_GeometryListener> {
   void didChangeDependencies() {
     super.didChangeDependencies();
     final ValueListenable<ScaffoldGeometry> newListenable = Scaffold.geometryOf(context);
-    if (geometryListenable == newListenable)
+    if (geometryListenable == newListenable) {
       return;
+    }
 
-    if (geometryListenable != null)
+    if (geometryListenable != null) {
       geometryListenable!.removeListener(onGeometryChanged);
+    }
 
     geometryListenable = newListenable;
     geometryListenable!.addListener(onGeometryChanged);
@@ -1561,6 +1637,7 @@ const double _miniRightOffsetX = _rightOffsetX + kMiniButtonOffsetAdjustment;
 const double _topOffsetY = 56.0;
 const double _floatOffsetY = 500.0;
 const double _dockedOffsetY = 544.0;
+const double _containedOffsetY = 544.0 + 56.0 / 2;
 const double _miniFloatOffsetY = _floatOffsetY + kMiniButtonOffsetAdjustment;
 
 Widget _singleFabScaffold(
@@ -1661,16 +1738,13 @@ class _StartTopFloatingActionButtonLocation extends FloatingActionButtonLocation
   @override
   Offset getOffset(ScaffoldPrelayoutGeometry scaffoldGeometry) {
     double fabX;
-    assert(scaffoldGeometry.textDirection != null);
     switch (scaffoldGeometry.textDirection) {
       case TextDirection.rtl:
         final double startPadding = kFloatingActionButtonMargin + scaffoldGeometry.minInsets.right;
         fabX = scaffoldGeometry.scaffoldSize.width - scaffoldGeometry.floatingActionButtonSize.width - startPadding;
-        break;
       case TextDirection.ltr:
         final double startPadding = kFloatingActionButtonMargin + scaffoldGeometry.minInsets.left;
         fabX = startPadding;
-        break;
     }
     final double fabY = scaffoldGeometry.contentTop - (scaffoldGeometry.floatingActionButtonSize.height / 2.0);
     return Offset(fabX, fabY);
@@ -1713,3 +1787,20 @@ class _LinearMovementFabAnimator extends FloatingActionButtonAnimator {
     return const AlwaysStoppedAnimation<double>(1.0);
   }
 }
+
+class _NoScalingFabAnimator extends FloatingActionButtonAnimator {
+  @override
+  Offset getOffset({required Offset begin, required Offset end, required double progress}) {
+    return progress < 0.5 ? begin : end;
+  }
+
+  @override
+  Animation<double> getScaleAnimation({required Animation<double> parent}) {
+    return const AlwaysStoppedAnimation<double>(1.0);
+  }
+
+  @override
+  Animation<double> getRotationAnimation({required Animation<double> parent}) {
+    return const AlwaysStoppedAnimation<double>(1.0);
+  }
+}
