@@ -207,7 +207,7 @@ void main() {
     expect(tester.binding.renderView.toStringDeep(minLevel: DiagnosticLevel.info), equalsIgnoringHashCodes(
       'RenderView#00000\n'
       ' │ debug mode enabled - ${Platform.operatingSystem}\n'
-      ' │ window size: Size(2400.0, 1800.0) (in physical pixels)\n'
+      ' │ view size: Size(2400.0, 1800.0) (in physical pixels)\n'
       ' │ device pixel ratio: 3.0 (physical pixels per logical pixel)\n'
       ' │ configuration: Size(800.0, 600.0) at 3.0x (in logical pixels)\n'
       ' │\n'
@@ -303,7 +303,7 @@ void main() {
       '                     │   ScrollDirection.idle)\n'
       '                     │ anchor: 0.0\n'
       '                     │\n'
-      '                     └─center child: RenderSliverFixedExtentList#00000 relayoutBoundary=up1\n'
+      '                     └─center child: RenderSliverPadding#00000 relayoutBoundary=up1\n'
       '                       │ parentData: paintOffset=Offset(0.0, 0.0) (can use size)\n'
       '                       │ constraints: SliverConstraints(AxisDirection.down,\n'
       '                       │   GrowthDirection.forward, ScrollDirection.idle, scrollOffset:\n'
@@ -314,49 +314,63 @@ void main() {
       '                       │ geometry: SliverGeometry(scrollExtent: 40000.0, paintExtent:\n'
       '                       │   600.0, maxPaintExtent: 40000.0, hasVisualOverflow: true,\n'
       '                       │   cacheExtent: 850.0)\n'
-      '                       │ currently live children: 0 to 2\n'
+      '                       │ padding: EdgeInsets.zero\n'
+      '                       │ textDirection: ltr\n'
       '                       │\n'
-      '                       ├─child with index 0: RenderLimitedBox#00000\n'
-      '                       │ │ parentData: index=0; layoutOffset=0.0\n'
-      '                       │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │ │ size: Size(800.0, 400.0)\n'
-      '                       │ │ maxWidth: 400.0\n'
-      '                       │ │ maxHeight: 400.0\n'
-      '                       │ │\n'
-      '                       │ └─child: RenderCustomPaint#00000\n'
-      '                       │     parentData: <none> (can use size)\n'
-      '                       │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │     size: Size(800.0, 400.0)\n'
-      '                       │     painter: _PlaceholderPainter#00000()\n'
-      '                       │     preferredSize: Size(Infinity, Infinity)\n'
-      '                       │\n'
-      '                       ├─child with index 1: RenderLimitedBox#00000\n'                                     // <----- no dashed line starts here
-      '                       │ │ parentData: index=1; layoutOffset=400.0\n'
-      '                       │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │ │ size: Size(800.0, 400.0)\n'
-      '                       │ │ maxWidth: 400.0\n'
-      '                       │ │ maxHeight: 400.0\n'
-      '                       │ │\n'
-      '                       │ └─child: RenderCustomPaint#00000\n'
-      '                       │     parentData: <none> (can use size)\n'
-      '                       │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │     size: Size(800.0, 400.0)\n'
-      '                       │     painter: _PlaceholderPainter#00000()\n'
-      '                       │     preferredSize: Size(Infinity, Infinity)\n'
-      '                       │\n'
-      '                       └─child with index 2: RenderLimitedBox#00000 NEEDS-PAINT\n'
-      '                         │ parentData: index=2; layoutOffset=800.0\n'
-      '                         │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                         │ size: Size(800.0, 400.0)\n'
-      '                         │ maxWidth: 400.0\n'
-      '                         │ maxHeight: 400.0\n'
+      '                       └─child: RenderSliverFixedExtentList#00000 relayoutBoundary=up2\n'
+      '                         │ parentData: paintOffset=Offset(0.0, 0.0) (can use size)\n'
+      '                         │ constraints: SliverConstraints(AxisDirection.down,\n'
+      '                         │   GrowthDirection.forward, ScrollDirection.idle, scrollOffset:\n'
+      '                         │   0.0, remainingPaintExtent: 600.0, crossAxisExtent: 800.0,\n'
+      '                         │   crossAxisDirection: AxisDirection.right,\n'
+      '                         │   viewportMainAxisExtent: 600.0, remainingCacheExtent: 850.0,\n'
+      '                         │   cacheOrigin: 0.0)\n'
+      '                         │ geometry: SliverGeometry(scrollExtent: 40000.0, paintExtent:\n'
+      '                         │   600.0, maxPaintExtent: 40000.0, hasVisualOverflow: true,\n'
+      '                         │   cacheExtent: 850.0)\n'
+      '                         │ currently live children: 0 to 2\n'
       '                         │\n'
-      '                         └─child: RenderCustomPaint#00000 NEEDS-PAINT\n'
-      '                             parentData: <none> (can use size)\n'
-      '                             constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                             size: Size(800.0, 400.0)\n'
-      '                             painter: _PlaceholderPainter#00000()\n'
-      '                             preferredSize: Size(Infinity, Infinity)\n',
+      '                         ├─child with index 0: RenderLimitedBox#00000\n'
+      '                         │ │ parentData: index=0; layoutOffset=0.0\n'
+      '                         │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │ │ size: Size(800.0, 400.0)\n'
+      '                         │ │ maxWidth: 400.0\n'
+      '                         │ │ maxHeight: 400.0\n'
+      '                         │ │\n'
+      '                         │ └─child: RenderCustomPaint#00000\n'
+      '                         │     parentData: <none> (can use size)\n'
+      '                         │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │     size: Size(800.0, 400.0)\n'
+      '                         │     painter: _PlaceholderPainter#00000()\n'
+      '                         │     preferredSize: Size(Infinity, Infinity)\n'
+      '                         │\n'
+      '                         ├─child with index 1: RenderLimitedBox#00000\n'                                      // <----- no dashed line starts here
+      '                         │ │ parentData: index=1; layoutOffset=400.0\n'
+      '                         │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │ │ size: Size(800.0, 400.0)\n'
+      '                         │ │ maxWidth: 400.0\n'
+      '                         │ │ maxHeight: 400.0\n'
+      '                         │ │\n'
+      '                         │ └─child: RenderCustomPaint#00000\n'
+      '                         │     parentData: <none> (can use size)\n'
+      '                         │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │     size: Size(800.0, 400.0)\n'
+      '                         │     painter: _PlaceholderPainter#00000()\n'
+      '                         │     preferredSize: Size(Infinity, Infinity)\n'
+      '                         │\n'
+      '                         └─child with index 2: RenderLimitedBox#00000 NEEDS-PAINT\n'
+      '                           │ parentData: index=2; layoutOffset=800.0\n'
+      '                           │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                           │ size: Size(800.0, 400.0)\n'
+      '                           │ maxWidth: 400.0\n'
+      '                           │ maxHeight: 400.0\n'
+      '                           │\n'
+      '                           └─child: RenderCustomPaint#00000 NEEDS-PAINT\n'
+      '                               parentData: <none> (can use size)\n'
+      '                               constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                               size: Size(800.0, 400.0)\n'
+      '                               painter: _PlaceholderPainter#00000()\n'
+      '                               preferredSize: Size(Infinity, Infinity)\n',
     ));
     const GlobalObjectKey<_LeafState>(0).currentState!.setKeepAlive(true);
     await tester.drag(find.byType(ListView), const Offset(0.0, -1000.0));
@@ -367,7 +381,7 @@ void main() {
     expect(tester.binding.renderView.toStringDeep(minLevel: DiagnosticLevel.info), equalsIgnoringHashCodes(
       'RenderView#00000\n'
       ' │ debug mode enabled - ${Platform.operatingSystem}\n'
-      ' │ window size: Size(2400.0, 1800.0) (in physical pixels)\n'
+      ' │ view size: Size(2400.0, 1800.0) (in physical pixels)\n'
       ' │ device pixel ratio: 3.0 (physical pixels per logical pixel)\n'
       ' │ configuration: Size(800.0, 600.0) at 3.0x (in logical pixels)\n'
       ' │\n'
@@ -463,7 +477,7 @@ void main() {
       '                     │   ScrollDirection.idle)\n'
       '                     │ anchor: 0.0\n'
       '                     │\n'
-      '                     └─center child: RenderSliverFixedExtentList#00000 relayoutBoundary=up1\n'
+      '                     └─center child: RenderSliverPadding#00000 relayoutBoundary=up1\n'
       '                       │ parentData: paintOffset=Offset(0.0, 0.0) (can use size)\n'
       '                       │ constraints: SliverConstraints(AxisDirection.down,\n'
       '                       │   GrowthDirection.forward, ScrollDirection.idle, scrollOffset:\n'
@@ -474,92 +488,105 @@ void main() {
       '                       │ geometry: SliverGeometry(scrollExtent: 40000.0, paintExtent:\n'
       '                       │   600.0, maxPaintExtent: 40000.0, hasVisualOverflow: true,\n'
       '                       │   cacheExtent: 1100.0)\n'
-      '                       │ currently live children: 4 to 7\n'
+      '                       │ padding: EdgeInsets.zero\n'
+      '                       │ textDirection: ltr\n'
       '                       │\n'
-      '                       ├─child with index 4: RenderLimitedBox#00000 NEEDS-PAINT\n'
-      '                       │ │ parentData: index=4; layoutOffset=1600.0\n'
-      '                       │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │ │ size: Size(800.0, 400.0)\n'
-      '                       │ │ maxWidth: 400.0\n'
-      '                       │ │ maxHeight: 400.0\n'
-      '                       │ │\n'
-      '                       │ └─child: RenderCustomPaint#00000 NEEDS-PAINT\n'
-      '                       │     parentData: <none> (can use size)\n'
-      '                       │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │     size: Size(800.0, 400.0)\n'
-      '                       │     painter: _PlaceholderPainter#00000()\n'
-      '                       │     preferredSize: Size(Infinity, Infinity)\n'
-      '                       │\n'
-      '                       ├─child with index 5: RenderLimitedBox#00000\n'                                     // <----- this is index 5, not 0
-      '                       │ │ parentData: index=5; layoutOffset=2000.0\n'
-      '                       │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │ │ size: Size(800.0, 400.0)\n'
-      '                       │ │ maxWidth: 400.0\n'
-      '                       │ │ maxHeight: 400.0\n'
-      '                       │ │\n'
-      '                       │ └─child: RenderCustomPaint#00000\n'
-      '                       │     parentData: <none> (can use size)\n'
-      '                       │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │     size: Size(800.0, 400.0)\n'
-      '                       │     painter: _PlaceholderPainter#00000()\n'
-      '                       │     preferredSize: Size(Infinity, Infinity)\n'
-      '                       │\n'
-      '                       ├─child with index 6: RenderLimitedBox#00000\n'
-      '                       │ │ parentData: index=6; layoutOffset=2400.0\n'
-      '                       │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │ │ size: Size(800.0, 400.0)\n'
-      '                       │ │ maxWidth: 400.0\n'
-      '                       │ │ maxHeight: 400.0\n'
-      '                       │ │\n'
-      '                       │ └─child: RenderCustomPaint#00000\n'
-      '                       │     parentData: <none> (can use size)\n'
-      '                       │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       │     size: Size(800.0, 400.0)\n'
-      '                       │     painter: _PlaceholderPainter#00000()\n'
-      '                       │     preferredSize: Size(Infinity, Infinity)\n'
-      '                       │\n'
-      '                       ├─child with index 7: RenderLimitedBox#00000 NEEDS-PAINT\n'
-      '                       ╎ │ parentData: index=7; layoutOffset=2800.0\n'
-      '                       ╎ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       ╎ │ size: Size(800.0, 400.0)\n'
-      '                       ╎ │ maxWidth: 400.0\n'
-      '                       ╎ │ maxHeight: 400.0\n'
-      '                       ╎ │\n'
-      '                       ╎ └─child: RenderCustomPaint#00000 NEEDS-PAINT\n'
-      '                       ╎     parentData: <none> (can use size)\n'
-      '                       ╎     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       ╎     size: Size(800.0, 400.0)\n'
-      '                       ╎     painter: _PlaceholderPainter#00000()\n'
-      '                       ╎     preferredSize: Size(Infinity, Infinity)\n'
-      '                       ╎\n'
-      '                       ╎╌child with index 0 (kept alive but not laid out): RenderLimitedBox#00000\n'               // <----- this one is index 0 and is marked as being kept alive but not laid out
-      '                       ╎ │ parentData: index=0; keepAlive; layoutOffset=0.0\n'
-      '                       ╎ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       ╎ │ size: Size(800.0, 400.0)\n'
-      '                       ╎ │ maxWidth: 400.0\n'
-      '                       ╎ │ maxHeight: 400.0\n'
-      '                       ╎ │\n'
-      '                       ╎ └─child: RenderCustomPaint#00000\n'
-      '                       ╎     parentData: <none> (can use size)\n'
-      '                       ╎     constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                       ╎     size: Size(800.0, 400.0)\n'
-      '                       ╎     painter: _PlaceholderPainter#00000()\n'
-      '                       ╎     preferredSize: Size(Infinity, Infinity)\n'
-      '                       ╎\n'                                                                                // <----- dashed line ends here
-      '                       └╌child with index 3 (kept alive but not laid out): RenderLimitedBox#00000\n'
-      '                         │ parentData: index=3; keepAlive; layoutOffset=1200.0\n'
-      '                         │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                         │ size: Size(800.0, 400.0)\n'
-      '                         │ maxWidth: 400.0\n'
-      '                         │ maxHeight: 400.0\n'
+      '                       └─child: RenderSliverFixedExtentList#00000 relayoutBoundary=up2\n'
+      '                         │ parentData: paintOffset=Offset(0.0, 0.0) (can use size)\n'
+      '                         │ constraints: SliverConstraints(AxisDirection.down,\n'
+      '                         │   GrowthDirection.forward, ScrollDirection.idle, scrollOffset:\n'
+      '                         │   2000.0, remainingPaintExtent: 600.0, crossAxisExtent: 800.0,\n'
+      '                         │   crossAxisDirection: AxisDirection.right,\n'
+      '                         │   viewportMainAxisExtent: 600.0, remainingCacheExtent: 1100.0,\n'
+      '                         │   cacheOrigin: -250.0)\n'
+      '                         │ geometry: SliverGeometry(scrollExtent: 40000.0, paintExtent:\n'
+      '                         │   600.0, maxPaintExtent: 40000.0, hasVisualOverflow: true,\n'
+      '                         │   cacheExtent: 1100.0)\n'
+      '                         │ currently live children: 4 to 7\n'
       '                         │\n'
-      '                         └─child: RenderCustomPaint#00000\n'
-      '                             parentData: <none> (can use size)\n'
-      '                             constraints: BoxConstraints(w=800.0, h=400.0)\n'
-      '                             size: Size(800.0, 400.0)\n'
-      '                             painter: _PlaceholderPainter#00000()\n'
-      '                             preferredSize: Size(Infinity, Infinity)\n',
+      '                         ├─child with index 4: RenderLimitedBox#00000 NEEDS-PAINT\n'
+      '                         │ │ parentData: index=4; layoutOffset=1600.0\n'
+      '                         │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │ │ size: Size(800.0, 400.0)\n'
+      '                         │ │ maxWidth: 400.0\n'
+      '                         │ │ maxHeight: 400.0\n'
+      '                         │ │\n'
+      '                         │ └─child: RenderCustomPaint#00000 NEEDS-PAINT\n'
+      '                         │     parentData: <none> (can use size)\n'
+      '                         │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │     size: Size(800.0, 400.0)\n'
+      '                         │     painter: _PlaceholderPainter#00000()\n'
+      '                         │     preferredSize: Size(Infinity, Infinity)\n'
+      '                         │\n'
+      '                         ├─child with index 5: RenderLimitedBox#00000\n'                                     // <----- this is index 5, not 0
+      '                         │ │ parentData: index=5; layoutOffset=2000.0\n'
+      '                         │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │ │ size: Size(800.0, 400.0)\n'
+      '                         │ │ maxWidth: 400.0\n'
+      '                         │ │ maxHeight: 400.0\n'
+      '                         │ │\n'
+      '                         │ └─child: RenderCustomPaint#00000\n'
+      '                         │     parentData: <none> (can use size)\n'
+      '                         │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │     size: Size(800.0, 400.0)\n'
+      '                         │     painter: _PlaceholderPainter#00000()\n'
+      '                         │     preferredSize: Size(Infinity, Infinity)\n'
+      '                         │\n'
+      '                         ├─child with index 6: RenderLimitedBox#00000\n'
+      '                         │ │ parentData: index=6; layoutOffset=2400.0\n'
+      '                         │ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │ │ size: Size(800.0, 400.0)\n'
+      '                         │ │ maxWidth: 400.0\n'
+      '                         │ │ maxHeight: 400.0\n'
+      '                         │ │\n'
+      '                         │ └─child: RenderCustomPaint#00000\n'
+      '                         │     parentData: <none> (can use size)\n'
+      '                         │     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         │     size: Size(800.0, 400.0)\n'
+      '                         │     painter: _PlaceholderPainter#00000()\n'
+      '                         │     preferredSize: Size(Infinity, Infinity)\n'
+      '                         │\n'
+      '                         ├─child with index 7: RenderLimitedBox#00000 NEEDS-PAINT\n'
+      '                         ╎ │ parentData: index=7; layoutOffset=2800.0\n'
+      '                         ╎ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         ╎ │ size: Size(800.0, 400.0)\n'
+      '                         ╎ │ maxWidth: 400.0\n'
+      '                         ╎ │ maxHeight: 400.0\n'
+      '                         ╎ │\n'
+      '                         ╎ └─child: RenderCustomPaint#00000 NEEDS-PAINT\n'
+      '                         ╎     parentData: <none> (can use size)\n'
+      '                         ╎     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         ╎     size: Size(800.0, 400.0)\n'
+      '                         ╎     painter: _PlaceholderPainter#00000()\n'
+      '                         ╎     preferredSize: Size(Infinity, Infinity)\n'
+      '                         ╎\n'
+      '                         ╎╌child with index 0 (kept alive but not laid out): RenderLimitedBox#00000\n'               // <----- this one is index 0 and is marked as being kept alive but not laid out
+      '                         ╎ │ parentData: index=0; keepAlive; layoutOffset=0.0\n'
+      '                         ╎ │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         ╎ │ size: Size(800.0, 400.0)\n'
+      '                         ╎ │ maxWidth: 400.0\n'
+      '                         ╎ │ maxHeight: 400.0\n'
+      '                         ╎ │\n'
+      '                         ╎ └─child: RenderCustomPaint#00000\n'
+      '                         ╎     parentData: <none> (can use size)\n'
+      '                         ╎     constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                         ╎     size: Size(800.0, 400.0)\n'
+      '                         ╎     painter: _PlaceholderPainter#00000()\n'
+      '                         ╎     preferredSize: Size(Infinity, Infinity)\n'
+      '                         ╎\n'
+      '                         └╌child with index 3 (kept alive but not laid out): RenderLimitedBox#00000\n'
+      '                           │ parentData: index=3; keepAlive; layoutOffset=1200.0\n'
+      '                           │ constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                           │ size: Size(800.0, 400.0)\n'
+      '                           │ maxWidth: 400.0\n'
+      '                           │ maxHeight: 400.0\n'
+      '                           │\n'
+      '                           └─child: RenderCustomPaint#00000\n'
+      '                               parentData: <none> (can use size)\n'
+      '                               constraints: BoxConstraints(w=800.0, h=400.0)\n'
+      '                               size: Size(800.0, 400.0)\n'
+      '                               painter: _PlaceholderPainter#00000()\n'
+      '                               preferredSize: Size(Infinity, Infinity)\n',
     ));
   }, skip: kIsWeb); // https://github.com/flutter/flutter/issues/87876
-
 }
