@@ -2,17 +2,54 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import 'dart:ui' show Brightness, DisplayFeature, DisplayFeatureState, DisplayFeatureType, GestureSettings, ViewConfiguration;
+import 'dart:ui' show Brightness, DisplayFeature, DisplayFeatureState, DisplayFeatureType, GestureSettings;
 
 import 'package:flutter/gestures.dart';
-import 'package:flutter/widgets.dart';
+import 'package:flutter/material.dart';
 import 'package:flutter_test/flutter_test.dart';
 
+class _MediaQueryAspectCase {
+  const _MediaQueryAspectCase(this.method, this.data);
+  final Function(BuildContext) method;
+  final MediaQueryData data;
+}
+
+class _MediaQueryAspectVariant extends TestVariant<_MediaQueryAspectCase> {
+  _MediaQueryAspectVariant({
+    required this.values
+  });
+
+  @override
+  final List<_MediaQueryAspectCase> values;
+
+  static _MediaQueryAspectCase? aspect;
+
+  @override
+  String describeValue(_MediaQueryAspectCase value) {
+    return value.method.toString();
+  }
+
+  @override
+  Future<_MediaQueryAspectCase?> setUp(_MediaQueryAspectCase value) async {
+    final _MediaQueryAspectCase? oldAspect = aspect;
+    aspect = value;
+    return oldAspect;
+  }
+
+  @override
+  Future<void> tearDown(_MediaQueryAspectCase value, _MediaQueryAspectCase? memento) async {
+    aspect = memento;
+  }
+}
+
 void main() {
   testWidgets('MediaQuery does not have a default', (WidgetTester tester) async {
     bool tested = false;
-    await tester.pumpWidget(
-      Builder(
+    // Cannot use tester.pumpWidget here because it wraps the widget in a View,
+    // which introduces a MediaQuery ancestor.
+    await pumpWidgetWithoutViewWrapper(
+      tester: tester,
+      widget: Builder(
         builder: (BuildContext context) {
           tested = true;
           MediaQuery.of(context); // should throw
@@ -29,20 +66,24 @@ void main() {
     expect(error.diagnostics.last, isA<ErrorHint>());
     expect(
       error.toStringDeep(),
-      equalsIgnoringHashCodes(
+      startsWith(
         'FlutterError\n'
         '   No MediaQuery widget ancestor found.\n'
         '   Builder widgets require a MediaQuery widget ancestor.\n'
         '   The specific widget that could not find a MediaQuery ancestor\n'
         '   was:\n'
         '     Builder\n'
-        '   The ownership chain for the affected widget is: "Builder ←\n'
-        '     [root]"\n'
+        '   The ownership chain for the affected widget is: "Builder ←', // Full ownership chain omitted, not relevant for test.
+      ),
+    );
+    expect(
+      error.toStringDeep(),
+      endsWith(
+        '[root]"\n' // End of ownership chain.
         '   No MediaQuery ancestor could be found starting from the context\n'
-        '   that was passed to MediaQuery.of(). This can happen because you\n'
-        '   have not added a WidgetsApp, CupertinoApp, or MaterialApp widget\n'
-        '   (those widgets introduce a MediaQuery), or it can happen if the\n'
-        '   context you use comes from a widget above those widgets.\n',
+        '   that was passed to MediaQuery.of(). This can happen because the\n'
+        '   context used is not a descendant of a View widget, which\n'
+        '   introduces a MediaQuery.\n'
       ),
     );
   });
@@ -69,8 +110,11 @@ void main() {
 
   testWidgets('MediaQuery.maybeOf defaults to null', (WidgetTester tester) async {
     bool tested = false;
-    await tester.pumpWidget(
-      Builder(
+    // Cannot use tester.pumpWidget here because it wraps the widget in a View,
+    // which introduces a MediaQuery ancestor.
+    await pumpWidgetWithoutViewWrapper(
+      tester: tester,
+      widget: Builder(
         builder: (BuildContext context) {
           final MediaQueryData? data = MediaQuery.maybeOf(context);
           expect(data, isNull);
@@ -100,11 +144,11 @@ void main() {
     expect(tested, isTrue);
   });
 
-  testWidgets('MediaQueryData.fromWindow is sane', (WidgetTester tester) async {
-    final MediaQueryData data = MediaQueryData.fromWindow(WidgetsBinding.instance.window);
+  testWidgets('MediaQueryData.fromView is sane', (WidgetTester tester) async {
+    final MediaQueryData data = MediaQueryData.fromView(tester.view);
     expect(data, hasOneLineDescription);
     expect(data.hashCode, equals(data.copyWith().hashCode));
-    expect(data.size, equals(WidgetsBinding.instance.window.physicalSize / WidgetsBinding.instance.window.devicePixelRatio));
+    expect(data.size, equals(tester.view.physicalSize / tester.view.devicePixelRatio));
     expect(data.accessibleNavigation, false);
     expect(data.invertColors, false);
     expect(data.disableAnimations, false);
@@ -115,8 +159,322 @@ void main() {
     expect(data.displayFeatures, isEmpty);
   });
 
+  testWidgets('MediaQueryData.fromView uses platformData if provided', (WidgetTester tester) async {
+    const MediaQueryData platformData = MediaQueryData(
+      textScaleFactor: 1234,
+      platformBrightness: Brightness.dark,
+      accessibleNavigation: true,
+      invertColors: true,
+      disableAnimations: true,
+      boldText: true,
+      highContrast: true,
+      alwaysUse24HourFormat: true,
+      navigationMode: NavigationMode.directional,
+    );
+
+    final MediaQueryData data = MediaQueryData.fromView(tester.view, platformData: platformData);
+    expect(data, hasOneLineDescription);
+    expect(data.hashCode, data.copyWith().hashCode);
+    expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
+    expect(data.devicePixelRatio, tester.view.devicePixelRatio);
+    expect(data.textScaleFactor, platformData.textScaleFactor);
+    expect(data.platformBrightness, platformData.platformBrightness);
+    expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
+    expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
+    expect(data.viewInsets, EdgeInsets.fromViewPadding(tester.view.viewInsets, tester.view.devicePixelRatio));
+    expect(data.systemGestureInsets, EdgeInsets.fromViewPadding(tester.view.systemGestureInsets, tester.view.devicePixelRatio));
+    expect(data.accessibleNavigation, platformData.accessibleNavigation);
+    expect(data.invertColors, platformData.invertColors);
+    expect(data.disableAnimations, platformData.disableAnimations);
+    expect(data.boldText, platformData.boldText);
+    expect(data.highContrast, platformData.highContrast);
+    expect(data.alwaysUse24HourFormat, platformData.alwaysUse24HourFormat);
+    expect(data.navigationMode, platformData.navigationMode);
+    expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
+    expect(data.displayFeatures, tester.view.displayFeatures);
+  });
+
+  testWidgets('MediaQueryData.fromView uses data from platformDispatcher if no platformData is provided', (WidgetTester tester) async {
+    tester.platformDispatcher
+      ..textScaleFactorTestValue = 123
+      ..platformBrightnessTestValue = Brightness.dark
+      ..accessibilityFeaturesTestValue = FakeAccessibilityFeatures.allOn;
+    addTearDown(() => tester.platformDispatcher.clearAllTestValues());
+
+    final MediaQueryData data = MediaQueryData.fromView(tester.view);
+    expect(data, hasOneLineDescription);
+    expect(data.hashCode, data.copyWith().hashCode);
+    expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
+    expect(data.devicePixelRatio, tester.view.devicePixelRatio);
+    expect(data.textScaleFactor, tester.platformDispatcher.textScaleFactor);
+    expect(data.platformBrightness, tester.platformDispatcher.platformBrightness);
+    expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
+    expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
+    expect(data.viewInsets, EdgeInsets.fromViewPadding(tester.view.viewInsets, tester.view.devicePixelRatio));
+    expect(data.systemGestureInsets, EdgeInsets.fromViewPadding(tester.view.systemGestureInsets, tester.view.devicePixelRatio));
+    expect(data.accessibleNavigation, tester.platformDispatcher.accessibilityFeatures.accessibleNavigation);
+    expect(data.invertColors, tester.platformDispatcher.accessibilityFeatures.invertColors);
+    expect(data.disableAnimations, tester.platformDispatcher.accessibilityFeatures.disableAnimations);
+    expect(data.boldText, tester.platformDispatcher.accessibilityFeatures.boldText);
+    expect(data.highContrast, tester.platformDispatcher.accessibilityFeatures.highContrast);
+    expect(data.alwaysUse24HourFormat, tester.platformDispatcher.alwaysUse24HourFormat);
+    expect(data.navigationMode, NavigationMode.traditional);
+    expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
+    expect(data.displayFeatures, tester.view.displayFeatures);
+  });
+
+  testWidgets('MediaQuery.fromView injects a new MediaQuery with data from view, preserving platform-specific data', (WidgetTester tester) async {
+    const MediaQueryData platformData = MediaQueryData(
+      textScaleFactor: 1234,
+      platformBrightness: Brightness.dark,
+      accessibleNavigation: true,
+      invertColors: true,
+      disableAnimations: true,
+      boldText: true,
+      highContrast: true,
+      alwaysUse24HourFormat: true,
+      navigationMode: NavigationMode.directional,
+    );
+
+    late MediaQueryData data;
+    await tester.pumpWidget(MediaQuery(
+      data: platformData,
+      child: MediaQuery.fromView(
+        view: tester.view,
+        child: Builder(
+          builder: (BuildContext context) {
+            data = MediaQuery.of(context);
+            return const Placeholder();
+          },
+        )
+      )
+    ));
+
+    expect(data, isNot(platformData));
+    expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
+    expect(data.devicePixelRatio, tester.view.devicePixelRatio);
+    expect(data.textScaleFactor, platformData.textScaleFactor);
+    expect(data.platformBrightness, platformData.platformBrightness);
+    expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
+    expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
+    expect(data.viewInsets, EdgeInsets.fromViewPadding(tester.view.viewInsets, tester.view.devicePixelRatio));
+    expect(data.systemGestureInsets, EdgeInsets.fromViewPadding(tester.view.systemGestureInsets, tester.view.devicePixelRatio));
+    expect(data.accessibleNavigation, platformData.accessibleNavigation);
+    expect(data.invertColors, platformData.invertColors);
+    expect(data.disableAnimations, platformData.disableAnimations);
+    expect(data.boldText, platformData.boldText);
+    expect(data.highContrast, platformData.highContrast);
+    expect(data.alwaysUse24HourFormat, platformData.alwaysUse24HourFormat);
+    expect(data.navigationMode, platformData.navigationMode);
+    expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
+    expect(data.displayFeatures, tester.view.displayFeatures);
+  });
+
+  testWidgets('MediaQuery.fromView injects a new MediaQuery with data from view when no surrounding MediaQuery exists', (WidgetTester tester) async {
+    tester.platformDispatcher
+      ..textScaleFactorTestValue = 123
+      ..platformBrightnessTestValue = Brightness.dark
+      ..accessibilityFeaturesTestValue = FakeAccessibilityFeatures.allOn;
+    addTearDown(() => tester.platformDispatcher.clearAllTestValues());
+
+    late MediaQueryData data;
+    MediaQueryData? outerData;
+    await pumpWidgetWithoutViewWrapper(
+      tester: tester,
+      widget: Builder(
+        builder: (BuildContext context) {
+          outerData = MediaQuery.maybeOf(context);
+          return MediaQuery.fromView(
+              view: tester.view,
+              child: Builder(
+                builder: (BuildContext context) {
+                  data = MediaQuery.of(context);
+                  return const Placeholder();
+                },
+              )
+          );
+        },
+      ),
+    );
+
+    expect(outerData, isNull);
+    expect(data.size, tester.view.physicalSize / tester.view.devicePixelRatio);
+    expect(data.devicePixelRatio, tester.view.devicePixelRatio);
+    expect(data.textScaleFactor, tester.platformDispatcher.textScaleFactor);
+    expect(data.platformBrightness, tester.platformDispatcher.platformBrightness);
+    expect(data.padding, EdgeInsets.fromViewPadding(tester.view.padding, tester.view.devicePixelRatio));
+    expect(data.viewPadding, EdgeInsets.fromViewPadding(tester.view.viewPadding, tester.view.devicePixelRatio));
+    expect(data.viewInsets, EdgeInsets.fromViewPadding(tester.view.viewInsets, tester.view.devicePixelRatio));
+    expect(data.systemGestureInsets, EdgeInsets.fromViewPadding(tester.view.systemGestureInsets, tester.view.devicePixelRatio));
+    expect(data.accessibleNavigation, tester.platformDispatcher.accessibilityFeatures.accessibleNavigation);
+    expect(data.invertColors, tester.platformDispatcher.accessibilityFeatures.invertColors);
+    expect(data.disableAnimations, tester.platformDispatcher.accessibilityFeatures.disableAnimations);
+    expect(data.boldText, tester.platformDispatcher.accessibilityFeatures.boldText);
+    expect(data.highContrast, tester.platformDispatcher.accessibilityFeatures.highContrast);
+    expect(data.alwaysUse24HourFormat, tester.platformDispatcher.alwaysUse24HourFormat);
+    expect(data.navigationMode, NavigationMode.traditional);
+    expect(data.gestureSettings, DeviceGestureSettings.fromView(tester.view));
+    expect(data.displayFeatures, tester.view.displayFeatures);
+  });
+
+  testWidgets('MediaQuery.fromView updates on notifications (no parent data)', (WidgetTester tester) async {
+    addTearDown(() => tester.platformDispatcher.clearAllTestValues());
+    addTearDown(() => tester.view.reset());
+
+    tester.platformDispatcher
+      ..textScaleFactorTestValue = 123
+      ..platformBrightnessTestValue = Brightness.dark
+      ..accessibilityFeaturesTestValue = FakeAccessibilityFeatures.allOn;
+    tester.view.devicePixelRatio = 44;
+
+    late MediaQueryData data;
+    MediaQueryData? outerData;
+    int rebuildCount = 0;
+    await pumpWidgetWithoutViewWrapper(
+      tester: tester,
+      widget: Builder(
+        builder: (BuildContext context) {
+          outerData = MediaQuery.maybeOf(context);
+          return MediaQuery.fromView(
+              view: tester.view,
+              child: Builder(
+                builder: (BuildContext context) {
+                  rebuildCount++;
+                  data = MediaQuery.of(context);
+                  return const Placeholder();
+                },
+              ),
+          );
+        },
+      ),
+    );
+
+    expect(outerData, isNull);
+    expect(rebuildCount, 1);
+
+    expect(data.textScaleFactor, 123);
+    tester.platformDispatcher.textScaleFactorTestValue = 456;
+    await tester.pump();
+    expect(data.textScaleFactor, 456);
+    expect(rebuildCount, 2);
+
+    expect(data.platformBrightness, Brightness.dark);
+    tester.platformDispatcher.platformBrightnessTestValue = Brightness.light;
+    await tester.pump();
+    expect(data.platformBrightness, Brightness.light);
+    expect(rebuildCount, 3);
+
+    expect(data.accessibleNavigation, true);
+    tester.platformDispatcher.accessibilityFeaturesTestValue = const FakeAccessibilityFeatures();
+    await tester.pump();
+    expect(data.accessibleNavigation, false);
+    expect(rebuildCount, 4);
+
+    expect(data.devicePixelRatio, 44);
+    tester.view.devicePixelRatio = 55;
+    await tester.pump();
+    expect(data.devicePixelRatio, 55);
+    expect(rebuildCount, 5);
+  });
+
+  testWidgets('MediaQuery.fromView updates on notifications (with parent data)', (WidgetTester tester) async {
+    addTearDown(() => tester.platformDispatcher.clearAllTestValues());
+    addTearDown(() => tester.view.reset());
+
+    tester.platformDispatcher
+      ..textScaleFactorTestValue = 123
+      ..platformBrightnessTestValue = Brightness.dark
+      ..accessibilityFeaturesTestValue = FakeAccessibilityFeatures.allOn;
+    tester.view.devicePixelRatio = 44;
+
+    late MediaQueryData data;
+    int rebuildCount = 0;
+    await tester.pumpWidget(
+      MediaQuery(
+        data: const MediaQueryData(
+          textScaleFactor: 44,
+          platformBrightness: Brightness.dark,
+          accessibleNavigation: true,
+        ),
+        child: MediaQuery.fromView(
+          view: tester.view,
+          child: Builder(
+            builder: (BuildContext context) {
+              rebuildCount++;
+              data = MediaQuery.of(context);
+              return const Placeholder();
+            },
+          ),
+        ),
+      ),
+    );
+
+    expect(rebuildCount, 1);
+
+    expect(data.textScaleFactor, 44);
+    tester.platformDispatcher.textScaleFactorTestValue = 456;
+    await tester.pump();
+    expect(data.textScaleFactor, 44);
+    expect(rebuildCount, 1);
+
+    expect(data.platformBrightness, Brightness.dark);
+    tester.platformDispatcher.platformBrightnessTestValue = Brightness.light;
+    await tester.pump();
+    expect(data.platformBrightness, Brightness.dark);
+    expect(rebuildCount, 1);
+
+    expect(data.accessibleNavigation, true);
+    tester.platformDispatcher.accessibilityFeaturesTestValue = const FakeAccessibilityFeatures();
+    await tester.pump();
+    expect(data.accessibleNavigation, true);
+    expect(rebuildCount, 1);
+
+    expect(data.devicePixelRatio, 44);
+    tester.view.devicePixelRatio = 55;
+    await tester.pump();
+    expect(data.devicePixelRatio, 55);
+    expect(rebuildCount, 2);
+  });
+
+  testWidgets('MediaQuery.fromView updates when parent data changes', (WidgetTester tester) async {
+    late MediaQueryData data;
+    int rebuildCount = 0;
+    double textScaleFactor = 55;
+    late StateSetter stateSetter;
+    await tester.pumpWidget(
+      StatefulBuilder(
+        builder: (BuildContext context, StateSetter setState) {
+          stateSetter = setState;
+          return MediaQuery(
+            data: MediaQueryData(textScaleFactor: textScaleFactor),
+            child: MediaQuery.fromView(
+              view: tester.view,
+              child: Builder(
+                builder: (BuildContext context) {
+                  rebuildCount++;
+                  data = MediaQuery.of(context);
+                  return const Placeholder();
+                },
+              ),
+            ),
+          );
+        },
+      ),
+    );
+
+    expect(rebuildCount, 1);
+    expect(data.textScaleFactor, 55);
+
+    stateSetter(() {
+      textScaleFactor = 66;
+    });
+    await tester.pump();
+    expect(data.textScaleFactor, 66);
+    expect(rebuildCount, 2);
+  });
+
   testWidgets('MediaQueryData.copyWith defaults to source', (WidgetTester tester) async {
-    final MediaQueryData data = MediaQueryData.fromWindow(WidgetsBinding.instance.window);
+    final MediaQueryData data = MediaQueryData.fromView(tester.view);
     final MediaQueryData copied = data.copyWith();
     expect(copied.size, data.size);
     expect(copied.devicePixelRatio, data.devicePixelRatio);
@@ -155,7 +513,7 @@ void main() {
       ),
     ];
 
-    final MediaQueryData data = MediaQueryData.fromWindow(WidgetsBinding.instance.window);
+    final MediaQueryData data = MediaQueryData.fromView(tester.view);
     final MediaQueryData copied = data.copyWith(
       size: customSize,
       devicePixelRatio: customDevicePixelRatio,
@@ -686,21 +1044,21 @@ void main() {
     expect(insideHighContrast, true);
   });
 
-  testWidgets('MediaQuery.boldTextOverride', (WidgetTester tester) async {
+  testWidgets('MediaQuery.boldTextOf', (WidgetTester tester) async {
     late bool outsideBoldTextOverride;
     late bool insideBoldTextOverride;
 
     await tester.pumpWidget(
       Builder(
         builder: (BuildContext context) {
-          outsideBoldTextOverride = MediaQuery.boldTextOverride(context);
+          outsideBoldTextOverride = MediaQuery.boldTextOf(context);
           return MediaQuery(
             data: const MediaQueryData(
               boldText: true,
             ),
             child: Builder(
               builder: (BuildContext context) {
-                insideBoldTextOverride = MediaQuery.boldTextOverride(context);
+                insideBoldTextOverride = MediaQuery.boldTextOf(context);
                 return Container();
               },
             ),
@@ -713,20 +1071,19 @@ void main() {
     expect(insideBoldTextOverride, true);
   });
 
-  testWidgets('MediaQuery.fromWindow creates a MediaQuery', (WidgetTester tester) async {
-    bool hasMediaQueryAsParentOutside = false;
-    bool hasMediaQueryAsParentInside = false;
+  testWidgets('MediaQuery.fromView creates a MediaQuery', (WidgetTester tester) async {
+    MediaQuery? mediaQueryOutside;
+    MediaQuery? mediaQueryInside;
 
     await tester.pumpWidget(
       Builder(
         builder: (BuildContext context) {
-          hasMediaQueryAsParentOutside =
-              context.findAncestorWidgetOfExactType<MediaQuery>() != null;
-          return MediaQuery.fromWindow(
+          mediaQueryOutside = context.findAncestorWidgetOfExactType<MediaQuery>();
+          return MediaQuery.fromView(
+            view: View.of(context),
             child: Builder(
               builder: (BuildContext context) {
-                hasMediaQueryAsParentInside =
-                    context.findAncestorWidgetOfExactType<MediaQuery>() != null;
+                mediaQueryInside = context.findAncestorWidgetOfExactType<MediaQuery>();
                 return const SizedBox();
               },
             ),
@@ -735,12 +1092,12 @@ void main() {
       ),
     );
 
-    expect(hasMediaQueryAsParentOutside, false);
-    expect(hasMediaQueryAsParentInside, true);
+    expect(mediaQueryInside, isNotNull);
+    expect(mediaQueryOutside, isNot(mediaQueryInside));
   });
 
   testWidgets('MediaQueryData.fromWindow is created using window values', (WidgetTester tester) async {
-    final MediaQueryData windowData = MediaQueryData.fromWindow(WidgetsBinding.instance.window);
+    final MediaQueryData windowData = MediaQueryData.fromWindow(tester.view);
     late MediaQueryData fromWindowData;
 
     await tester.pumpWidget(
@@ -929,12 +1286,189 @@ void main() {
     expect(subScreenMediaQuery.displayFeatures, <DisplayFeature>[cutoutDisplayFeature]);
   });
 
-  testWidgets('MediaQueryData.gestureSettings is set from window.viewConfiguration', (WidgetTester tester) async {
-    tester.binding.window.viewConfigurationTestValue = const ViewConfiguration(
-      gestureSettings: GestureSettings(physicalDoubleTapSlop: 100, physicalTouchSlop: 100),
+  testWidgets('MediaQueryData.gestureSettings is set from view.gestureSettings', (WidgetTester tester) async {
+    tester.view.gestureSettings = const GestureSettings(physicalDoubleTapSlop: 100, physicalTouchSlop: 100);
+    addTearDown(() => tester.view.resetGestureSettings());
+
+    expect(MediaQueryData.fromView(tester.view).gestureSettings.touchSlop, closeTo(33.33, 0.1)); // Repeating, of course
+  });
+
+  testWidgets('MediaQuery can be partially depended-on', (WidgetTester tester) async {
+    MediaQueryData data = const MediaQueryData(
+      size: Size(800, 600),
+      textScaleFactor: 1.1
     );
 
-    expect(MediaQueryData.fromWindow(tester.binding.window).gestureSettings.touchSlop, closeTo(33.33, 0.1)); // Repeating, of course
-    tester.binding.window.viewConfigurationTestValue = null;
+    int sizeBuildCount = 0;
+    int textScaleFactorBuildCount = 0;
+
+    final Widget showSize = Builder(
+      builder: (BuildContext context) {
+        sizeBuildCount++;
+        return Text('size: ${MediaQuery.sizeOf(context)}');
+      }
+    );
+
+    final Widget showTextScaleFactor = Builder(
+      builder: (BuildContext context) {
+        textScaleFactorBuildCount++;
+        return Text('textScaleFactor: ${MediaQuery.textScaleFactorOf(context).toStringAsFixed(1)}');
+      }
+    );
+
+    final Widget page = StatefulBuilder(
+      builder: (BuildContext context, StateSetter setState) {
+        return MediaQuery(
+          data: data,
+          child: Center(
+            child: Column(
+              children: <Widget>[
+                showSize,
+                showTextScaleFactor,
+                ElevatedButton(
+                  onPressed: () {
+                    setState(() {
+                      data = data.copyWith(size: Size(data.size.width + 100, data.size.height));
+                    });
+                  },
+                  child: const Text('Increase width by 100')
+                ),
+                ElevatedButton(
+                  onPressed: () {
+                    setState(() {
+                      data = data.copyWith(textScaleFactor: data.textScaleFactor + 0.1);
+                    });
+                  },
+                  child: const Text('Increase textScaleFactor by 0.1')
+                )
+              ]
+            )
+          )
+        );
+      },
+    );
+
+    await tester.pumpWidget(MaterialApp(home: page));
+    expect(find.text('size: Size(800.0, 600.0)'), findsOneWidget);
+    expect(find.text('textScaleFactor: 1.1'), findsOneWidget);
+    expect(sizeBuildCount, 1);
+    expect(textScaleFactorBuildCount, 1);
+
+    await tester.tap(find.text('Increase width by 100'));
+    await tester.pumpAndSettle();
+    expect(find.text('size: Size(900.0, 600.0)'), findsOneWidget);
+    expect(find.text('textScaleFactor: 1.1'), findsOneWidget);
+    expect(sizeBuildCount, 2);
+    expect(textScaleFactorBuildCount, 1);
+
+    await tester.tap(find.text('Increase textScaleFactor by 0.1'));
+    await tester.pumpAndSettle();
+    expect(find.text('size: Size(900.0, 600.0)'), findsOneWidget);
+    expect(find.text('textScaleFactor: 1.2'), findsOneWidget);
+    expect(sizeBuildCount, 2);
+    expect(textScaleFactorBuildCount, 2);
   });
+
+  testWidgets('MediaQuery partial dependencies', (WidgetTester tester) async {
+    MediaQueryData data = const MediaQueryData();
+
+    int buildCount = 0;
+
+    final Widget builder = Builder(
+      builder: (BuildContext context) {
+        _MediaQueryAspectVariant.aspect!.method(context);
+        buildCount++;
+        return const SizedBox.shrink();
+      }
+    );
+
+    final Widget page = StatefulBuilder(
+      builder: (BuildContext context, StateSetter setState) {
+        return MediaQuery(
+          data: data,
+          child: ListView(
+            children: <Widget>[
+              builder,
+              ElevatedButton(
+                onPressed: () {
+                  setState(() {
+                    data = _MediaQueryAspectVariant.aspect!.data;
+                  });
+                },
+                child: const Text('Change data')
+              ),
+              ElevatedButton(
+                onPressed: () {
+                  setState(() {
+                    data = data.copyWith();
+                  });
+                },
+                child: const Text('Copy data')
+              )
+            ]
+          )
+        );
+      },
+    );
+
+    await tester.pumpWidget(MaterialApp(home: page));
+    expect(buildCount, 1);
+
+    await tester.tap(find.text('Copy data'));
+    await tester.pumpAndSettle();
+    expect(buildCount, 1);
+
+    await tester.tap(find.text('Change data'));
+    await tester.pumpAndSettle();
+    expect(buildCount, 2);
+
+    await tester.tap(find.text('Copy data'));
+    await tester.pumpAndSettle();
+    expect(buildCount, 2);
+  }, variant: _MediaQueryAspectVariant(
+    values: <_MediaQueryAspectCase>[
+      const _MediaQueryAspectCase(MediaQuery.sizeOf, MediaQueryData(size: Size(1, 1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeSizeOf, MediaQueryData(size: Size(1, 1))),
+      const _MediaQueryAspectCase(MediaQuery.orientationOf, MediaQueryData(size: Size(2, 1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeOrientationOf, MediaQueryData(size: Size(2, 1))),
+      const _MediaQueryAspectCase(MediaQuery.devicePixelRatioOf, MediaQueryData(devicePixelRatio: 1.1)),
+      const _MediaQueryAspectCase(MediaQuery.maybeDevicePixelRatioOf, MediaQueryData(devicePixelRatio: 1.1)),
+      const _MediaQueryAspectCase(MediaQuery.textScaleFactorOf, MediaQueryData(textScaleFactor: 1.1)),
+      const _MediaQueryAspectCase(MediaQuery.maybeTextScaleFactorOf, MediaQueryData(textScaleFactor: 1.1)),
+      const _MediaQueryAspectCase(MediaQuery.platformBrightnessOf, MediaQueryData(platformBrightness: Brightness.dark)),
+      const _MediaQueryAspectCase(MediaQuery.maybePlatformBrightnessOf, MediaQueryData(platformBrightness: Brightness.dark)),
+      const _MediaQueryAspectCase(MediaQuery.paddingOf, MediaQueryData(padding: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.maybePaddingOf, MediaQueryData(padding: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.viewInsetsOf, MediaQueryData(viewInsets: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeViewInsetsOf, MediaQueryData(viewInsets: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.systemGestureInsetsOf, MediaQueryData(systemGestureInsets: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeSystemGestureInsetsOf, MediaQueryData(systemGestureInsets: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.viewPaddingOf, MediaQueryData(viewPadding: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeViewPaddingOf, MediaQueryData(viewPadding: EdgeInsets.all(1))),
+      const _MediaQueryAspectCase(MediaQuery.alwaysUse24HourFormatOf, MediaQueryData(alwaysUse24HourFormat: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeAlwaysUse24HourFormatOf, MediaQueryData(alwaysUse24HourFormat: true)),
+      const _MediaQueryAspectCase(MediaQuery.accessibleNavigationOf, MediaQueryData(accessibleNavigation: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeAccessibleNavigationOf, MediaQueryData(accessibleNavigation: true)),
+      const _MediaQueryAspectCase(MediaQuery.invertColorsOf, MediaQueryData(invertColors: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeInvertColorsOf, MediaQueryData(invertColors: true)),
+      const _MediaQueryAspectCase(MediaQuery.highContrastOf, MediaQueryData(highContrast: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeHighContrastOf, MediaQueryData(highContrast: true)),
+      const _MediaQueryAspectCase(MediaQuery.disableAnimationsOf, MediaQueryData(disableAnimations: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeDisableAnimationsOf, MediaQueryData(disableAnimations: true)),
+      const _MediaQueryAspectCase(MediaQuery.boldTextOf, MediaQueryData(boldText: true)),
+      const _MediaQueryAspectCase(MediaQuery.maybeBoldTextOf, MediaQueryData(boldText: true)),
+      const _MediaQueryAspectCase(MediaQuery.navigationModeOf, MediaQueryData(navigationMode: NavigationMode.directional)),
+      const _MediaQueryAspectCase(MediaQuery.maybeNavigationModeOf, MediaQueryData(navigationMode: NavigationMode.directional)),
+      const _MediaQueryAspectCase(MediaQuery.gestureSettingsOf, MediaQueryData(gestureSettings: DeviceGestureSettings(touchSlop: 1))),
+      const _MediaQueryAspectCase(MediaQuery.maybeGestureSettingsOf, MediaQueryData(gestureSettings: DeviceGestureSettings(touchSlop: 1))),
+      const _MediaQueryAspectCase(MediaQuery.displayFeaturesOf, MediaQueryData(displayFeatures: <DisplayFeature>[DisplayFeature(bounds: Rect.zero, type: DisplayFeatureType.unknown, state: DisplayFeatureState.unknown)])),
+      const _MediaQueryAspectCase(MediaQuery.maybeDisplayFeaturesOf, MediaQueryData(displayFeatures: <DisplayFeature>[DisplayFeature(bounds: Rect.zero, type: DisplayFeatureType.unknown, state: DisplayFeatureState.unknown)])),
+    ]
+  ));
+}
+
+Future<void> pumpWidgetWithoutViewWrapper({required WidgetTester tester, required  Widget widget}) {
+  tester.binding.attachRootWidget(widget);
+  tester.binding.scheduleFrame();
+  return tester.binding.pump();
 }
