@@ -2,11 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// @dart = 2.8
-
 import 'package:file/memory.dart';
 import 'package:file_testing/file_testing.dart';
 import 'package:flutter_tools/src/base/file_system.dart';
+import 'package:flutter_tools/src/base/logger.dart';
 import 'package:flutter_tools/src/base/platform.dart';
 import 'package:flutter_tools/src/cache.dart';
 import 'package:flutter_tools/src/commands/build_windows.dart';
@@ -21,9 +20,11 @@ import '../../src/fakes.dart';
 import '../../src/test_flutter_command_runner.dart';
 
 const String flutterRoot = r'C:\flutter';
-const String buildFilePath = r'C:\windows\CMakeLists.txt';
-const String visualStudioPath = r'C:\Program Files (x86)\Microsoft Visual Studio\2017\Community';
-const String _cmakePath = visualStudioPath + r'\Common7\IDE\CommonExtensions\Microsoft\CMake\CMake\bin\cmake.exe';
+const String buildFilePath = r'windows\CMakeLists.txt';
+const String visualStudioPath =
+    r'C:\Program Files (x86)\Microsoft Visual Studio\2017\Community';
+const String _cmakePath = visualStudioPath +
+    r'\Common7\IDE\CommonExtensions\Microsoft\CMake\CMake\bin\cmake.exe';
 const String _defaultGenerator = 'Visual Studio 16 2019';
 
 final Platform windowsPlatform = FakePlatform(
@@ -41,10 +42,9 @@ final Platform notWindowsPlatform = FakePlatform(
 );
 
 void main() {
-  FileSystem fileSystem;
-
-  ProcessManager processManager;
-  TestUsage usage;
+  late FileSystem fileSystem;
+  late ProcessManager processManager;
+  late TestUsage usage;
 
   setUpAll(() {
     Cache.disableLocking();
@@ -73,14 +73,14 @@ void main() {
   // Returns the command matching the build_windows call to generate CMake
   // files.
   FakeCommand cmakeGenerationCommand({
-    void Function() onRun,
+    void Function()? onRun,
     String generator = _defaultGenerator,
   }) {
     return FakeCommand(
       command: <String>[
         _cmakePath,
         '-S',
-        fileSystem.path.dirname(buildFilePath),
+        fileSystem.path.absolute(fileSystem.path.dirname(buildFilePath)),
         '-B',
         r'build\windows',
         '-G',
@@ -93,7 +93,7 @@ void main() {
   // Returns the command matching the build_windows call to build.
   FakeCommand buildCommand(String buildMode, {
     bool verbose = false,
-    void Function() onRun,
+    void Function()? onRun,
     String stdout = '',
   }) {
     return FakeCommand(
@@ -117,7 +117,7 @@ void main() {
   }
 
   testUsingContext('Windows build fails when there is no cmake path', () async {
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = FakeVisualStudio(cmakePath: null);
     setUpMockProjectFilesForBuild();
 
@@ -133,7 +133,7 @@ void main() {
 
   testUsingContext('Windows build fails when there is no windows project', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockCoreProjectFiles();
 
@@ -151,7 +151,7 @@ void main() {
 
   testUsingContext('Windows build fails on non windows platform', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -167,7 +167,7 @@ void main() {
 
   testUsingContext('Windows build fails when feature is disabled', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -183,7 +183,7 @@ void main() {
 
   testUsingContext('Windows build does not spew stdout to status logger', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -208,7 +208,7 @@ void main() {
 
   testUsingContext('Windows build extracts errors from stdout', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -266,7 +266,7 @@ C:\foo\windows\runner\main.cpp(17,1): error C2065: 'Baz': undeclared identifier
 
   testUsingContext('Windows verbose build sets VERBOSE_SCRIPT_LOGGING', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -292,7 +292,7 @@ C:\foo\windows\runner\main.cpp(17,1): error C2065: 'Baz': undeclared identifier
 
   testUsingContext('Windows build works around CMake generation bug', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio(displayVersion: '17.1.0');
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -428,7 +428,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build invokes build and writes generated files', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -498,7 +498,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows profile build passes Profile configuration', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -521,7 +521,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
     const String generator = 'A different generator';
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio(
       cmakeGenerator: generator);
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -542,7 +542,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext("Windows build uses pubspec's version", () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -588,7 +588,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build uses build-name and build-number', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -632,7 +632,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build build-name overrides pubspec', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -679,7 +679,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build build-number overrides pubspec', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -726,7 +726,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build build-name and build-number override pubspec', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -774,7 +774,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build warns on non-numeric build-number', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -824,7 +824,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Windows build warns on complex build-number', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -873,14 +873,14 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
   });
 
   testUsingContext('hidden when not enabled on Windows host', () {
-    expect(BuildWindowsCommand().hidden, true);
+    expect(BuildWindowsCommand(logger: BufferLogger.test()).hidden, true);
   }, overrides: <Type, Generator>{
     FeatureFlags: () => TestFeatureFlags(),
     Platform: () => windowsPlatform,
   });
 
   testUsingContext('Not hidden when enabled and on Windows host', () {
-    expect(BuildWindowsCommand().hidden, false);
+    expect(BuildWindowsCommand(logger: BufferLogger.test()).hidden, false);
   }, overrides: <Type, Generator>{
     FeatureFlags: () => TestFeatureFlags(isWindowsEnabled: true),
     Platform: () => windowsPlatform,
@@ -888,7 +888,7 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
 
   testUsingContext('Performs code size analysis and sends analytics', () async {
     final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
-    final BuildWindowsCommand command = BuildWindowsCommand()
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
       ..visualStudioOverride = fakeVisualStudio;
     setUpMockProjectFilesForBuild();
 
@@ -921,9 +921,9 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
     );
 
     expect(testLogger.statusText, contains('A summary of your Windows bundle analysis can be found at'));
-    expect(testLogger.statusText, contains('flutter pub global activate devtools; flutter pub global run devtools --appSizeBase='));
+    expect(testLogger.statusText, contains('dart devtools --appSizeBase='));
     expect(usage.events, contains(
-       const TestUsageEvent('code-size-analysis', 'windows'),
+        const TestUsageEvent('code-size-analysis', 'windows'),
     ));
   }, overrides: <Type, Generator>{
     FeatureFlags: () => TestFeatureFlags(isWindowsEnabled: true),
@@ -933,6 +933,35 @@ if %errorlevel% neq 0 goto :VCEnd</Command>
     FileSystemUtils: () => FileSystemUtils(fileSystem: fileSystem, platform: windowsPlatform),
     Usage: () => usage,
   });
+
+  // Confirms that running for Windows in a directory with a
+  // bad character (' in this case) throws the desired error message
+  // If the issue https://github.com/flutter/flutter/issues/104802 ever
+  // is resolved on the VS side, we can allow these paths again
+  testUsingContext('Test bad path characters', () async {
+    final FakeVisualStudio fakeVisualStudio = FakeVisualStudio();
+    final BuildWindowsCommand command = BuildWindowsCommand(logger: BufferLogger.test())
+      ..visualStudioOverride = fakeVisualStudio;
+    fileSystem.currentDirectory = fileSystem.directory("test_'path")
+      ..createSync();
+    final String absPath = fileSystem.currentDirectory.absolute.path;
+    setUpMockCoreProjectFiles();
+
+    expect(
+        createTestCommandRunner(command).run(const <String>['windows', '--no-pub']),
+        throwsToolExit(
+          message:
+              'Path $absPath contains invalid characters in "\'#!\$^&*=|,;<>?". '
+              'Please rename your directory so as to not include any of these characters '
+              'and retry.',
+        ),
+    );
+  }, overrides: <Type, Generator>{
+    Platform: () => windowsPlatform,
+    FileSystem: () => fileSystem,
+    ProcessManager: () => FakeProcessManager.any(),
+    FeatureFlags: () => TestFeatureFlags(isWindowsEnabled: true),
+  });
 }
 
 class FakeVisualStudio extends Fake implements VisualStudio {
@@ -943,7 +972,7 @@ class FakeVisualStudio extends Fake implements VisualStudio {
   });
 
   @override
-  final String cmakePath;
+  final String? cmakePath;
 
   @override
   final String cmakeGenerator;
