@@ -2,8 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// @dart = 2.8
-
 import 'package:file/memory.dart';
 import 'package:file_testing/file_testing.dart';
 import 'package:flutter_tools/src/android/gradle_errors.dart';
@@ -13,14 +11,23 @@ import 'package:flutter_tools/src/base/file_system.dart';
 import 'package:flutter_tools/src/base/logger.dart';
 import 'package:flutter_tools/src/base/platform.dart';
 import 'package:flutter_tools/src/base/terminal.dart';
-import 'package:flutter_tools/src/globals.dart' as globals;
 import 'package:flutter_tools/src/project.dart';
+import 'package:test/fake.dart';
 
 import '../../src/common.dart';
 import '../../src/context.dart';
+import '../../src/fake_process_manager.dart';
 import '../../src/fakes.dart';
 
 void main() {
+  late FileSystem fileSystem;
+  late FakeProcessManager processManager;
+
+  setUp(() {
+    fileSystem = MemoryFileSystem.test();
+    processManager = FakeProcessManager.empty();
+  });
+
   group('gradleErrors', () {
     testWithoutContext('list of errors', () {
       // If you added a new Gradle error, please update this test.
@@ -41,6 +48,7 @@ void main() {
           outdatedGradleHandler,
           sslExceptionHandler,
           zipExceptionHandler,
+          incompatibleJavaAndGradleVersionsHandler,
         ])
       );
     });
@@ -63,7 +71,12 @@ at org.gradle.wrapper.WrapperExecutor.execute(WrapperExecutor.java:128)
 at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -71,8 +84,8 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if gradle fails downloading with proxy error', () async {
@@ -93,7 +106,12 @@ at org.gradle.wrapper.WrapperExecutor.execute(WrapperExecutor.java:128)
 at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -101,8 +119,8 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if gradle times out waiting for exclusive access to zip', () async {
@@ -114,7 +132,12 @@ Exception in thread "main" java.lang.RuntimeException: Timeout of 120000 reached
 	at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -122,8 +145,8 @@ Exception in thread "main" java.lang.RuntimeException: Timeout of 120000 reached
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if remote host closes connection', () async {
@@ -151,7 +174,12 @@ Exception in thread "main" javax.net.ssl.SSLHandshakeException: Remote host clos
 	at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -159,8 +187,8 @@ Exception in thread "main" javax.net.ssl.SSLHandshakeException: Remote host clos
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if file opening fails', () async {
@@ -180,7 +208,12 @@ Exception in thread "main" java.io.FileNotFoundException: https://downloads.grad
 	at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -188,8 +221,8 @@ Exception in thread "main" java.io.FileNotFoundException: https://downloads.grad
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if the connection is reset', () async {
@@ -220,7 +253,12 @@ Exception in thread "main" java.net.SocketException: Connection reset
 	at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -228,8 +266,8 @@ Exception in thread "main" java.net.SocketException: Connection reset
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if Gradle could not get a resource', () async {
@@ -247,7 +285,12 @@ A problem occurred configuring root project 'android'.
                      > Could not GET 'https://jcenter.bintray.com/net/sf/proguard/proguard-parent/6.0.3/proguard-parent-6.0.3.pom'. Received status code 504 from server: Gateway Time-out''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -255,8 +298,8 @@ A problem occurred configuring root project 'android'.
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('retries if Gradle could not get a resource (non-Gateway)', () async {
@@ -278,7 +321,12 @@ A problem occurred configuring root project 'android'.
                > Remote host closed connection during handshake''';
 
       expect(formatTestErrorMessage(errorMessage, networkErrorHandler), isTrue);
-      expect(await networkErrorHandler.handler(), equals(GradleBuildStatus.retry));
+      expect(await networkErrorHandler.handler(
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+      ), equals(GradleBuildStatus.retry));
 
       expect(testLogger.errorText,
         contains(
@@ -286,8 +334,8 @@ A problem occurred configuring root project 'android'.
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -317,7 +365,7 @@ Execution failed for task ':app:mergeDexDebug'.
      Learn how to resolve this issue at https://developer.android.com/tools/building/multidex.html''';
 
       expect(formatTestErrorMessage(errorMessage, multidexErrorHandler), isTrue);
-      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(globals.fs.currentDirectory), multidexEnabled: true), equals(GradleBuildStatus.exit));
+      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(fileSystem.currentDirectory), multidexEnabled: true, usesAndroidX: true, line: ''), equals(GradleBuildStatus.exit));
 
       expect(testLogger.statusText,
         contains(
@@ -335,8 +383,8 @@ Execution failed for task ':app:mergeDexDebug'.
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
     testUsingContext('retries if multidex support enabled', () async {
       const String errorMessage = r'''
@@ -362,7 +410,7 @@ Execution failed for task ':app:mergeDexDebug'.
      The number of method references in a .dex file cannot exceed 64K.
      Learn how to resolve this issue at https://developer.android.com/tools/building/multidex.html''';
 
-      final File manifest = globals.fs.currentDirectory
+      final File manifest = fileSystem.currentDirectory
           .childDirectory('android')
           .childDirectory('app')
           .childDirectory('src')
@@ -381,7 +429,7 @@ Execution failed for task ':app:mergeDexDebug'.
 ''', flush: true);
 
       expect(formatTestErrorMessage(errorMessage, multidexErrorHandler), isTrue);
-      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(globals.fs.currentDirectory), multidexEnabled: true), equals(GradleBuildStatus.retry));
+      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(fileSystem.currentDirectory), multidexEnabled: true, line: '', usesAndroidX: true), equals(GradleBuildStatus.retry));
 
       expect(testLogger.statusText,
         contains(
@@ -394,8 +442,8 @@ Execution failed for task ':app:mergeDexDebug'.
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
       AnsiTerminal: () => _TestPromptTerminal('y'),
     });
 
@@ -423,7 +471,7 @@ Execution failed for task ':app:mergeDexDebug'.
      The number of method references in a .dex file cannot exceed 64K.
      Learn how to resolve this issue at https://developer.android.com/tools/building/multidex.html''';
 
-      final File manifest = globals.fs.currentDirectory
+      final File manifest = fileSystem.currentDirectory
           .childDirectory('android')
           .childDirectory('app')
           .childDirectory('src')
@@ -442,7 +490,7 @@ Execution failed for task ':app:mergeDexDebug'.
 ''', flush: true);
 
       expect(formatTestErrorMessage(errorMessage, multidexErrorHandler), isTrue);
-      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(globals.fs.currentDirectory), multidexEnabled: true), equals(GradleBuildStatus.exit));
+      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(fileSystem.currentDirectory), multidexEnabled: true, line: '', usesAndroidX: true), equals(GradleBuildStatus.exit));
 
       expect(testLogger.statusText,
         contains(
@@ -460,8 +508,8 @@ Execution failed for task ':app:mergeDexDebug'.
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
       AnsiTerminal: () => _TestPromptTerminal('n'),
     });
 
@@ -490,7 +538,7 @@ Execution failed for task ':app:mergeDexDebug'.
      Learn how to resolve this issue at https://developer.android.com/tools/building/multidex.html''';
 
       expect(formatTestErrorMessage(errorMessage, multidexErrorHandler), isTrue);
-      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(globals.fs.currentDirectory), multidexEnabled: false), equals(GradleBuildStatus.exit));
+      expect(await multidexErrorHandler.handler(project: FlutterProject.fromDirectory(fileSystem.currentDirectory), multidexEnabled: false, line: '', usesAndroidX: true), equals(GradleBuildStatus.exit));
 
       expect(testLogger.statusText,
         contains(
@@ -498,8 +546,8 @@ Execution failed for task ':app:mergeDexDebug'.
         )
       );
     }, overrides: <Type, Generator>{
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.any(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -510,7 +558,12 @@ Permission denied
 Command: /home/android/gradlew assembleRelease
 ''';
       expect(formatTestErrorMessage(errorMessage, permissionDeniedErrorHandler), isTrue);
-      expect(await permissionDeniedErrorHandler.handler(), equals(GradleBuildStatus.exit));
+      expect(await permissionDeniedErrorHandler.handler(
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+      ), equals(GradleBuildStatus.exit));
 
       expect(
         testLogger.statusText,
@@ -540,7 +593,12 @@ Command: /home/android/gradlew assembleRelease
     });
 
     testUsingContext('handler', () async {
-      expect(await permissionDeniedErrorHandler.handler(), equals(GradleBuildStatus.exit));
+      expect(await permissionDeniedErrorHandler.handler(
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+      ), equals(GradleBuildStatus.exit));
 
       expect(
         testLogger.statusText,
@@ -573,7 +631,9 @@ Command: /home/android/gradlew assembleRelease
     testUsingContext('handler', () async {
       await licenseNotAcceptedHandler.handler(
         line: 'You have not accepted the license agreements of the following SDK components: [foo, bar]',
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        multidexEnabled: true,
       );
 
       expect(
@@ -593,12 +653,6 @@ Command: /home/android/gradlew assembleRelease
   });
 
   group('flavor undefined', () {
-    FakeProcessManager fakeProcessManager;
-
-    setUp(() {
-      fakeProcessManager = FakeProcessManager.empty();
-    });
-
     testWithoutContext('pattern', () {
       expect(
         flavorUndefinedHandler.test(
@@ -627,7 +681,7 @@ Command: /home/android/gradlew assembleRelease
     });
 
     testUsingContext('handler - with flavor', () async {
-      fakeProcessManager.addCommand(const FakeCommand(
+      processManager.addCommand(const FakeCommand(
         command: <String>[
       'gradlew',
         'app:tasks' ,
@@ -648,7 +702,10 @@ assembleFooTest
       ));
 
       await flavorUndefinedHandler.handler(
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
       );
 
       expect(
@@ -670,16 +727,16 @@ assembleFooTest
           '└─────────────────────────────────────────────────────────────────────────────────────────────────┘\n'
         )
       );
-      expect(fakeProcessManager.hasRemainingExpectations, isFalse);
+      expect(processManager, hasNoRemainingExpectations);
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      ProcessManager: () => fakeProcessManager,
-      FileSystem: () => MemoryFileSystem.test(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
 
     testUsingContext('handler - without flavor', () async {
-      fakeProcessManager.addCommand(const FakeCommand(
+      processManager.addCommand(const FakeCommand(
         command: <String>[
           'gradlew',
           'app:tasks' ,
@@ -694,7 +751,10 @@ assembleProfile
       ));
 
       await flavorUndefinedHandler.handler(
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
       );
 
       expect(
@@ -709,12 +769,12 @@ assembleProfile
           '└───────────────────────────────────────────────────────────────────────────────────────────────┘\n'
         )
       );
-      expect(fakeProcessManager.hasRemainingExpectations, isFalse);
+      expect(processManager, hasNoRemainingExpectations);
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      ProcessManager: () => fakeProcessManager,
-      FileSystem: () => MemoryFileSystem.test(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -731,7 +791,9 @@ assembleProfile
     testUsingContext('suggestion', () async {
       await minSdkVersionHandler.handler(
         line: stdoutLine,
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        multidexEnabled: true,
       );
 
       expect(
@@ -747,19 +809,19 @@ assembleProfile
           '│   }                                                                                           │\n'
           '│ }                                                                                             │\n'
           '│                                                                                               │\n'
-          "│ Note that your app won't be available to users running Android SDKs below 19.                 │\n"
-          '│ Alternatively, try to find a version of this plugin that supports these lower versions of the │\n'
-          '│ Android SDK.                                                                                  │\n'
+          '│ Following this change, your app will not be available to users running Android SDKs below 19. │\n'
+          '│ Consider searching for a version of this plugin that supports these lower versions of the     │\n'
+          '│ Android SDK instead.                                                                          │\n'
           '│ For more information, see:                                                                    │\n'
-          '│ https://docs.flutter.dev/deployment/android#reviewing-the-build-configuration                 │\n'
+          '│ https://docs.flutter.dev/deployment/android#reviewing-the-gradle-build-configuration          │\n'
           '└───────────────────────────────────────────────────────────────────────────────────────────────┘\n'
         )
       );
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -776,7 +838,10 @@ assembleProfile
 
     testUsingContext('suggestion', () async {
       await transformInputIssueHandler.handler(
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
       );
 
       expect(
@@ -797,8 +862,8 @@ assembleProfile
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -818,7 +883,10 @@ Execution failed for task ':app:generateDebugFeatureTransitiveDeps'.
 
     testUsingContext('suggestion', () async {
       await lockFileDepMissingHandler.handler(
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
       );
 
       expect(
@@ -835,8 +903,8 @@ Execution failed for task ':app:generateDebugFeatureTransitiveDeps'.
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -854,26 +922,29 @@ Execution failed for task ':app:generateDebugFeatureTransitiveDeps'.
 
     testUsingContext('suggestion', () async {
       await incompatibleKotlinVersionHandler.handler(
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
       );
 
       expect(
         testLogger.statusText,
         contains(
           '\n'
-          '┌─ Flutter Fix ────────────────────────────────────────────────────────────────────────────────┐\n'
-          '│ [!] Your project requires a newer version of the Kotlin Gradle plugin.                       │\n'
-          '│ Find the latest version on https://kotlinlang.org/docs/gradle.html#plugin-and-versions, then │\n'
-          '│ update /android/build.gradle:                                                                │\n'
-          "│ ext.kotlin_version = '<latest-version>'                                                      │\n"
-          '└──────────────────────────────────────────────────────────────────────────────────────────────┘\n'
+          '┌─ Flutter Fix ──────────────────────────────────────────────────────────────────────────────┐\n'
+          '│ [!] Your project requires a newer version of the Kotlin Gradle plugin.                     │\n'
+          '│ Find the latest version on https://kotlinlang.org/docs/releases.html#release-details, then │\n'
+          '│ update /android/build.gradle:                                                              │\n'
+          "│ ext.kotlin_version = '<latest-version>'                                                    │\n"
+          '└────────────────────────────────────────────────────────────────────────────────────────────┘\n'
         )
       );
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -894,7 +965,9 @@ A problem occurred evaluating project ':app'.
     testUsingContext('suggestion', () async {
       await outdatedGradleHandler.handler(
         line: errorMessage,
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        multidexEnabled: true,
       );
 
       expect(
@@ -907,18 +980,18 @@ A problem occurred evaluating project ':app'.
           '│ To fix this issue, replace the following content:                                │\n'
           '│ /android/build.gradle:                                                           │\n'
           "│     - classpath 'com.android.tools.build:gradle:<current-version>'               │\n"
-          "│     + classpath 'com.android.tools.build:gradle:7.1.2'                           │\n"
+          "│     + classpath 'com.android.tools.build:gradle:7.3.0'                           │\n"
           '│ /android/gradle/wrapper/gradle-wrapper.properties:                               │\n'
           '│     - https://services.gradle.org/distributions/gradle-<current-version>-all.zip │\n'
-          '│     + https://services.gradle.org/distributions/gradle-7.4-all.zip               │\n'
+          '│     + https://services.gradle.org/distributions/gradle-7.5-all.zip               │\n'
           '└──────────────────────────────────────────────────────────────────────────────────┘\n'
         )
       );
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -951,7 +1024,9 @@ Execution failed for task ':app:checkDebugAarMetadata'.
     testUsingContext('suggestion', () async {
       await minCompileSdkVersionHandler.handler(
         line: errorMessage,
-        project: FlutterProject.fromDirectoryTest(globals.fs.currentDirectory),
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        multidexEnabled: true,
       );
 
       expect(
@@ -970,8 +1045,8 @@ Execution failed for task ':app:checkDebugAarMetadata'.
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -993,7 +1068,12 @@ A problem occurred evaluating project ':flutter'.
     });
 
     testUsingContext('suggestion', () async {
-      await jvm11RequiredHandler.handler();
+      await jvm11RequiredHandler.handler(
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
+      );
 
       expect(
         testLogger.statusText,
@@ -1012,8 +1092,8 @@ A problem occurred evaluating project ':flutter'.
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -1068,7 +1148,12 @@ at java.base/sun.security.ssl.SSLTransport.decode(SSLTransport.java:108)'''
     });
 
     testUsingContext('suggestion', () async {
-      final GradleBuildStatus status = await sslExceptionHandler.handler();
+      final GradleBuildStatus status = await sslExceptionHandler.handler(
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
+      );
 
       expect(status, GradleBuildStatus.retry);
       expect(testLogger.errorText,
@@ -1079,8 +1164,8 @@ at java.base/sun.security.ssl.SSLTransport.decode(SSLTransport.java:108)'''
     }, overrides: <Type, Generator>{
       GradleUtils: () => FakeGradleUtils(),
       Platform: () => fakePlatform('android'),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
     });
   });
 
@@ -1107,12 +1192,17 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)'''
     });
 
     testUsingContext('suggestion', () async {
-      globals.fs.file('foo/.gradle/fizz.zip').createSync(recursive: true);
+      fileSystem.file('foo/.gradle/fizz.zip').createSync(recursive: true);
 
-      final GradleBuildStatus result = await zipExceptionHandler.handler();
+      final GradleBuildStatus result = await zipExceptionHandler.handler(
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
+      );
 
       expect(result, equals(GradleBuildStatus.retry));
-      expect(globals.fs.file('foo/.gradle/fizz.zip'), exists);
+      expect(fileSystem.file('foo/.gradle/fizz.zip'), exists);
       expect(
         testLogger.errorText,
         contains(
@@ -1122,18 +1212,23 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)'''
        expect(testLogger.statusText, '');
     }, overrides: <Type, Generator>{
       Platform: () => FakePlatform(environment: <String, String>{'HOME': 'foo/'}),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
       BotDetector: () => const FakeBotDetector(false),
     });
 
     testUsingContext('suggestion if running as bot', () async {
-      globals.fs.file('foo/.gradle/fizz.zip').createSync(recursive: true);
+      fileSystem.file('foo/.gradle/fizz.zip').createSync(recursive: true);
 
-      final GradleBuildStatus result = await zipExceptionHandler.handler();
+      final GradleBuildStatus result = await zipExceptionHandler.handler(
+        project: FakeFlutterProject(),
+        usesAndroidX: true,
+        line: '',
+        multidexEnabled: true,
+      );
 
       expect(result, equals(GradleBuildStatus.retry));
-      expect(globals.fs.file('foo/.gradle/fizz.zip'), isNot(exists));
+      expect(fileSystem.file('foo/.gradle/fizz.zip'), isNot(exists));
 
       expect(
         testLogger.errorText,
@@ -1147,18 +1242,23 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)'''
       );
     }, overrides: <Type, Generator>{
       Platform: () => FakePlatform(environment: <String, String>{'HOME': 'foo/'}),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
       BotDetector: () => const FakeBotDetector(true),
     });
 
     testUsingContext('suggestion if stdin has terminal and user entered y', () async {
-      globals.fs.file('foo/.gradle/fizz.zip').createSync(recursive: true);
+      fileSystem.file('foo/.gradle/fizz.zip').createSync(recursive: true);
 
-      final GradleBuildStatus result = await zipExceptionHandler.handler();
+      final GradleBuildStatus result = await zipExceptionHandler.handler(
+        line: '',
+        usesAndroidX: true,
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+      );
 
       expect(result, equals(GradleBuildStatus.retry));
-      expect(globals.fs.file('foo/.gradle/fizz.zip'), isNot(exists));
+      expect(fileSystem.file('foo/.gradle/fizz.zip'), isNot(exists));
       expect(
         testLogger.errorText,
         contains(
@@ -1171,19 +1271,24 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)'''
       );
     }, overrides: <Type, Generator>{
       Platform: () => FakePlatform(environment: <String, String>{'HOME': 'foo/'}),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
       AnsiTerminal: () => _TestPromptTerminal('y'),
       BotDetector: () => const FakeBotDetector(false),
     });
 
     testUsingContext('suggestion if stdin has terminal and user entered n', () async {
-      globals.fs.file('foo/.gradle/fizz.zip').createSync(recursive: true);
+      fileSystem.file('foo/.gradle/fizz.zip').createSync(recursive: true);
 
-      final GradleBuildStatus result = await zipExceptionHandler.handler();
+      final GradleBuildStatus result = await zipExceptionHandler.handler(
+        line: '',
+        usesAndroidX: true,
+        multidexEnabled: true,
+        project: FakeFlutterProject(),
+      );
 
       expect(result, equals(GradleBuildStatus.retry));
-      expect(globals.fs.file('foo/.gradle/fizz.zip'), exists);
+      expect(fileSystem.file('foo/.gradle/fizz.zip'), exists);
       expect(
         testLogger.errorText,
         contains(
@@ -1193,12 +1298,46 @@ at org.gradle.wrapper.GradleWrapperMain.main(GradleWrapperMain.java:61)'''
       expect(testLogger.statusText, '');
     }, overrides: <Type, Generator>{
       Platform: () => FakePlatform(environment: <String, String>{'HOME': 'foo/'}),
-      FileSystem: () => MemoryFileSystem.test(),
-      ProcessManager: () => FakeProcessManager.empty(),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
       AnsiTerminal: () => _TestPromptTerminal('n'),
       BotDetector: () => const FakeBotDetector(false),
     });
   });
+
+  group('incompatible java and gradle versions error', () {
+    const String errorMessage = '''
+Could not compile build file '…/example/android/build.gradle'.
+> startup failed:
+  General error during conversion: Unsupported class file major version 61
+  java.lang.IllegalArgumentException: Unsupported class file major version 61
+''';
+
+    testWithoutContext('pattern', () {
+      expect(
+        incompatibleJavaAndGradleVersionsHandler.test(errorMessage),
+        isTrue,
+      );
+    });
+
+    testUsingContext('suggestion', () async {
+      await incompatibleJavaAndGradleVersionsHandler.handler(
+        line: errorMessage,
+        project: FlutterProject.fromDirectoryTest(fileSystem.currentDirectory),
+        usesAndroidX: true,
+        multidexEnabled: true,
+      );
+
+      // Ensure the error notes the incompatible Gradle/AGP/Java versions and links to related resources.
+      expect(testLogger.statusText, contains('Gradle version is incompatible with the Java version'));
+      expect(testLogger.statusText, contains('https://docs.gradle.org/current/userguide/compatibility.html#java'));
+    }, overrides: <Type, Generator>{
+      GradleUtils: () => FakeGradleUtils(),
+      Platform: () => fakePlatform('android'),
+      FileSystem: () => fileSystem,
+      ProcessManager: () => processManager,
+    });
+  });
 }
 
 bool formatTestErrorMessage(String errorMessage, GradleHandledError error) {
@@ -1216,7 +1355,7 @@ Platform fakePlatform(String name) {
   );
 }
 
-class FakeGradleUtils extends GradleUtils {
+class FakeGradleUtils extends Fake implements GradleUtils {
   @override
   String getExecutable(FlutterProject project) {
     return 'gradlew';
@@ -1225,21 +1364,24 @@ class FakeGradleUtils extends GradleUtils {
 
 /// Simple terminal that returns the specified string when
 /// promptForCharInput is called.
-class _TestPromptTerminal extends AnsiTerminal {
+class _TestPromptTerminal extends Fake implements AnsiTerminal {
   _TestPromptTerminal(this.promptResult);
 
   final String promptResult;
 
   @override
-  Future<String> promptForCharInput(List<String> acceptedCharacters, {
-    Logger logger,
-    String prompt,
-    int defaultChoiceIndex,
+  bool get stdinHasTerminal => true;
+
+  @override
+  Future<String> promptForCharInput(
+    List<String> acceptedCharacters, {
+    required Logger logger,
+    String? prompt,
+    int? defaultChoiceIndex,
     bool displayAcceptedCharacters = true,
   }) {
     return Future<String>.value(promptResult);
   }
-
-  @override
-  bool get stdinHasTerminal => true;
 }
+
+class FakeFlutterProject extends Fake implements FlutterProject {}
