@@ -11,6 +11,7 @@ import 'package:flutter_tools/src/build_info.dart';
 import 'package:flutter_tools/src/build_system/build_system.dart';
 import 'package:flutter_tools/src/build_system/targets/macos.dart';
 import 'package:flutter_tools/src/convert.dart';
+import 'package:flutter_tools/src/reporting/reporting.dart';
 
 import '../../../src/common.dart';
 import '../../../src/context.dart';
@@ -27,12 +28,14 @@ void main() {
   late FakeCommand lipoInfoNonFatCommand;
   late FakeCommand lipoInfoFatCommand;
   late FakeCommand lipoVerifyX86_64Command;
+  late TestUsage usage;
 
   setUp(() {
     processManager = FakeProcessManager.empty();
     artifacts = Artifacts.test();
     fileSystem = MemoryFileSystem.test();
     logger = BufferLogger.test();
+    usage = TestUsage();
     environment = Environment.test(
       fileSystem.currentDirectory,
       defines: <String, String>{
@@ -45,7 +48,8 @@ void main() {
       processManager: processManager,
       logger: logger,
       fileSystem: fileSystem,
-      engineVersion: '2'
+      engineVersion: '2',
+      usage: usage,
     );
 
     binary = environment.outputDir
@@ -291,6 +295,28 @@ void main() {
     ProcessManager: () => processManager,
   });
 
+  testUsingContext('release macOS application creates App.framework.dSYM', () async {
+    fileSystem.file('bin/cache/artifacts/engine/darwin-x64/vm_isolate_snapshot.bin')
+      .createSync(recursive: true);
+    fileSystem.file('bin/cache/artifacts/engine/darwin-x64/isolate_snapshot.bin')
+      .createSync(recursive: true);
+    fileSystem.file('${environment.buildDir.path}/App.framework/App')
+      .createSync(recursive: true);
+    fileSystem.file('${environment.buildDir.path}/App.framework.dSYM/Contents/Resources/DWARF/App')
+      .createSync(recursive: true);
+
+    await const ReleaseMacOSBundleFlutterAssets()
+      .build(environment..defines[kBuildMode] = 'release');
+
+    expect(fileSystem.file(
+      'App.framework.dSYM/Contents/Resources/DWARF/App'),
+      exists,
+    );
+  }, overrides: <Type, Generator>{
+    FileSystem: () => fileSystem,
+    ProcessManager: () => processManager,
+  });
+
   testUsingContext('release/profile macOS application updates when App.framework updates', () async {
     fileSystem.file('bin/cache/artifacts/engine/darwin-x64/vm_isolate_snapshot.bin')
       .createSync(recursive: true);
@@ -314,6 +340,37 @@ void main() {
     ProcessManager: () => processManager,
   });
 
+  testUsingContext('ReleaseMacOSBundleFlutterAssets sends archive success event', () async {
+    environment.defines[kBuildMode] = 'release';
+    environment.defines[kXcodeAction] = 'install';
+
+    fileSystem.file('bin/cache/artifacts/engine/darwin-x64/vm_isolate_snapshot.bin')
+        .createSync(recursive: true);
+    fileSystem.file('bin/cache/artifacts/engine/darwin-x64/isolate_snapshot.bin')
+        .createSync(recursive: true);
+    fileSystem.file(fileSystem.path.join(environment.buildDir.path, 'App.framework', 'App'))
+        .createSync(recursive: true);
+
+    await const ReleaseMacOSBundleFlutterAssets().build(environment);
+    expect(usage.events, contains(const TestUsageEvent('assemble', 'macos-archive', label: 'success')));
+  }, overrides: <Type, Generator>{
+    FileSystem: () => fileSystem,
+    ProcessManager: () => processManager,
+  });
+
+  testUsingContext('ReleaseMacOSBundleFlutterAssets sends archive fail event', () async {
+    environment.defines[kBuildMode] = 'release';
+    environment.defines[kXcodeAction] = 'install';
+
+    // Throws because the project files are not set up.
+    await expectLater(() => const ReleaseMacOSBundleFlutterAssets().build(environment),
+        throwsA(const TypeMatcher<FileSystemException>()));
+    expect(usage.events, contains(const TestUsageEvent('assemble', 'macos-archive', label: 'fail')));
+  }, overrides: <Type, Generator>{
+    FileSystem: () => fileSystem,
+    ProcessManager: () => processManager,
+  });
+
   testUsingContext('DebugMacOSFramework creates expected binary with arm64 only arch', () async {
     environment.defines[kDarwinArchs] = 'arm64';
     processManager.addCommand(
@@ -328,6 +385,7 @@ void main() {
         '-dynamiclib',
         '-Xlinker', '-rpath', '-Xlinker', '@executable_path/Frameworks',
         '-Xlinker', '-rpath', '-Xlinker', '@loader_path/Frameworks',
+        '-fapplication-extension',
         '-install_name', '@rpath/App.framework/App',
         '-o',
         environment.buildDir
@@ -338,7 +396,7 @@ void main() {
     );
 
     await const DebugMacOSFramework().build(environment);
-    expect(processManager.hasRemainingExpectations, isFalse);
+    expect(processManager, hasNoRemainingExpectations);
   }, overrides: <Type, Generator>{
     FileSystem: () => fileSystem,
     ProcessManager: () => processManager,
@@ -360,6 +418,7 @@ void main() {
         '-dynamiclib',
         '-Xlinker', '-rpath', '-Xlinker', '@executable_path/Frameworks',
         '-Xlinker', '-rpath', '-Xlinker', '@loader_path/Frameworks',
+        '-fapplication-extension',
         '-install_name', '@rpath/App.framework/App',
         '-o',
         environment.buildDir
@@ -370,7 +429,7 @@ void main() {
     );
 
     await const DebugMacOSFramework().build(environment);
-    expect(processManager.hasRemainingExpectations, isFalse);
+    expect(processManager, hasNoRemainingExpectations);
   }, overrides: <Type, Generator>{
     FileSystem: () => fileSystem,
     ProcessManager: () => processManager,
@@ -380,13 +439,20 @@ void main() {
     environment.defines[kDarwinArchs] = 'arm64 x86_64';
     environment.defines[kBuildMode] = 'release';
 
+    // Input dSYMs need to exist for `lipo` to combine them
+    environment.buildDir
+      .childFile('arm64/App.framework.dSYM/Contents/Resources/DWARF/App')
+      .createSync(recursive: true);
+    environment.buildDir
+      .childFile('x86_64/App.framework.dSYM/Contents/Resources/DWARF/App')
+      .createSync(recursive: true);
+
     processManager.addCommands(<FakeCommand>[
       FakeCommand(command: <String>[
         'Artifact.genSnapshot.TargetPlatform.darwin.release_arm64',
         '--deterministic',
         '--snapshot_kind=app-aot-assembly',
         '--assembly=${environment.buildDir.childFile('arm64/snapshot_assembly.S').path}',
-        '--strip',
         environment.buildDir.childFile('app.dill').path,
       ]),
       FakeCommand(command: <String>[
@@ -394,7 +460,6 @@ void main() {
         '--deterministic',
         '--snapshot_kind=app-aot-assembly',
         '--assembly=${environment.buildDir.childFile('x86_64/snapshot_assembly.S').path}',
-        '--strip',
         environment.buildDir.childFile('app.dill').path,
       ]),
       FakeCommand(command: <String>[
@@ -411,6 +476,7 @@ void main() {
         'xcrun', 'clang', '-arch', 'arm64', '-dynamiclib', '-Xlinker', '-rpath',
         '-Xlinker', '@executable_path/Frameworks', '-Xlinker', '-rpath',
         '-Xlinker', '@loader_path/Frameworks',
+        '-fapplication-extension',
         '-install_name', '@rpath/App.framework/App',
         '-o', environment.buildDir.childFile('arm64/App.framework/App').path,
         environment.buildDir.childFile('arm64/snapshot_assembly.o').path,
@@ -419,10 +485,41 @@ void main() {
         'xcrun', 'clang', '-arch', 'x86_64', '-dynamiclib', '-Xlinker', '-rpath',
         '-Xlinker', '@executable_path/Frameworks', '-Xlinker', '-rpath',
         '-Xlinker', '@loader_path/Frameworks',
+        '-fapplication-extension',
         '-install_name', '@rpath/App.framework/App',
         '-o', environment.buildDir.childFile('x86_64/App.framework/App').path,
         environment.buildDir.childFile('x86_64/snapshot_assembly.o').path,
       ]),
+      FakeCommand(command: <String>[
+        'xcrun',
+        'dsymutil',
+        '-o',
+        environment.buildDir.childFile('arm64/App.framework.dSYM').path,
+        environment.buildDir.childFile('arm64/App.framework/App').path,
+      ]),
+      FakeCommand(command: <String>[
+        'xcrun',
+        'dsymutil',
+        '-o',
+        environment.buildDir.childFile('x86_64/App.framework.dSYM').path,
+        environment.buildDir.childFile('x86_64/App.framework/App').path,
+      ]),
+      FakeCommand(command: <String>[
+        'xcrun',
+        'strip',
+        '-x',
+        environment.buildDir.childFile('arm64/App.framework/App').path,
+        '-o',
+        environment.buildDir.childFile('arm64/App.framework/App').path,
+      ]),
+      FakeCommand(command: <String>[
+        'xcrun',
+        'strip',
+        '-x',
+        environment.buildDir.childFile('x86_64/App.framework/App').path,
+        '-o',
+        environment.buildDir.childFile('x86_64/App.framework/App').path,
+      ]),
       FakeCommand(command: <String>[
         'lipo',
         environment.buildDir.childFile('arm64/App.framework/App').path,
@@ -431,10 +528,18 @@ void main() {
         '-output',
         environment.buildDir.childFile('App.framework/App').path,
       ]),
+      FakeCommand(command: <String>[
+        'lipo',
+        environment.buildDir.childFile('arm64/App.framework.dSYM/Contents/Resources/DWARF/App').path,
+        environment.buildDir.childFile('x86_64/App.framework.dSYM/Contents/Resources/DWARF/App').path,
+        '-create',
+        '-output',
+        environment.buildDir.childFile('App.framework.dSYM/Contents/Resources/DWARF/App').path,
+      ]),
     ]);
 
     await const CompileMacOSFramework().build(environment);
-    expect(processManager.hasRemainingExpectations, isFalse);
+    expect(processManager, hasNoRemainingExpectations);
 
   }, overrides: <Type, Generator>{
     FileSystem: () => fileSystem,
