@@ -20,7 +20,7 @@ void main() {
       testConfig = Config.test();
       platform = FakePlatform(environment: <String, String>{});
 
-      for (final Feature feature in allFeatures) {
+      for (final Feature feature in allConfigurableFeatures) {
         testConfig.setValue(feature.configSetting!, false);
       }
 
@@ -81,6 +81,12 @@ void main() {
       expect(featureFlags.isWebEnabled, true);
     });
 
+    testWithoutContext('Flutter web wasm only enable on master', () {
+      expect(flutterWebWasm.getSettingForChannel('master').enabledByDefault, isTrue);
+      expect(flutterWebWasm.getSettingForChannel('beta').enabledByDefault, isFalse);
+      expect(flutterWebWasm.getSettingForChannel('stable').enabledByDefault, isFalse);
+    });
+
     testWithoutContext('Flutter web help string', () {
       expect(flutterWebFeature.generateHelpMessage(),
       'Enable or disable Flutter for web. '
