@@ -38,77 +38,20 @@ void main() {
 
   FlutterTesterTestDevice createDevice({
     List<String> dartEntrypointArgs = const <String>[],
-    bool enableObservatory = false,
+    bool enableVmService = false,
   }) =>
     TestFlutterTesterDevice(
       platform: platform,
       fileSystem: fileSystem,
       processManager: processManager,
-      enableObservatory: enableObservatory,
+      enableVmService: enableVmService,
       dartEntrypointArgs: dartEntrypointArgs,
+      uriConverter: (String input) => '$input/converted',
     );
 
-  testUsingContext('runs in Rosetta on arm64 Mac', () async {
-    final FakeProcessManager processManager = FakeProcessManager.empty();
-    final FlutterTesterTestDevice device = TestFlutterTesterDevice(
-      platform: FakePlatform(operatingSystem: 'macos'),
-      fileSystem: fileSystem,
-      processManager: processManager,
-      enableObservatory: false,
-      dartEntrypointArgs: const <String>[],
-    );
-    processManager.addCommands(<FakeCommand>[
-      const FakeCommand(
-        command: <String>[
-          'which',
-          'sysctl',
-        ],
-      ),
-      const FakeCommand(
-        command: <String>[
-          'sysctl',
-          'hw.optional.arm64',
-        ],
-        stdout: 'hw.optional.arm64: 1',
-      ),
-      FakeCommand(command: const <String>[
-        '/usr/bin/arch',
-        '-x86_64',
-        '/',
-        '--disable-observatory',
-        '--ipv6',
-        '--enable-checked-mode',
-        '--verify-entry-points',
-        '--enable-software-rendering',
-        '--skia-deterministic-rendering',
-        '--enable-dart-profiling',
-        '--non-interactive',
-        '--use-test-fonts',
-        '--disable-asset-fonts',
-        '--packages=.dart_tool/package_config.json',
-        'example.dill',
-      ], environment: <String, String>{
-        'FLUTTER_TEST': 'true',
-        'FONTCONFIG_FILE': device.fontConfigManager.fontConfigFile.path,
-        'SERVER_PORT': '0',
-        'APP_NAME': '',
-      }),
-    ]);
-    await device.start('example.dill');
-    expect(processManager.hasRemainingExpectations, isFalse);
-  });
-
   group('The FLUTTER_TEST environment variable is passed to the test process', () {
     setUp(() {
-      processManager = FakeProcessManager.list(<FakeCommand>[
-        const FakeCommand(
-          command: <String>[
-            'uname',
-            '-m',
-          ],
-          stdout: 'x86_64',
-        ),
-      ]);
+      processManager = FakeProcessManager.list(<FakeCommand>[]);
       device = createDevice();
 
       fileSystem
@@ -120,7 +63,7 @@ void main() {
     FakeCommand flutterTestCommand(String expectedFlutterTestValue) {
       return FakeCommand(command: const <String>[
         '/',
-        '--disable-observatory',
+        '--disable-vm-service',
         '--ipv6',
         '--enable-checked-mode',
         '--verify-entry-points',
@@ -144,7 +87,7 @@ void main() {
       processManager.addCommand(flutterTestCommand('true'));
 
       await device.start('example.dill');
-      expect(processManager.hasRemainingExpectations, isFalse);
+      expect(processManager, hasNoRemainingExpectations);
     });
 
     testUsingContext('as true when set to true', () async {
@@ -152,7 +95,7 @@ void main() {
       processManager.addCommand(flutterTestCommand('true'));
 
       await device.start('example.dill');
-      expect(processManager.hasRemainingExpectations, isFalse);
+      expect(processManager, hasNoRemainingExpectations);
     });
 
     testUsingContext('as false when set to false', () async {
@@ -160,7 +103,7 @@ void main() {
       processManager.addCommand(flutterTestCommand('false'));
 
       await device.start('example.dill');
-      expect(processManager.hasRemainingExpectations, isFalse);
+      expect(processManager, hasNoRemainingExpectations);
     });
 
     testUsingContext('unchanged when set', () async {
@@ -168,24 +111,17 @@ void main() {
       processManager.addCommand(flutterTestCommand('neither true nor false'));
 
       await device.start('example.dill');
-      expect(processManager.hasRemainingExpectations, isFalse);
+      expect(processManager, hasNoRemainingExpectations);
     });
   });
 
   group('Dart Entrypoint Args', () {
     setUp(() {
       processManager = FakeProcessManager.list(<FakeCommand>[
-        const FakeCommand(
-          command: <String>[
-            'uname',
-            '-m',
-          ],
-          stdout: 'x86_64',
-        ),
         const FakeCommand(
           command: <String>[
             '/',
-            '--disable-observatory',
+            '--disable-vm-service',
             '--ipv6',
             '--enable-checked-mode',
             '--verify-entry-points',
@@ -217,17 +153,10 @@ void main() {
   group('DDS', () {
     setUp(() {
       processManager = FakeProcessManager.list(<FakeCommand>[
-        const FakeCommand(
-          command: <String>[
-            'uname',
-            '-m',
-          ],
-          stdout: 'x86_64',
-        ),
         const FakeCommand(
           command: <String>[
             '/',
-            '--observatory-port=0',
+            '--vm-service-port=0',
             '--ipv6',
             '--enable-checked-mode',
             '--verify-entry-points',
@@ -244,16 +173,28 @@ void main() {
           stderr: 'failure',
         ),
       ]);
-      device = createDevice(enableObservatory: true);
+      device = createDevice(enableVmService: true);
     });
 
-    testUsingContext('skips setting observatory port and uses the input port for DDS instead', () async {
+    testUsingContext('skips setting VM Service port and uses the input port for DDS instead', () async {
       await device.start('example.dill');
-      await device.observatoryUri;
+      await device.vmServiceUri;
 
       final Uri uri = await (device as TestFlutterTesterDevice).ddsServiceUriFuture();
       expect(uri.port, 1234);
     });
+
+    testUsingContext('sets up UriConverter from context', () async {
+      await device.start('example.dill');
+      await device.vmServiceUri;
+
+      final FakeDartDevelopmentService dds = (device as TestFlutterTesterDevice).dds
+      as FakeDartDevelopmentService;
+      final String? result = dds
+          .uriConverter
+          ?.call('test');
+      expect(result, 'test/converted');
+    });
   });
 }
 
@@ -265,8 +206,9 @@ class TestFlutterTesterDevice extends FlutterTesterTestDevice {
     required super.platform,
     required super.fileSystem,
     required super.processManager,
-    required super.enableObservatory,
+    required super.enableVmService,
     required List<String> dartEntrypointArgs,
+    required UriConverter uriConverter,
   }) : super(
     id: 999,
     shellPath: '/',
@@ -287,16 +229,26 @@ class TestFlutterTesterDevice extends FlutterTesterTestDevice {
     icudtlPath: null,
     compileExpression: null,
     fontConfigManager: FontConfigManager(),
+    uriConverter: uriConverter,
   );
+  late DartDevelopmentService dds;
 
   final Completer<Uri> _ddsServiceUriCompleter = Completer<Uri>();
 
   Future<Uri> ddsServiceUriFuture() => _ddsServiceUriCompleter.future;
 
   @override
-  Future<DartDevelopmentService> startDds(Uri uri) async {
+  Future<DartDevelopmentService> startDds(
+    Uri uri, {
+    UriConverter? uriConverter,
+  }) async {
     _ddsServiceUriCompleter.complete(uri);
-    return FakeDartDevelopmentService(Uri.parse('http://localhost:${debuggingOptions.hostVmServicePort}'), Uri.parse('http://localhost:8080'));
+    dds = FakeDartDevelopmentService(
+      Uri.parse('http://localhost:${debuggingOptions.hostVmServicePort}'),
+      Uri.parse('http://localhost:8080'),
+      uriConverter: uriConverter,
+    );
+    return dds;
   }
 
   @override
@@ -307,9 +259,10 @@ class TestFlutterTesterDevice extends FlutterTesterTestDevice {
 }
 
 class FakeDartDevelopmentService extends Fake implements DartDevelopmentService {
-  FakeDartDevelopmentService(this.uri, this.original);
+  FakeDartDevelopmentService(this.uri, this.original, {this.uriConverter});
 
   final Uri original;
+  final UriConverter? uriConverter;
 
   @override
   final Uri uri;
