@@ -8,7 +8,6 @@ import 'dart:io' as io;
 import 'package:file/file.dart';
 import 'package:file/memory.dart';
 import 'package:flutter_tools/src/artifacts.dart';
-import 'package:flutter_tools/src/base/common.dart';
 import 'package:flutter_tools/src/base/file_system.dart';
 import 'package:flutter_tools/src/base/io.dart';
 import 'package:flutter_tools/src/base/logger.dart';
@@ -64,7 +63,7 @@ void main() {
     });
 
     testWithoutContext('successfully instantiates on Mac OS', () {
-      IOSDevice(
+      final IOSDevice device = IOSDevice(
         'device-123',
         iProxy: IProxy.test(logger: logger, processManager: FakeProcessManager.any()),
         fileSystem: fileSystem,
@@ -75,8 +74,27 @@ void main() {
         name: 'iPhone 1',
         sdkVersion: '13.3',
         cpuArchitecture: DarwinArch.arm64,
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       );
+      expect(device.isSupported(), isTrue);
+    });
+
+    testWithoutContext('32-bit devices are unsupported', () {
+      final IOSDevice device = IOSDevice(
+        'device-123',
+        iProxy: IProxy.test(logger: logger, processManager: FakeProcessManager.any()),
+        fileSystem: fileSystem,
+        logger: logger,
+        platform: macPlatform,
+        iosDeploy: iosDeploy,
+        iMobileDevice: iMobileDevice,
+        name: 'iPhone 1',
+        cpuArchitecture: DarwinArch.armv7,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
+      );
+      expect(device.isSupported(), isFalse);
     });
 
     testWithoutContext('parses major version', () {
@@ -91,7 +109,8 @@ void main() {
         name: 'iPhone 1',
         cpuArchitecture: DarwinArch.arm64,
         sdkVersion: '1.0.0',
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       ).majorSdkVersion, 1);
       expect(IOSDevice(
         'device-123',
@@ -104,7 +123,8 @@ void main() {
         name: 'iPhone 1',
         cpuArchitecture: DarwinArch.arm64,
         sdkVersion: '13.1.1',
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       ).majorSdkVersion, 13);
       expect(IOSDevice(
         'device-123',
@@ -117,7 +137,8 @@ void main() {
         name: 'iPhone 1',
         cpuArchitecture: DarwinArch.arm64,
         sdkVersion: '10',
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       ).majorSdkVersion, 10);
       expect(IOSDevice(
         'device-123',
@@ -130,7 +151,8 @@ void main() {
         name: 'iPhone 1',
         cpuArchitecture: DarwinArch.arm64,
         sdkVersion: '0',
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       ).majorSdkVersion, 0);
       expect(IOSDevice(
         'device-123',
@@ -143,7 +165,8 @@ void main() {
         name: 'iPhone 1',
         cpuArchitecture: DarwinArch.arm64,
         sdkVersion: 'bogus',
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       ).majorSdkVersion, 0);
     });
 
@@ -159,7 +182,8 @@ void main() {
         name: 'iPhone 1',
         sdkVersion: '13.3 17C54',
         cpuArchitecture: DarwinArch.arm64,
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       );
 
       expect(await device.sdkNameAndVersion,'iOS 13.3 17C54');
@@ -177,7 +201,8 @@ void main() {
         name: 'iPhone 1',
         sdkVersion: '13.3',
         cpuArchitecture: DarwinArch.arm64,
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       );
 
       expect(device.supportsRuntimeMode(BuildMode.debug), true);
@@ -201,7 +226,8 @@ void main() {
               name: 'iPhone 1',
               sdkVersion: '13.3',
               cpuArchitecture: DarwinArch.arm64,
-              interfaceType: IOSDeviceConnectionInterface.usb,
+              connectionInterface: DeviceConnectionInterface.attached,
+              isConnected: true,
             );
           },
           throwsAssertionError,
@@ -291,7 +317,8 @@ void main() {
           name: 'iPhone 1',
           sdkVersion: '13.3',
           cpuArchitecture: DarwinArch.arm64,
-          interfaceType: IOSDeviceConnectionInterface.usb,
+          connectionInterface: DeviceConnectionInterface.attached,
+          isConnected: true,
         );
         logReader1 = createLogReader(device, appPackage1, process1);
         logReader2 = createLogReader(device, appPackage2, process2);
@@ -352,7 +379,8 @@ void main() {
         logger: logger,
         platform: macPlatform,
         fileSystem: MemoryFileSystem.test(),
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       );
 
       device2 = IOSDevice(
@@ -366,7 +394,8 @@ void main() {
         logger: logger,
         platform: macPlatform,
         fileSystem: MemoryFileSystem.test(),
-        interfaceType: IOSDeviceConnectionInterface.usb,
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: true,
       );
     });
 
@@ -384,7 +413,7 @@ void main() {
     });
 
     testWithoutContext('start polling', () async {
-      final IOSDevices iosDevices = IOSDevices(
+      final TestIOSDevices iosDevices = TestIOSDevices(
         platform: macPlatform,
         xcdevice: xcdevice,
         iosWorkflow: iosWorkflow,
@@ -418,25 +447,68 @@ void main() {
       expect(iosDevices.deviceNotifier!.items, isEmpty);
       expect(xcdevice.deviceEventController.hasListener, isTrue);
 
-      xcdevice.deviceEventController.add(<XCDeviceEvent, String>{
-        XCDeviceEvent.attach: 'd83d5bc53967baa0ee18626ba87b6254b2ab5418'
-      });
+      xcdevice.deviceEventController.add(
+        XCDeviceEventNotification(
+          XCDeviceEvent.attach,
+          XCDeviceEventInterface.usb,
+          'd83d5bc53967baa0ee18626ba87b6254b2ab5418'
+        ),
+      );
       await added.future;
       expect(iosDevices.deviceNotifier!.items.length, 2);
       expect(iosDevices.deviceNotifier!.items, contains(device1));
       expect(iosDevices.deviceNotifier!.items, contains(device2));
+      expect(iosDevices.eventsReceived, 1);
 
-      xcdevice.deviceEventController.add(<XCDeviceEvent, String>{
-        XCDeviceEvent.detach: 'd83d5bc53967baa0ee18626ba87b6254b2ab5418'
-      });
+      iosDevices.resetEventCompleter();
+      xcdevice.deviceEventController.add(
+        XCDeviceEventNotification(
+          XCDeviceEvent.attach,
+          XCDeviceEventInterface.wifi,
+          'd83d5bc53967baa0ee18626ba87b6254b2ab5418'
+        ),
+      );
+      await iosDevices.receivedEvent.future;
+      expect(iosDevices.deviceNotifier!.items.length, 2);
+      expect(iosDevices.deviceNotifier!.items, contains(device1));
+      expect(iosDevices.deviceNotifier!.items, contains(device2));
+      expect(iosDevices.eventsReceived, 2);
+
+      iosDevices.resetEventCompleter();
+      xcdevice.deviceEventController.add(
+        XCDeviceEventNotification(
+          XCDeviceEvent.detach,
+          XCDeviceEventInterface.usb,
+          'd83d5bc53967baa0ee18626ba87b6254b2ab5418'
+        ),
+      );
+      await iosDevices.receivedEvent.future;
+      expect(iosDevices.deviceNotifier!.items.length, 2);
+      expect(iosDevices.deviceNotifier!.items, contains(device1));
+      expect(iosDevices.deviceNotifier!.items, contains(device2));
+      expect(iosDevices.eventsReceived, 3);
+
+      xcdevice.deviceEventController.add(
+        XCDeviceEventNotification(
+          XCDeviceEvent.detach,
+          XCDeviceEventInterface.wifi,
+          'd83d5bc53967baa0ee18626ba87b6254b2ab5418'
+        ),
+      );
       await removed.future;
       expect(iosDevices.deviceNotifier!.items, <Device>[device2]);
+      expect(iosDevices.eventsReceived, 4);
 
-      // Remove stream will throw over-completion if called more than once
-      // which proves this is ignored.
-      xcdevice.deviceEventController.add(<XCDeviceEvent, String>{
-        XCDeviceEvent.detach: 'bogus'
-      });
+      iosDevices.resetEventCompleter();
+      xcdevice.deviceEventController.add(
+        XCDeviceEventNotification(
+          XCDeviceEvent.detach,
+          XCDeviceEventInterface.usb,
+          'bogus'
+        ),
+      );
+      await iosDevices.receivedEvent.future;
+      expect(iosDevices.eventsReceived, 5);
 
       expect(addedCount, 2);
 
@@ -456,7 +528,7 @@ void main() {
       xcdevice.devices.add(<IOSDevice>[]);
       xcdevice.devices.add(<IOSDevice>[]);
 
-      final StreamController<Map<XCDeviceEvent, String>> rescheduledStream = StreamController<Map<XCDeviceEvent, String>>();
+      final StreamController<XCDeviceEventNotification> rescheduledStream = StreamController<XCDeviceEventNotification>();
 
       unawaited(xcdevice.deviceEventController.done.whenComplete(() {
         xcdevice.deviceEventController = rescheduledStream;
@@ -571,6 +643,120 @@ void main() {
       expect(diagnostics.first, 'Generic pairing error');
     });
   });
+
+  group('waitForDeviceToConnect', () {
+    late FakeXcdevice xcdevice;
+    late Cache cache;
+    late FakeProcessManager fakeProcessManager;
+    late BufferLogger logger;
+    late IOSDeploy iosDeploy;
+    late IMobileDevice iMobileDevice;
+    late IOSWorkflow iosWorkflow;
+    late IOSDevice notConnected1;
+
+    setUp(() {
+      xcdevice = FakeXcdevice();
+      final Artifacts artifacts = Artifacts.test();
+      cache = Cache.test(processManager: FakeProcessManager.any());
+      logger = BufferLogger.test();
+      iosWorkflow = FakeIOSWorkflow();
+      fakeProcessManager = FakeProcessManager.any();
+      iosDeploy = IOSDeploy(
+        artifacts: artifacts,
+        cache: cache,
+        logger: logger,
+        platform: macPlatform,
+        processManager: fakeProcessManager,
+      );
+      iMobileDevice = IMobileDevice(
+        artifacts: artifacts,
+        cache: cache,
+        processManager: fakeProcessManager,
+        logger: logger,
+      );
+      notConnected1 = IOSDevice(
+        '00000001-0000000000000000',
+        name: 'iPad',
+        sdkVersion: '13.3',
+        cpuArchitecture: DarwinArch.arm64,
+        iProxy: IProxy.test(logger: logger, processManager: FakeProcessManager.any()),
+        iosDeploy: iosDeploy,
+        iMobileDevice: iMobileDevice,
+        logger: logger,
+        platform: macPlatform,
+        fileSystem: MemoryFileSystem.test(),
+        connectionInterface: DeviceConnectionInterface.attached,
+        isConnected: false,
+      );
+    });
+
+    testWithoutContext('wait for device to connect via wifi', () async {
+      final IOSDevices iosDevices = IOSDevices(
+        platform: macPlatform,
+        xcdevice: xcdevice,
+        iosWorkflow: iosWorkflow,
+        logger: logger,
+      );
+      xcdevice.isInstalled = true;
+
+      xcdevice.waitForDeviceEvent = XCDeviceEventNotification(
+        XCDeviceEvent.attach,
+        XCDeviceEventInterface.wifi,
+        '00000001-0000000000000000'
+      );
+
+      final Device? device = await iosDevices.waitForDeviceToConnect(
+        notConnected1,
+        logger
+      );
+
+      expect(device?.isConnected, isTrue);
+      expect(device?.connectionInterface, DeviceConnectionInterface.wireless);
+    });
+
+    testWithoutContext('wait for device to connect via usb', () async {
+      final IOSDevices iosDevices = IOSDevices(
+        platform: macPlatform,
+        xcdevice: xcdevice,
+        iosWorkflow: iosWorkflow,
+        logger: logger,
+      );
+      xcdevice.isInstalled = true;
+
+      xcdevice.waitForDeviceEvent = XCDeviceEventNotification(
+        XCDeviceEvent.attach,
+        XCDeviceEventInterface.usb,
+        '00000001-0000000000000000'
+      );
+
+      final Device? device = await iosDevices.waitForDeviceToConnect(
+        notConnected1,
+        logger
+      );
+
+      expect(device?.isConnected, isTrue);
+      expect(device?.connectionInterface, DeviceConnectionInterface.attached);
+    });
+
+    testWithoutContext('wait for device returns null', () async {
+      final IOSDevices iosDevices = IOSDevices(
+        platform: macPlatform,
+        xcdevice: xcdevice,
+        iosWorkflow: iosWorkflow,
+        logger: logger,
+      );
+      xcdevice.isInstalled = true;
+
+      xcdevice.waitForDeviceEvent = null;
+
+      final Device? device = await iosDevices.waitForDeviceToConnect(
+        notConnected1,
+        logger
+      );
+
+      expect(device, isNull);
+    });
+  });
 }
 
 class FakeIOSApp extends Fake implements IOSApp {
@@ -580,13 +766,36 @@ class FakeIOSApp extends Fake implements IOSApp {
   final String name;
 }
 
+class TestIOSDevices extends IOSDevices {
+  TestIOSDevices({required super.platform, required super.xcdevice, required super.iosWorkflow, required super.logger,});
+
+  Completer<void> receivedEvent = Completer<void>();
+  int eventsReceived = 0;
+
+  void resetEventCompleter() {
+    receivedEvent = Completer<void>();
+  }
+
+  @override
+  Future<void> onDeviceEvent(XCDeviceEventNotification event) async {
+    await super.onDeviceEvent(event);
+    if (!receivedEvent.isCompleted) {
+      receivedEvent.complete();
+    }
+    eventsReceived++;
+    return;
+  }
+}
+
 class FakeIOSWorkflow extends Fake implements IOSWorkflow { }
 
 class FakeXcdevice extends Fake implements XCDevice {
   int getAvailableIOSDevicesCount = 0;
   final List<List<IOSDevice>> devices = <List<IOSDevice>>[];
   final List<String> diagnostics = <String>[];
-  StreamController<Map<XCDeviceEvent, String>> deviceEventController = StreamController<Map<XCDeviceEvent, String>>();
+  StreamController<XCDeviceEventNotification> deviceEventController = StreamController<XCDeviceEventNotification>();
+
+  XCDeviceEventNotification? waitForDeviceEvent;
 
   @override
   bool isInstalled = true;
@@ -597,7 +806,7 @@ class FakeXcdevice extends Fake implements XCDevice {
   }
 
   @override
-  Stream<Map<XCDeviceEvent, String>> observedDeviceEvents() {
+  Stream<XCDeviceEventNotification> observedDeviceEvents() {
     return deviceEventController.stream;
   }
 
@@ -605,6 +814,16 @@ class FakeXcdevice extends Fake implements XCDevice {
   Future<List<IOSDevice>> getAvailableIOSDevices({Duration? timeout}) async {
     return devices[getAvailableIOSDevicesCount++];
   }
+
+  @override
+  Future<XCDeviceEventNotification?> waitForDeviceToConnect(String deviceId) async {
+    final XCDeviceEventNotification? waitEvent = waitForDeviceEvent;
+    if (waitEvent != null) {
+      return XCDeviceEventNotification(waitEvent.eventType, waitEvent.eventInterface, waitEvent.deviceIdentifier);
+    } else {
+      return null;
+    }
+  }
 }
 
 class FakeProcess extends Fake implements Process {
