@@ -19,9 +19,11 @@ import 'package:flutter_tools/src/ios/devices.dart';
 import 'package:flutter_tools/src/ios/ios_deploy.dart';
 import 'package:flutter_tools/src/ios/iproxy.dart';
 import 'package:flutter_tools/src/ios/mac.dart';
+import 'package:flutter_tools/src/mdns_discovery.dart';
 import 'package:test/fake.dart';
 
 import '../../src/common.dart';
+import '../../src/context.dart';
 import '../../src/fake_devices.dart';
 import '../../src/fake_process_manager.dart';
 import '../../src/fakes.dart';
@@ -38,7 +40,7 @@ const FakeCommand kLaunchReleaseCommand = FakeCommand(
     '--justlaunch',
     // These args are the default on DebuggingOptions.
     '--args',
-    '--enable-dart-profiling --disable-service-auth-codes',
+    '--enable-dart-profiling',
   ],
   environment: <String, String>{
     'PATH': '/usr/bin:null',
@@ -56,7 +58,7 @@ const FakeCommand kLaunchDebugCommand = FakeCommand(command: <String>[
   '--no-wifi',
   '--justlaunch',
   '--args',
-  '--enable-dart-profiling --disable-service-auth-codes --enable-checked-mode --verify-entry-points',
+  '--enable-dart-profiling --enable-checked-mode --verify-entry-points',
 ], environment: <String, String>{
   'PATH': '/usr/bin:null',
   'DYLD_LIBRARY_PATH': '/path/to/libraries',
@@ -66,9 +68,10 @@ const FakeCommand kLaunchDebugCommand = FakeCommand(command: <String>[
 FakeCommand attachDebuggerCommand({
   IOSink? stdin,
   Completer<void>? completer,
+  bool isNetworkDevice = false,
 }) {
   return FakeCommand(
-    command: const <String>[
+    command: <String>[
       'script',
       '-t',
       '0',
@@ -79,9 +82,12 @@ FakeCommand attachDebuggerCommand({
       '--bundle',
       '/',
       '--debug',
-      '--no-wifi',
+      if (!isNetworkDevice) '--no-wifi',
       '--args',
-      '--enable-dart-profiling --disable-service-auth-codes --enable-checked-mode --verify-entry-points',
+      if (isNetworkDevice)
+        '--enable-dart-profiling --enable-checked-mode --verify-entry-points --vm-service-host=0.0.0.0'
+      else
+        '--enable-dart-profiling --enable-checked-mode --verify-entry-points',
     ],
     completer: completer,
     environment: const <String, String>{
@@ -146,7 +152,7 @@ void main() {
     );
 
     expect(launchResult.started, true);
-    expect(launchResult.hasObservatory, true);
+    expect(launchResult.hasVmService, true);
     expect(await device.stopApp(iosApp), false);
   });
 
@@ -184,11 +190,11 @@ void main() {
     );
 
     expect(launchResult.started, true);
-    expect(launchResult.hasObservatory, true);
+    expect(launchResult.hasVmService, true);
     expect(await device.stopApp(iosApp), false);
   });
 
-  testWithoutContext('IOSDevice.startApp prints warning message if discovery takes longer than configured timeout', () async {
+  testWithoutContext('IOSDevice.startApp prints warning message if discovery takes longer than configured timeout for wired device', () async {
     final FileSystem fileSystem = MemoryFileSystem.test();
     final BufferLogger logger = BufferLogger.test();
     final CompleterIOSink stdin = CompleterIOSink();
@@ -224,14 +230,61 @@ void main() {
     );
 
     expect(launchResult.started, true);
-    expect(launchResult.hasObservatory, true);
+    expect(launchResult.hasVmService, true);
     expect(await device.stopApp(iosApp), false);
-    expect(logger.errorText, contains('iOS Observatory not discovered after 30 seconds. This is taking much longer than expected...'));
+    expect(logger.errorText, contains('The Dart VM Service was not discovered after 30 seconds. This is taking much longer than expected...'));
     expect(utf8.decoder.convert(stdin.writes.first), contains('process interrupt'));
     completer.complete();
     expect(processManager, hasNoRemainingExpectations);
   });
 
+  testUsingContext('IOSDevice.startApp prints warning message if discovery takes longer than configured timeout for wireless device', () async {
+    final FileSystem fileSystem = MemoryFileSystem.test();
+    final BufferLogger logger = BufferLogger.test();
+    final CompleterIOSink stdin = CompleterIOSink();
+    final Completer<void> completer = Completer<void>();
+    final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
+      attachDebuggerCommand(stdin: stdin, completer: completer, isNetworkDevice: true),
+    ]);
+    final IOSDevice device = setUpIOSDevice(
+      processManager: processManager,
+      fileSystem: fileSystem,
+      logger: logger,
+      interfaceType: DeviceConnectionInterface.wireless,
+    );
+    final IOSApp iosApp = PrebuiltIOSApp(
+      projectBundleId: 'app',
+      bundleName: 'Runner',
+      uncompressedBundle: fileSystem.currentDirectory,
+      applicationPackage: fileSystem.currentDirectory,
+    );
+    final FakeDeviceLogReader deviceLogReader = FakeDeviceLogReader();
+
+    device.portForwarder = const NoOpDevicePortForwarder();
+    device.setLogReader(iosApp, deviceLogReader);
+
+    // Start writing messages to the log reader.
+    deviceLogReader.addLine('Foo');
+    deviceLogReader.addLine('The Dart VM service is listening on http://127.0.0.1:456');
+
+    final LaunchResult launchResult = await device.startApp(iosApp,
+      prebuiltApplication: true,
+      debuggingOptions: DebuggingOptions.enabled(BuildInfo.debug),
+      platformArgs: <String, dynamic>{},
+      discoveryTimeout: Duration.zero,
+    );
+
+    expect(launchResult.started, true);
+    expect(launchResult.hasVmService, true);
+    expect(await device.stopApp(iosApp), false);
+    expect(logger.errorText, contains('The Dart VM Service was not discovered after 45 seconds. This is taking much longer than expected...'));
+    expect(logger.errorText, contains('Click "Allow" to the prompt asking if you would like to find and connect devices on your local network.'));
+    completer.complete();
+    expect(processManager, hasNoRemainingExpectations);
+  }, overrides: <Type, Generator>{
+    MDnsVmServiceDiscovery: () => FakeMDnsVmServiceDiscovery(),
+  });
+
   testWithoutContext('IOSDevice.startApp succeeds in release mode', () async {
     final FileSystem fileSystem = MemoryFileSystem.test();
     final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
@@ -255,7 +308,7 @@ void main() {
     );
 
     expect(launchResult.started, true);
-    expect(launchResult.hasObservatory, false);
+    expect(launchResult.hasVmService, false);
     expect(await device.stopApp(iosApp), false);
     expect(processManager, hasNoRemainingExpectations);
   });
@@ -282,21 +335,25 @@ void main() {
           <String>[
             '--enable-dart-profiling',
             '--disable-service-auth-codes',
-            '--disable-observatory-publication',
+            '--disable-vm-service-publication',
             '--start-paused',
             '--dart-flags="--foo,--null_assertions"',
+            '--use-test-fonts',
             '--enable-checked-mode',
             '--verify-entry-points',
             '--enable-software-rendering',
             '--trace-systrace',
             '--skia-deterministic-rendering',
             '--trace-skia',
+            '--trace-allowlist="foo"',
+            '--trace-skia-allowlist="skia.a,skia.b"',
             '--endless-trace-buffer',
             '--dump-skp-on-shader-compilation',
             '--verbose-logging',
             '--cache-sksl',
             '--purge-persistent-cache',
-            '--enable-impeller',
+            '--enable-impeller=false',
+            '--enable-embedder-api',
           ].join(' '),
         ],
         environment: const <String, String>{
@@ -335,17 +392,21 @@ void main() {
         disableServiceAuthCodes: true,
         disablePortPublication: true,
         dartFlags: '--foo',
+        useTestFonts: true,
         enableSoftwareRendering: true,
         skiaDeterministicRendering: true,
         traceSkia: true,
+        traceAllowlist: 'foo',
+        traceSkiaAllowlist: 'skia.a,skia.b',
         traceSystrace: true,
         endlessTraceBuffer: true,
         dumpSkpOnShaderCompilation: true,
         cacheSkSL: true,
         purgePersistentCache: true,
         verboseSystemLogs: true,
+        enableImpeller: ImpellerStatus.disabled,
         nullAssertions: true,
-        enableImpeller: true,
+        enableEmbedderApi: true,
       ),
       platformArgs: <String, dynamic>{},
     );
@@ -354,6 +415,143 @@ void main() {
     expect(await device.stopApp(iosApp), false);
     expect(processManager, hasNoRemainingExpectations);
   });
+
+  testWithoutContext('startApp using route', () async {
+    final FileSystem fileSystem = MemoryFileSystem.test();
+    final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
+      FakeCommand(
+        command: <String>[
+          'script',
+          '-t',
+          '0',
+          '/dev/null',
+          'HostArtifact.iosDeploy',
+          '--id',
+          '123',
+          '--bundle',
+          '/',
+          '--debug',
+          '--no-wifi',
+          '--args',
+          <String>[
+            '--enable-dart-profiling',
+            '--enable-checked-mode',
+            '--verify-entry-points',
+            // The --route argument below is determined by what is passed into
+            // route argument to startApp.
+            '--route=/animation',
+          ].join(' '),
+        ],
+        environment: const <String, String>{
+          'PATH': '/usr/bin:null',
+          'DYLD_LIBRARY_PATH': '/path/to/libraries',
+        },
+        stdout: '(lldb)     run\nsuccess',
+      ),
+    ]);
+    final IOSDevice device = setUpIOSDevice(
+      sdkVersion: '13.3',
+      processManager: processManager,
+      fileSystem: fileSystem,
+    );
+    final IOSApp iosApp = PrebuiltIOSApp(
+      projectBundleId: 'app',
+      bundleName: 'Runner',
+      uncompressedBundle: fileSystem.currentDirectory,
+      applicationPackage: fileSystem.currentDirectory,
+    );
+    final FakeDeviceLogReader deviceLogReader = FakeDeviceLogReader();
+
+    device.portForwarder = const NoOpDevicePortForwarder();
+    device.setLogReader(iosApp, deviceLogReader);
+
+    // Start writing messages to the log reader.
+    Timer.run(() {
+      deviceLogReader.addLine('The Dart VM service is listening on http://127.0.0.1:1234');
+    });
+
+    final LaunchResult launchResult = await device.startApp(iosApp,
+      prebuiltApplication: true,
+      debuggingOptions: DebuggingOptions.enabled(
+        BuildInfo.debug,
+      ),
+      platformArgs: <String, dynamic>{},
+      route: '/animation',
+    );
+
+    expect(launchResult.started, true);
+    expect(await device.stopApp(iosApp), false);
+    expect(processManager, hasNoRemainingExpectations);
+  });
+
+  testWithoutContext('startApp using trace-startup', () async {
+    final FileSystem fileSystem = MemoryFileSystem.test();
+    final FakeProcessManager processManager = FakeProcessManager.list(<FakeCommand>[
+      FakeCommand(
+        command: <String>[
+          'script',
+          '-t',
+          '0',
+          '/dev/null',
+          'HostArtifact.iosDeploy',
+          '--id',
+          '123',
+          '--bundle',
+          '/',
+          '--debug',
+          '--no-wifi',
+          '--args',
+          <String>[
+            '--enable-dart-profiling',
+            '--enable-checked-mode',
+            '--verify-entry-points',
+            // The --trace-startup argument below is determined by what is passed into
+            // platformArgs argument to startApp.
+            '--trace-startup',
+          ].join(' '),
+        ],
+        environment: const <String, String>{
+          'PATH': '/usr/bin:null',
+          'DYLD_LIBRARY_PATH': '/path/to/libraries',
+        },
+        stdout: '(lldb)     run\nsuccess',
+      ),
+    ]);
+    final IOSDevice device = setUpIOSDevice(
+      sdkVersion: '13.3',
+      processManager: processManager,
+      fileSystem: fileSystem,
+    );
+    final IOSApp iosApp = PrebuiltIOSApp(
+      projectBundleId: 'app',
+      bundleName: 'Runner',
+      uncompressedBundle: fileSystem.currentDirectory,
+      applicationPackage: fileSystem.currentDirectory,
+    );
+    final FakeDeviceLogReader deviceLogReader = FakeDeviceLogReader();
+
+    device.portForwarder = const NoOpDevicePortForwarder();
+    device.setLogReader(iosApp, deviceLogReader);
+
+    // Start writing messages to the log reader.
+    Timer.run(() {
+      deviceLogReader.addLine('The Dart VM service is listening on http://127.0.0.1:1234');
+    });
+
+    final LaunchResult launchResult = await device.startApp(iosApp,
+      prebuiltApplication: true,
+      debuggingOptions: DebuggingOptions.enabled(
+        BuildInfo.debug,
+      ),
+      platformArgs: <String, dynamic>{
+        'trace-startup': true,
+      },
+    );
+
+    expect(launchResult.started, true);
+    expect(await device.stopApp(iosApp), false);
+    expect(processManager, hasNoRemainingExpectations);
+  });
 }
 
 IOSDevice setUpIOSDevice({
@@ -362,6 +560,7 @@ IOSDevice setUpIOSDevice({
   Logger? logger,
   ProcessManager? processManager,
   IOSDeploy? iosDeploy,
+  DeviceConnectionInterface interfaceType = DeviceConnectionInterface.attached,
 }) {
   final Artifacts artifacts = Artifacts.test();
   final FakePlatform macPlatform = FakePlatform(
@@ -399,7 +598,8 @@ IOSDevice setUpIOSDevice({
       cache: cache,
     ),
     cpuArchitecture: DarwinArch.arm64,
-    interfaceType: IOSDeviceConnectionInterface.usb,
+    connectionInterface: interfaceType,
+    isConnected: true,
   );
 }
 
@@ -411,3 +611,18 @@ class FakeDevicePortForwarder extends Fake implements DevicePortForwarder {
     disposed = true;
   }
 }
+
+class FakeMDnsVmServiceDiscovery extends Fake implements MDnsVmServiceDiscovery {
+  @override
+  Future<Uri?> getVMServiceUriForLaunch(
+    String applicationId,
+    Device device, {
+    bool usesIpv6 = false,
+    int? hostVmservicePort,
+    required int deviceVmservicePort,
+    bool isNetworkDevice = false,
+    Duration timeout = Duration.zero,
+  }) async {
+    return Uri.tryParse('http://0.0.0.0:1234');
+  }
+}
